@@ -44,6 +44,7 @@
 #include <KGlobalSettings>
 #include <KPushButton>
 #include <KTitleWidget>
+#include <KWindowSystem>
 
 #include <Plasma/AbstractRunner>
 #include <Plasma/RunnerManager>
@@ -56,6 +57,7 @@
 #include "interfaces/default/resultitem.h"
 #include "interfaces/default/krunnertabfilter.h"
 #include "interfaces/default/resultsview.h"
+#include "toolbutton.h"
 
 static const int MIN_WIDTH = 420;
 
@@ -65,37 +67,27 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
       m_running(false),
       m_queryRunning(false)
 {
+    m_resultData.processHoverEvents = true;
+    m_resultData.mouseHovering = false;
+
     m_hideResultsTimer.setSingleShot(true);
     connect(&m_hideResultsTimer, SIGNAL(timeout()), this, SLOT(hideResultsArea()));
 
-    QWidget *w = mainWidget();
-    m_layout = new QVBoxLayout(w);
+    m_layout = new QVBoxLayout(this);
     m_layout->setMargin(0);
 
-    m_buttonContainer = new QWidget(w);
+    m_buttonContainer = new QWidget(this);
     QHBoxLayout *bottomLayout = new QHBoxLayout(m_buttonContainer);
     bottomLayout->setMargin(0);
 
-    m_configButton = new QToolButton(m_buttonContainer);
+    m_configButton = new ToolButton(m_buttonContainer);
     m_configButton->setText(i18n("Settings"));
     m_configButton->setToolTip(i18n("Settings"));
-    m_configButton->setIcon(m_iconSvg->pixmap("configure"));
-    connect(m_configButton, SIGNAL(clicked()), SLOT(showConfigDialog()));
-    bottomLayout->addWidget( m_configButton );
-
-    /*
-    KPushButton *m_optionsButton = new KPushButton(KStandardGuiItem::configure(), m_buttonContainer);
-    m_optionsButton->setDefault(false);
-    m_optionsButton->setAutoDefault(false);
-    m_optionsButton->setText(i18n("Show Options"));
-    m_optionsButton->setEnabled(false);
-    m_optionsButton->setCheckable(true);
-    connect(m_optionsButton, SIGNAL(toggled(bool)), SLOT(showOptions(bool)));
-    bottomLayout->addWidget( m_optionsButton );
-    */
+    connect(m_configButton, SIGNAL(clicked()), SLOT(toggleConfigDialog()));
+    bottomLayout->addWidget(m_configButton);
 
     //Set up the system activity button, using the krunner global action, showing the global shortcut in the tooltip
-    m_activityButton = new QToolButton(m_buttonContainer);
+    m_activityButton = new ToolButton(m_buttonContainer);
     KRunnerApp *krunnerApp = KRunnerApp::self();
     QAction *showSystemActivityAction = krunnerApp->actionCollection()->action("Show System Activity");
     m_activityButton->setDefaultAction(showSystemActivityAction);
@@ -104,17 +96,15 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     connect(showSystemActivityAction, SIGNAL(globalShortcutChanged(const QKeySequence &)), this, SLOT(updateSystemActivityToolTip()));
     connect(showSystemActivityAction, SIGNAL(triggered(bool)), this, SLOT(resetAndClose()));
     bottomLayout->addWidget(m_activityButton);
-    //bottomLayout->addStretch(10);
 
     m_singleRunnerIcon = new QLabel();
     bottomLayout->addWidget(m_singleRunnerIcon);
     m_singleRunnerDisplayName = new QLabel();
     bottomLayout->addWidget(m_singleRunnerDisplayName);
 
-    m_helpButton = new QToolButton(m_buttonContainer);
+    m_helpButton = new ToolButton(m_buttonContainer);
     m_helpButton->setText(i18n("Help"));
     m_helpButton->setToolTip(i18n("Information on using this application"));
-    m_helpButton->setIcon(m_iconSvg->pixmap("help"));
     connect(m_helpButton, SIGNAL(clicked(bool)), SLOT(showHelp()));
     connect(m_helpButton, SIGNAL(clicked(bool)), SLOT(configCompleted()));
     bottomLayout->addWidget(m_helpButton);
@@ -122,19 +112,16 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     QSpacerItem* closeButtonSpacer = new QSpacerItem(0,0,QSizePolicy::MinimumExpanding,QSizePolicy::Fixed);
     bottomLayout->addSpacerItem(closeButtonSpacer);
 
-    m_closeButton = new QToolButton(m_buttonContainer);
+    m_closeButton = new ToolButton(m_buttonContainer);
     KGuiItem guiItem = KStandardGuiItem::close();
     m_closeButton->setText(guiItem.text());
     m_closeButton->setToolTip(guiItem.text().remove('&'));
-    m_closeButton->setIcon(m_iconSvg->pixmap("close"));
-//    m_closeButton->setDefault(false);
-//    m_closeButton->setAutoDefault(false);
     connect(m_closeButton, SIGNAL(clicked(bool)), SLOT(resetAndClose()));
     bottomLayout->addWidget(m_closeButton);
 
     m_layout->addWidget(m_buttonContainer);
 
-    m_searchTerm = new KHistoryComboBox(false, w);
+    m_searchTerm = new KHistoryComboBox(false, this);
     m_searchTerm->setPalette(QApplication::palette());
     m_searchTerm->setDuplicatesEnabled(false);
 
@@ -142,11 +129,11 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     QAction *focusEdit = new QAction(this);
     focusEdit->setShortcut(Qt::Key_F6);
 
-    // in therory, the widget should detect the direction from the content
+    // in theory, the widget should detect the direction from the content
     // but this is not available in Qt4.4/KDE 4.2, so the best default for this widget
     // is LTR: as it's more or less a "command line interface"
     // FIXME remove this code when KLineEdit has automatic direction detection of the "paragraph"
-    m_searchTerm->setLayoutDirection( Qt::LeftToRight );
+    m_searchTerm->setLayoutDirection(Qt::LeftToRight);
 
     connect(focusEdit, SIGNAL(triggered(bool)), this, SLOT(searchTermSetFocus()));
     addAction(focusEdit);
@@ -166,41 +153,28 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     m_completion->insertItems(pastQueryItems);
     bottomLayout->insertWidget(4, m_searchTerm, 10);
 
-    m_singleRunnerSearchTerm = new KLineEdit(w);
+    m_singleRunnerSearchTerm = new KLineEdit(this);
     bottomLayout->insertWidget(4, m_singleRunnerSearchTerm, 10 );
 
-    m_resultsContainer = new QWidget(w);
-    QVBoxLayout* resultsLayout = new QVBoxLayout(m_resultsContainer);
-    resultsLayout->setMargin(0);
-
-    m_dividerLine = new QWidget(m_resultsContainer);
-    m_dividerLine->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Minimum);
-    m_dividerLine->setFixedHeight(1);
-    m_dividerLine->setAutoFillBackground(true);
-    resultsLayout->addWidget(m_dividerLine);
-
-    m_resultsView = new ResultsView(m_resultsContainer);
-
     //kDebug() << "size:" << m_resultsView->size() << m_resultsView->minimumSize();
-    m_resultsScene = new ResultScene(runnerManager, m_searchTerm, this);
-    m_resultsView->setScene(m_resultsScene);
-    m_resultsView->setMinimumSize(m_resultsScene->minimumSizeHint());
+    m_resultsScene = new ResultScene(&m_resultData, runnerManager, m_searchTerm, this);
+    m_resultsView = new ResultsView(m_resultsScene, &m_resultData, this);
+    m_layout->addWidget(m_resultsView);
 
     connect(m_resultsScene, SIGNAL(matchCountChanged(int)), this, SLOT(matchCountChanged(int)));
     connect(m_resultsScene, SIGNAL(itemActivated(ResultItem *)), this, SLOT(run(ResultItem *)));
-    connect(m_resultsScene, SIGNAL(ensureVisibility(QGraphicsItem *)), this, SLOT(ensureVisibility(QGraphicsItem *)));
 
-    resultsLayout->addWidget(m_resultsView);
-
-    m_layout->addWidget(m_resultsContainer);
-
-    connect(lineEdit, SIGNAL(userTextChanged(QString)), this, SLOT(queryTextEdited(QString)));
+    connect(lineEdit, SIGNAL(textEdited(QString)), this, SLOT(queryTextEdited(QString)));
+    connect(m_searchTerm, SIGNAL(currentIndexChanged(QString)), this, SLOT(queryTextEdited(QString)));
     connect(m_searchTerm, SIGNAL(returnPressed()), this, SLOT(runDefaultResultItem()));
     connect(m_singleRunnerSearchTerm, SIGNAL(textChanged(QString)), this, SLOT(queryTextEdited(QString)));
     connect(m_singleRunnerSearchTerm, SIGNAL(returnPressed()),  this, SLOT(runDefaultResultItem()));
 
     KrunnerTabFilter *krunnerTabFilter = new KrunnerTabFilter(m_resultsScene, lineEdit, this);
     m_searchTerm->installEventFilter(krunnerTabFilter);
+    m_singleRunnerSearchTerm->installEventFilter(krunnerTabFilter);
+    connect(krunnerTabFilter, SIGNAL(historyRotated(QString)), this, SLOT(queryTextEdited(QString)));
+    lineEdit->installEventFilter(this);
 
     themeUpdated();
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeUpdated()));
@@ -212,7 +186,8 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     setTabOrder(0, m_configButton);
     setTabOrder(m_configButton, m_activityButton);
     setTabOrder(m_activityButton, m_searchTerm);
-    setTabOrder(m_searchTerm, m_resultsView);
+    setTabOrder(m_searchTerm, m_singleRunnerSearchTerm);
+    setTabOrder(m_singleRunnerSearchTerm, m_resultsView);
     setTabOrder(m_resultsView, m_helpButton);
     setTabOrder(m_helpButton, m_closeButton);
 
@@ -222,18 +197,20 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
     // normal size, then we hide the results view and resize the dialog
 
     setMinimumSize(QSize(MIN_WIDTH , 0));
-    adjustSize();
 
     // we load the last used size; the saved value is the size of the dialog when the
     // results are visible;
+    adjustSize();
 
     if (KGlobal::config()->hasGroup("Interface")) {
         KConfigGroup interfaceConfig(KGlobal::config(), "Interface");
         restoreDialogSize(interfaceConfig);
+        m_defaultSize = size();
+    } else {
+        m_defaultSize = QSize(size().width(), 500);
     }
 
-    m_defaultSize = size();
-    m_resultsContainer->hide();
+    m_resultsView->hide();
 
     m_delayedQueryTimer.setSingleShot(true);
     m_delayedQueryTimer.setInterval(100);
@@ -241,26 +218,60 @@ Interface::Interface(Plasma::RunnerManager *runnerManager, QWidget *parent)
 
     QTimer::singleShot(0, this, SLOT(resetInterface()));
 }
+
+bool Interface::eventFilter(QObject *obj, QEvent *event)
+{
+    if (event->type() == QEvent::MouseButtonPress) {
+        if (KWindowSystem::activeWindow() != winId()) {
+            // this overcomes problems with click-to-focus and being a Dock window
+            KWindowSystem::forceActiveWindow(winId());
+            searchTermSetFocus();
+        }
+    }
+
+    return KRunnerDialog::eventFilter(obj, event);
+}
+
+void Interface::saveDialogSize(KConfigGroup &group)
+{
+    group.writeEntry("Size", size());
+}
+
+void Interface::restoreDialogSize(KConfigGroup &group)
+{
+    resize(group.readEntry("Size", size()));
+}
+
 void Interface::updateSystemActivityToolTip()
 {
     /* Set the tooltip for the Show System Activity button to include the global shortcut */
     KRunnerApp *krunnerApp = KRunnerApp::self();
     KAction *showSystemActivityAction = dynamic_cast<KAction *>(krunnerApp->actionCollection()->action("Show System Activity"));
-    if(!showSystemActivityAction)
-        return;
-
-    QString shortcut = showSystemActivityAction->globalShortcut().toString();
-    if(!shortcut.isEmpty())
-        m_activityButton->setToolTip( i18nc("tooltip, shortcut", "%1 (%2)", showSystemActivityAction->toolTip(), shortcut));
-    else
-        m_activityButton->setToolTip( showSystemActivityAction->toolTip() );
+    if (showSystemActivityAction) {
+        QString shortcut = showSystemActivityAction->globalShortcut().toString();
+        if (shortcut.isEmpty()) {
+            m_activityButton->setToolTip( showSystemActivityAction->toolTip() );
+        } else {
+            m_activityButton->setToolTip( i18nc("tooltip, shortcut", "%1 (%2)", showSystemActivityAction->toolTip(), shortcut));
+        }
+    }
 }
+
 void Interface::setConfigWidget(QWidget *w)
 {
-    m_resultsContainer->hide();
+    //FIXME: would like to use kephal here, but it doesn't provide an availableGeometry call
+    const int screenId = qApp->desktop()->screenNumber(this); //Kephal::ScreenUtils::screenId(geometry().center());
+    const int maxHeight = qApp->desktop()->availableGeometry(screenId).height(); //Kephal::Screens::self()->screen(screenId)->geometry().height();
+
+    int left, top, right, bottom;
+    getContentsMargins(&left, &top, &right, &bottom);
+    const int padding = top + bottom + m_activityButton->height();
+    resize(width(), qMin(maxHeight, qMax(w->sizeHint().height() + padding, m_defaultSize.height())));
+
+    m_resultsView->hide();
     m_searchTerm->setEnabled(false);
     m_layout->addWidget(w);
-    resize(m_defaultSize);
+
     connect(w, SIGNAL(destroyed(QObject*)), this, SLOT(configWidgetDestroyed()));
 }
 
@@ -273,36 +284,19 @@ void Interface::cleanupAfterConfigWidget()
 {
     m_searchTerm->setEnabled(true);
     resetInterface();
-    m_searchTerm->setFocus();
+    searchTermSetFocus();
 }
 
 void Interface::resizeEvent(QResizeEvent *event)
 {
-    // We set m_defaultSize only when the event is spontaneous, i.e. when the user resizes the window
-    // We always update the width, but we update the height only if the resultsContainer is visible.
-
-    if (event->spontaneous() || isManualResizing()) {
-        m_defaultSize.setWidth(width());
-        if (m_resultsContainer->isVisible()) {
-            m_defaultSize.setHeight(height());
-        }
+    // We set m_defaultSize only when the event is spontaneous, i.e. when the user resizes the
+    // window, or if they are manually resizing it
+    if ((freeFloating() && event->spontaneous()) || isManualResizing()) {
+        m_defaultSize = size();
     }
 
-    Plasma::Theme *theme = Plasma::Theme::defaultTheme();
-    int gradientWidth = contentsRect().width() - KDialog::marginHint()*2;
-    QLinearGradient gr(0, 0, gradientWidth, 0);
-    gr.setColorAt(0, Qt::transparent);
-    gr.setColorAt(.35, theme->color(Plasma::Theme::TextColor));
-    gr.setColorAt(.65, theme->color(Plasma::Theme::TextColor));
-    gr.setColorAt(1, Qt::transparent);
-    {
-        QPalette p = palette();
-        p.setBrush(QPalette::Background, gr);
-        m_dividerLine->setPalette(p);
-    }
-
-    m_resultsScene->resize(m_resultsView->width(), qMax(m_resultsView->height(), int(m_resultsScene->height())));
-
+    m_resultsView->resize(m_buttonContainer->width(), m_resultsView->height());
+    m_resultsScene->setWidth(m_resultsView->width());
     KRunnerDialog::resizeEvent(event);
 }
 
@@ -332,26 +326,8 @@ void Interface::searchTermSetFocus()
 
 void Interface::themeUpdated()
 {
-    Plasma::Theme *theme = Plasma::Theme::defaultTheme();
-    QColor buttonBgColor = theme->color(Plasma::Theme::BackgroundColor);
-    QString buttonStyleSheet = QString("QToolButton { border: 1px solid %4; border-radius: 4px; padding: 2px;"
-                                       " background-color: rgba(%1, %2, %3, %5); }")
-                                      .arg(buttonBgColor.red())
-                                      .arg(buttonBgColor.green())
-                                      .arg(buttonBgColor.blue())
-                                      .arg(theme->color(Plasma::Theme::BackgroundColor).name(), "50%");
-    buttonBgColor = theme->color(Plasma::Theme::TextColor);
-    buttonStyleSheet += QString("QToolButton:hover { border: 2px solid %1; }")
-                               .arg(theme->color(Plasma::Theme::HighlightColor).name());
-    buttonStyleSheet += QString("QToolButton:focus { border: 2px solid %1; }")
-                               .arg(theme->color(Plasma::Theme::HighlightColor).name());
-    m_configButton->setStyleSheet(buttonStyleSheet);
-    m_activityButton->setStyleSheet(buttonStyleSheet);
-    m_helpButton->setStyleSheet(buttonStyleSheet);
-    m_closeButton->setStyleSheet(buttonStyleSheet);
-    //kDebug() << "stylesheet is" << buttonStyleSheet;
-
     //reset the icons
+    m_helpButton->setIcon(m_iconSvg->pixmap("help"));
     m_configButton->setIcon(m_iconSvg->pixmap("configure"));
     m_activityButton->setIcon(m_iconSvg->pixmap("status"));
     m_closeButton->setIcon(m_iconSvg->pixmap("close"));
@@ -387,25 +363,15 @@ void Interface::display(const QString &term)
     }
 }
 
-void Interface::setWidgetPalettes()
-{
-    // a nice palette to use with the widgets
-    QPalette widgetPalette = palette();
-    QColor bgColor = widgetPalette.color( QPalette::Active,
-                                                QPalette::Base );
-    bgColor.setAlpha( 200 );
-    widgetPalette.setColor( QPalette::Base, bgColor );
-
-    m_searchTerm->setPalette( widgetPalette );
-}
-
 void Interface::resetInterface()
 {
     setStaticQueryMode(false);
     m_delayedRun = false;
     m_searchTerm->setCurrentItem(QString(), true, 0);
     m_singleRunnerSearchTerm->clear();
-    m_resultsScene->clearQuery();
+    if (!m_running) {
+        m_resultsScene->clearQuery();
+    }
     resetResultsArea();
     resize(qMax(minimumSizeHint().width(), m_defaultSize.width()), minimumSizeHint().height());
 }
@@ -445,14 +411,6 @@ void Interface::showHelp()
     m_resultsScene->setQueryMatches(matches.values());
 }
 
-void Interface::ensureVisibility(QGraphicsItem* item)
-{
-    bool oldState = m_resultsScene->itemsAcceptHoverEvents();
-    m_resultsScene->setItemsAcceptHoverEvents(false);
-    m_resultsView->ensureVisible(item,0,0);
-    m_resultsScene->setItemsAcceptHoverEvents(oldState);
-}
-
 void Interface::setStaticQueryMode(bool staticQuery)
 {
     // don't show the search and other control buttons in the case of a static querymatch
@@ -464,7 +422,7 @@ void Interface::setStaticQueryMode(bool staticQuery)
     m_searchTerm->setVisible(visible && !singleRunner);
     m_singleRunnerSearchTerm->setVisible(visible && singleRunner);
     if (singleRunner) {
-        m_singleRunnerIcon->setPixmap(singleRunner->icon().pixmap( QSize( 22, 22 )));
+        m_singleRunnerIcon->setPixmap(singleRunner->icon().pixmap(QSize(22, 22)));
         m_singleRunnerDisplayName->setText(singleRunner->name());
     }
     m_singleRunnerIcon->setVisible(singleRunner);
@@ -473,21 +431,13 @@ void Interface::setStaticQueryMode(bool staticQuery)
 
 void Interface::hideEvent(QHideEvent *e)
 {
+    resetInterface();
     KRunnerDialog::hideEvent(e);
-
-    if (!m_running) {
-        resetInterface();
-    } else {
-        m_delayedRun = false;
-        resetResultsArea();
-        adjustSize();
-    }
-    e->accept();
 }
 
 void Interface::run(ResultItem *item)
 {
-    if (!item || item->group() < Plasma::QueryMatch::PossibleMatch) {
+    if (!item || !item->isValid() || item->group() < Plasma::QueryMatch::PossibleMatch) {
         m_delayedRun = true;
         return;
     }
@@ -510,28 +460,38 @@ void Interface::run(ResultItem *item)
                 }
             }
 
-            m_searchTerm->setItemText(0, info);
-            m_searchTerm->lineEdit()->setCursorPosition(editPos);
+            QStringList history = m_searchTerm->historyItems();
+            history.prepend(m_searchTerm->currentText().trimmed());
+            kDebug() << m_searchTerm->currentText() << history;
+            m_searchTerm->setHistoryItems(history);
             m_searchTerm->setCurrentIndex(0);
+            m_searchTerm->lineEdit()->setText(info);
+            m_searchTerm->lineEdit()->setCursorPosition(editPos);
             QApplication::clipboard()->setText(info);
         }
         return;
     }
 
     m_running = true;
+    // must run the result first before clearing the interface
+    // in a way that will cause the results scene to be cleared and
+    // the RunnerManager to be cleared of context as a result
     close();
     m_resultsScene->run(item);
     m_running = false;
 
     //TODO: check if run is succesful before adding the term to history
+    // Notice that we must add items to history after calling m_resultScene->run()
+    // otherwise the query result could be reset to QString() too early.
     m_searchTerm->addToHistory(m_searchTerm->currentText().trimmed());
+
     resetInterface();
 }
 
 void Interface::resetAndClose()
 {
-    close();
     resetInterface();
+    close();
 }
 
 
@@ -582,22 +542,32 @@ void Interface::matchCountChanged(int count)
         return;
     }
 
-    if (m_resultsContainer->isVisible() == show) {
-        return;
-    }
-
     if (show) {
         //kDebug() << "showing!" << minimumSizeHint();
 
-        // Next 2 lines are a workaround to allow arrow
-        // keys navigation in krunner's result list.
-        // Patch submited in bugreport #211578
-        QEvent event(QEvent::WindowActivate);
-        QApplication::sendEvent(m_resultsView, &event);
+        QSize s = m_defaultSize;
+        const int minHeight = minimumSizeHint().height();
+        const int resultsHeight = m_resultsScene->viewableHeight() + 2;
+        //kDebug() << minHeight << resultsHeight << s.height();
 
-        resize(m_defaultSize);
-        m_resultsContainer->show();
-        m_resultsScene->resize(m_resultsView->width(), qMax(m_resultsView->height(), int(m_resultsScene->height())));
+        if (minHeight + resultsHeight + m_layout->spacing() < s.height()) {
+            s.setHeight(minHeight + resultsHeight + m_layout->spacing());
+            m_resultsView->setMinimumHeight(resultsHeight);
+        }
+
+        resize(s);
+
+        if (!m_resultsView->isVisible()) {
+            // Next 2 lines are a workaround to allow arrow
+            // keys navigation in krunner's result list.
+            // Patch submited in bugreport #211578
+            QEvent event(QEvent::WindowActivate);
+            QApplication::sendEvent(m_resultsView, &event);
+
+            m_resultsView->show();
+        }
+        //m_resultsScene->resize(m_resultsView->width(), qMax(m_resultsView->height(), int(m_resultsScene->height())));
+        //kDebug() << s << size();
     } else {
         //kDebug() << "hiding ... eventually";
         m_delayedRun = false;
@@ -608,22 +578,15 @@ void Interface::matchCountChanged(int count)
 void Interface::hideResultsArea()
 {
     searchTermSetFocus();
-
     resetResultsArea();
-
     resize(qMax(minimumSizeHint().width(), m_defaultSize.width()), minimumSizeHint().height());
 }
 
 void Interface::resetResultsArea()
 {
-    setMinimumSize(QSize(MIN_WIDTH,0));
-    m_resultsContainer->hide();
-
-    //This is a workaround for some Qt bug which is not fully understood; it seems that
-    //adding a qgv to a layout and then hiding it gives some issues with resizing
-    //Calling updateGeometry should not be necessary, but it probably triggers some updates which do the trick
-
-    mainWidget()->updateGeometry();
+    m_resultsView->hide();
+    setMinimumSize(QSize(MIN_WIDTH, 0));
+    adjustSize();
 }
 
 #include "interface.moc"
