@@ -26,6 +26,7 @@
 
 // local includes
 #include "krunnerdialog.h"
+#include "resultitem.h"
 
 class QGraphicsView;
 class QGraphicsItem;
@@ -56,6 +57,7 @@ class Interface : public KRunnerDialog
         ~Interface();
 
         void setConfigWidget(QWidget *w);
+        bool eventFilter(QObject *obj, QEvent *event);
 
     public Q_SLOTS:
         void display(const QString& term = QString());
@@ -66,7 +68,6 @@ class Interface : public KRunnerDialog
         void hideEvent(QHideEvent *e);
 
     private slots:
-        void setWidgetPalettes();
         void run(ResultItem *item);
         void runDefaultResultItem();
         void queryTextEdited(const QString &query);
@@ -75,15 +76,16 @@ class Interface : public KRunnerDialog
         void themeUpdated();
         void resetInterface();
         void showHelp();
-        void ensureVisibility(QGraphicsItem *item);
         void cleanupAfterConfigWidget();
         void configWidgetDestroyed();
         void searchTermSetFocus();
-        void actionTriggered();
+        void resetAndClose();
         void delayedQueryLaunch();
         void updateSystemActivityToolTip();
 
     private:
+        void saveDialogSize(KConfigGroup &group);
+        void restoreDialogSize(KConfigGroup &group);
         void centerOnScreen();
         void setStaticQueryMode(bool staticQuery);
         void resetResultsArea();
@@ -91,7 +93,6 @@ class Interface : public KRunnerDialog
         QTimer m_hideResultsTimer;
 
         QWidget *m_buttonContainer;
-        QWidget *m_resultsContainer;
         QVBoxLayout* m_layout;
         QLabel *m_previousPage;
         QLabel *m_nextPage;
@@ -102,13 +103,13 @@ class Interface : public KRunnerDialog
         KHistoryComboBox* m_searchTerm;
         KCompletion *m_completion;
         QGraphicsView *m_resultsView;
-        QWidget *m_dividerLine;
         ResultScene *m_resultsScene;
         QSize m_defaultSize;
         QLabel *m_singleRunnerIcon;
         QLabel *m_singleRunnerDisplayName;
         KLineEdit *m_singleRunnerSearchTerm;
         QTimer m_delayedQueryTimer;
+        SharedResultData m_resultData;
         bool m_delayedRun : 1;
         bool m_running : 1;
         bool m_queryRunning : 1;
