@@ -27,7 +27,6 @@
 #include <QtCore/QMutexLocker>
 #include <QtGui/QPainter>
 #include <QtCore/QTimeLine>
-#include <QtGui/QGraphicsSceneWheelEvent>
 #include <QtGui/QGraphicsGridLayout>
 #include <QtGui/QGraphicsWidget>
 #include <QtGui/QGraphicsProxyWidget>
@@ -39,15 +38,15 @@
 #include <Plasma/AbstractRunner>
 #include <Plasma/RunnerManager>
 
-#include "resultitem.h"
 #include "selectionbar.h"
 
-ResultScene::ResultScene(Plasma::RunnerManager *manager, QWidget *focusBase, QObject *parent)
+ResultScene::ResultScene(SharedResultData *resultData, Plasma::RunnerManager *manager, QWidget *focusBase, QObject *parent)
     : QGraphicsScene(parent),
       m_runnerManager(manager),
+      m_viewableHeight(0),
       m_currentIndex(0),
       m_focusBase(focusBase),
-      m_itemsAcceptHoverEvents(false)
+      m_resultData(resultData)
 {
     setItemIndexMethod(NoIndex);
 
@@ -55,20 +54,20 @@ ResultScene::ResultScene(Plasma::RunnerManager *manager, QWidget *focusBase, QOb
             this, SLOT(setQueryMatches(const QList<Plasma::QueryMatch>&)));
 
     m_clearTimer.setSingleShot(true);
+    m_clearTimer.setInterval(200);
     connect(&m_clearTimer, SIGNAL(timeout()), this, SLOT(clearMatches()));
 
-    m_hoverTimer.setSingleShot(true);
-    connect(&m_hoverTimer, SIGNAL(timeout()), this, SLOT(initItemsHoverEvents()));
+    m_arrangeTimer.setSingleShot(true);
+    m_arrangeTimer.setInterval(50);
+    connect(&m_arrangeTimer, SIGNAL(timeout()), this, SLOT(arrangeItems()));
 
     m_selectionBar = new SelectionBar(0);
-    addItem(m_selectionBar);
-
+    connect(m_selectionBar, SIGNAL(appearanceChanged()), this, SLOT(updateItemMargins()));
+    connect(m_selectionBar, SIGNAL(targetItemReached(QGraphicsItem*)), this, SLOT(highlightItem(QGraphicsItem*)));
     m_selectionBar->hide();
     updateItemMargins();
 
-    connect(m_selectionBar, SIGNAL(appearanceChanged()), this, SLOT(updateItemMargins()));
-    //QColor bg(255, 255, 255, 126);
-    //setBackgroundBrush(bg);
+    addItem(m_selectionBar);
 }
 
 ResultScene::~ResultScene()
@@ -83,27 +82,29 @@ QSize ResultScene::minimumSizeHint() const
     return QSize(KIconLoader::SizeMedium * 4, (fm.height() * 5) * 3);
 }
 
-void ResultScene::resize(int width, int height)
+void ResultScene::setWidth(int width)
 {
-    bool resizeItems = width != sceneRect().width();
+    const bool resizeItems = width != sceneRect().width();
 
     if (resizeItems) {
         foreach (ResultItem *item, m_items) {
-            item->calculateSize(width, height);
+            item->calculateSize(width);
         }
+
+        setSceneRect(itemsBoundingRect());
     }
-    setSceneRect(itemsBoundingRect());
 }
 
 void ResultScene::clearMatches()
 {
+    clearSelection();
+    Plasma::QueryMatch dummy(0);
     foreach (ResultItem *item, m_items) {
-        removeItem(item);
-        item->deleteLater();
+        item->hide();
+        item->setMatch(dummy);
     }
 
-    m_itemsById.clear();
-    m_items.clear();
+    m_viewableHeight = 0;
     emit matchCountChanged(0);
 }
 
@@ -122,26 +123,9 @@ bool ResultScene::canMoveItemFocus() const
             (m_focusBase->hasFocus() && !focusedItem->mouseHovered()) ;
 }
 
-bool ResultScene::itemsAcceptHoverEvents()
+int ResultScene::viewableHeight() const
 {
-    return m_itemsAcceptHoverEvents;
-}
-
-void ResultScene::setItemsAcceptHoverEvents(bool enable)
-{
-    if (enable == m_itemsAcceptHoverEvents) {
-        return;
-    }
-
-    m_itemsAcceptHoverEvents = enable;
-    foreach (QGraphicsItem* tmpItem, items()) {
-        tmpItem->setAcceptHoverEvents(enable);
-    }
-}
-
-void ResultScene::initItemsHoverEvents()
-{
-    setItemsAcceptHoverEvents(true);
+    return m_viewableHeight;
 }
 
 void ResultScene::setQueryMatches(const QList<Plasma::QueryMatch> &m)
@@ -155,130 +139,107 @@ void ResultScene::setQueryMatches(const QList<Plasma::QueryMatch> &m)
 
     if (m.isEmpty()) {
         //kDebug() << "clearing";
-        resize(width(), 0);
-        emit itemHoverEnter(0);
-        m_clearTimer.start(200);
+        //resize(width(), 0);
+        m_clearTimer.start();
         return;
     }
 
-    m_hoverTimer.stop();
-    setItemsAcceptHoverEvents(false);
-
-    //resize(width(), m.count() * ResultItem::BOUNDING_HEIGHT);
     m_clearTimer.stop();
-    m_items.clear();
+    const int maxItemsAllowed = 50;
+
+    if (m_items.isEmpty()) {
+        QTime t;
+        t.start();
+        Plasma::QueryMatch dummy(0);
+        for (int i = 0; i < maxItemsAllowed; ++i) {
+            ResultItem *item = new ResultItem(m_resultData, dummy, m_runnerManager, 0);
+            item->setContentsMargins(m_itemMarginLeft, m_itemMarginTop,
+                                     m_itemMarginRight, m_itemMarginBottom);
+            item->hide();
+            item->setIndex(i);
+            connect(item, SIGNAL(ensureVisibility(QGraphicsItem*)), this, SIGNAL(ensureVisibility(QGraphicsItem*)));
+            connect(item, SIGNAL(activated(ResultItem*)), this, SIGNAL(itemActivated(ResultItem*)));
+            connect(item, SIGNAL(sizeChanged(ResultItem*)), this, SLOT(scheduleArrangeItems()));
+
+            m_items << item;
+            addItem(item);
+        }
+
+        arrangeItems();
+        kDebug() << "creating all items took" << t.elapsed();
+    }
 
     QList<Plasma::QueryMatch> matches = m;
-    QMutableListIterator<Plasma::QueryMatch> newMatchIt(matches);
+    qSort(matches.begin(), matches.end());
+    QListIterator<Plasma::QueryMatch> mit(matches);
+    mit.toBack();
+    QListIterator<ResultItem *> rit(m_items);
+    QGraphicsWidget *prevTabItem = 0;
 
-    // first pass: we try and match up items with existing ids (match persisitence)
-    while (!m_itemsById.isEmpty() && newMatchIt.hasNext()) {
-        ResultItem *item = addQueryMatch(newMatchIt.next(), false);
+    while (mit.hasPrevious() && rit.hasNext()) {
+        ResultItem * item = rit.next();
+        item->setMatch(mit.previous());
+        prevTabItem = item->arrangeTabOrder(prevTabItem);
+        item->show();
+        m_viewableHeight = item->sceneBoundingRect().bottom();
+    }
 
-        if (item) {
-            m_items.append(item);
-            newMatchIt.remove();
+    Plasma::QueryMatch dummy(0);
+    while (rit.hasNext()) {
+        ResultItem *item = rit.next();
+        item->hide();
+        if (item->isValid()) {
+            item->setMatch(dummy);
         }
     }
 
-    // second pass: now we just use any item that exists (item re-use)
-    newMatchIt.toFront();
-    while (newMatchIt.hasNext()) {
-        m_items.append(addQueryMatch(newMatchIt.next(), true));
+    clearSelection();
+    if (matches.count() > 0) {
+        ResultItem *first = m_items.at(0);
+        setFocusItem(first);
+        first->setSelected(true);
+        first->highlight(true);
+        emit ensureVisibility(first);
     }
 
-    // delete the stragglers
-    QMapIterator<QString, ResultItem *> it(m_itemsById);
-    while (it.hasNext()) {
-        removeItem(it.next().value());
-        it.value()->deleteLater();
-    }
-
-    // organize the remainders
-    m_itemsById.clear();
-
-    // this will leave them in *reverse* order
-    qSort(m_items.begin(), m_items.end(), ResultItem::compare);
-
-    emit matchCountChanged(m.count());
-    arrangeItems(0);
-
-    m_hoverTimer.start(200);
-
+    emit matchCountChanged(qMin(m.count(), maxItemsAllowed));
 }
 
-void ResultScene::arrangeItems(ResultItem *itemChanged)
+void ResultScene::scheduleArrangeItems()
+{
+    if (!m_arrangeTimer.isActive()) {
+        m_arrangeTimer.start();
+    }
+}
+
+void ResultScene::arrangeItems()
 {
-    QListIterator<ResultItem*> matchIt(m_items);
-    QGraphicsWidget *tab = 0;
     int y = 0;
-    int i = 0;
+    QListIterator<ResultItem*> matchIt(m_items);
     while (matchIt.hasNext()) {
         ResultItem *item = matchIt.next();
         //kDebug()  << item->name() << item->id() << item->priority() << i;
 
-        if (!itemChanged) {
-            // first time set up
-            tab = item->arrangeTabOrder(tab);
-            m_itemsById.insert(item->id(), item);
-            item->setIndex(i);
-        }
-
         item->setPos(0, y);
-        item->show();
         //kDebug() << item->pos();
         y += item->geometry().height();
-
-        // it is vital that focus is set *after* the index
-        if (!itemChanged && i == 0 && canMoveItemFocus()) {
-            setFocusItem(item);
-            emit ensureVisibility(item);
+        if (item->isVisible()) {
+            m_viewableHeight = item->sceneBoundingRect().bottom();
         }
-
-        ++i;
     }
 
-    // Here we cannot use itemsBoundingRect().height() because old items will be deleted on the next event cycle
-    // However we use itemsBoundingRect().width() to take care of width changes.
-    setSceneRect(QRect(0,0,itemsBoundingRect().width(),y));
+    //kDebug() << "setting scene rect to" << itemsBoundingRect();
+    setSceneRect(itemsBoundingRect());
 }
 
-ResultItem* ResultScene::addQueryMatch(const Plasma::QueryMatch &match, bool useAnyId)
+void ResultScene::highlightItem(QGraphicsItem *item)
 {
-    QMap<QString, ResultItem*>::iterator it = useAnyId ? m_itemsById.begin() : m_itemsById.find(match.id());
-    ResultItem *item = 0;
-    //kDebug() << "attempting" << match.id();
-
-    if (it == m_itemsById.end()) {
-        //kDebug() << "did not find for" << match.id();
-        if (useAnyId) {
-            //kDebug() << "creating for" << match.id();
-            item = new ResultItem(match, m_runnerManager, 0);
-            item->setContentsMargins(m_itemMarginLeft, m_itemMarginTop,
-                                     m_itemMarginRight, m_itemMarginBottom);
-            addItem(item);
-            item->hide();
-            connect(item, SIGNAL(ensureVisibility(QGraphicsItem*)), this, SIGNAL(ensureVisibility(QGraphicsItem*)));
-            connect(item, SIGNAL(sizeChanged(ResultItem*)), this, SLOT(arrangeItems(ResultItem*)));
-            connect(item, SIGNAL(activated(ResultItem*)), this, SIGNAL(itemActivated(ResultItem*)));
-            connect(item, SIGNAL(hoverEnter(ResultItem*)), this, SIGNAL(itemHoverEnter(ResultItem*)));
-            connect(item, SIGNAL(hoverLeave(ResultItem*)), this, SIGNAL(itemHoverLeave(ResultItem*)));
-            connect(item, SIGNAL(actionTriggered()), this, SIGNAL(actionTriggered()));
-        } else {
-            //kDebug() << "returning failure for" << match.id();
-            return 0;
-        }
-    } else {
-        item = it.value();
-        //kDebug() << "reusing" << item->name() << "for" << match.id();
-        item->setMatch(match);
-        m_itemsById.erase(it);
+    ResultItem *rItem = dynamic_cast<ResultItem *>(item);
+    if (rItem) {
+        rItem->highlight(true);
     }
-
-    return item;
 }
 
-
 void ResultScene::focusInEvent(QFocusEvent *focusEvent)
 {
 
@@ -304,26 +265,18 @@ void ResultScene::focusInEvent(QFocusEvent *focusEvent)
     }
 }
 
-void ResultScene::focusOutEvent(QFocusEvent *focusEvent)
-{
-    QGraphicsScene::focusOutEvent(focusEvent);
-    if (!m_items.isEmpty()) {
-        emit itemHoverEnter(m_items.at(0));
-    }
-}
-
 void ResultScene::keyPressEvent(QKeyEvent * keyEvent)
 {
     //kDebug() << "m_items (size): " << m_items.size() << "\n";
     switch (keyEvent->key()) {
         case Qt::Key_Up:
         case Qt::Key_Left:
-            selectNextItem();
+            selectPreviousItem();
             break;
 
         case Qt::Key_Down:
         case Qt::Key_Right:
-            selectPreviousItem();
+            selectNextItem();
         break;
 
         default:
@@ -351,35 +304,61 @@ ResultItem* ResultScene::currentlyFocusedItem() const
     return currentFocus;
 }
 
-void ResultScene::selectNextItem()
-{
-    ResultItem *currentFocus = currentlyFocusedItem();
-    int currentIndex = currentFocus ? currentFocus->index() : 0;
-
-    if (currentIndex > 0) {
-        --currentIndex;
-    } else {
-        currentIndex = m_items.size() - 1;
-    }
-
-    currentFocus = m_items.at(currentIndex);
-    setFocusItem(currentFocus);
-    emit ensureVisibility(currentFocus);
-}
-
 void ResultScene::selectPreviousItem()
 {
     ResultItem *currentFocus = currentlyFocusedItem();
     int currentIndex = currentFocus ? currentFocus->index() : 0;
 
-    ++currentIndex;
-    if (currentIndex >= m_items.size()) {
-        currentIndex = 0;
+    bool wrapped = false;
+    if (currentIndex > 0) {
+        currentFocus = m_items.at(currentIndex - 1);
+    } else {
+        currentIndex = m_items.size();
+        do {
+            currentFocus = m_items.at(--currentIndex);
+        } while (currentIndex > 0 && !currentFocus->isVisible());
+        wrapped = currentIndex > 2;
     }
 
-    currentFocus = m_items.at(currentIndex);
-    setFocusItem(currentFocus);
-    emit ensureVisibility(currentFocus);
+    if (currentFocus->isVisible()) {
+        if (wrapped) {
+            // with more than two items, having the selection zoom through the items between looks
+            // odd
+            m_selectionBar->setTargetItem(0);
+            currentFocus->highlight(true);
+        }
+
+        setFocusItem(currentFocus);
+        emit ensureVisibility(currentFocus);
+    }
+}
+
+void ResultScene::selectNextItem()
+{
+    ResultItem *currentFocus = currentlyFocusedItem();
+    int currentIndex = currentFocus ? currentFocus->index() : 0;
+    const int wasIndex = currentIndex;
+
+    bool wrapped = false;
+    do {
+        ++currentIndex;
+        if (currentIndex >= m_items.size()) {
+            // with more than two items, having the selection zoom through the items between looks
+            // odd
+            wrapped = wasIndex > 2;
+            currentIndex = 0;
+        }
+        currentFocus = m_items.at(currentIndex);
+    } while (!currentFocus->isVisible() && currentIndex < m_items.size());
+
+    if (currentFocus->isVisible()) {
+        if (wrapped) {
+            m_selectionBar->setTargetItem(0);
+            currentFocus->highlight(true);
+        }
+        setFocusItem(currentFocus);
+        emit ensureVisibility(currentFocus);
+    }
 }
 
 bool ResultScene::launchQuery(const QString &term)
@@ -398,7 +377,7 @@ bool ResultScene::launchQuery(const QString &term, const QString &runner)
 
 void ResultScene::clearQuery()
 {
-    m_selectionBar->setTargetItem(0);
+    //m_selectionBar->setTargetItem(0);
     setFocusItem(0);
     clearSelection();
     m_runnerManager->reset();
@@ -424,12 +403,6 @@ void ResultScene::run(ResultItem *item) const
     item->run(m_runnerManager);
 }
 
-/*
-Plasma::RunnerManager* ResultScene::manager() const
-{
-    return m_runnerManager;
-}
-*/
 void ResultScene::updateItemMargins()
 {
     m_selectionBar->getContentsMargins(&m_itemMarginLeft, &m_itemMarginTop,
