@@ -35,6 +35,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QEvent>
 #include <QMouseEvent>
 #include <kglobalsettings.h>
+#include <QtGui/QPainter>
+#include <QtGui/QGraphicsLinearLayout>
+#include <Plasma/PushButton>
+#include <Plasma/WindowEffects>
 
 namespace KWin
 {
@@ -157,7 +161,30 @@ void DesktopGridEffect::paintScreen( int mask, QRegion region, ScreenPaintData&
         paintingDesktop = desktop;
         effects->paintScreen( mask, region, d );
         }
-    if( isUsingPresentWindows() && windowMove )
+
+    // paint the add desktop button
+    for( QHash< DesktopButtonsView*, EffectWindow*>::iterator it = m_desktopButtonsViews.begin();
+        it != m_desktopButtonsViews.end(); ++it )
+        {
+        if( !it.value() )
+            {
+            EffectWindow *view = effects->findWindow( it.key()->winId() );
+            if( view )
+                {
+                view->setData( WindowForceBlurRole, QVariant( true ) );
+                it.value() = view;
+                }
+            }
+        if( it.value() )
+            {
+            WindowPaintData d( it.value() );
+            d.opacity *= timeline.value();
+            effects->drawWindow( it.value(), PAINT_WINDOW_TRANSLUCENT,
+                                 infiniteRegion(), d );
+            }
+        }
+
+    if( isUsingPresentWindows() && windowMove && wasWindowMove )
         {
         // the moving window has to be painted on top of all desktops
         QPoint diff = cursorPos() - m_windowMoveStartPoint;
@@ -176,7 +203,8 @@ void DesktopGridEffect::paintScreen( int mask, QRegion region, ScreenPaintData&
             {
             QRect screenGeom = effects->clientArea( ScreenArea, screen, 0 );
             PaintClipper pc( screenGeom ); // TODO: Doesn't work in XRender for some reason?
-            for( int desktop = 1; desktop <= effects->numberOfDesktops(); desktop++ )
+            int desktop = 1;
+            foreach( EffectFrame* frame, desktopNames )
                 {
                 QPointF posTL( scalePos( screenGeom.topLeft(), desktop, screen ));
                 QPointF posBR( scalePos( screenGeom.bottomRight(), desktop, screen ));
@@ -196,8 +224,9 @@ void DesktopGridEffect::paintScreen( int mask, QRegion region, ScreenPaintData&
                     y = textArea.bottom();
                 else
                     y = textArea.center().y();
-                desktopNames[desktop-1]->setPosition( QPoint( x, y ));
-                desktopNames[desktop-1]->render( region, timeline.value(), 0.7 );
+                frame->setPosition( QPoint( x, y ));
+                frame->render( region, timeline.value(), 0.7 );
+                ++desktop;
                 }
             }
         }
@@ -250,6 +279,8 @@ void DesktopGridEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData& dat
                 if( w->y() + w->height() > screenGeom.y() + screenGeom.height() )
                     data.quads = data.quads.splitAtY( screenGeom.y() + screenGeom.height() - w->y() );
                 }
+            if( windowMove && wasWindowMove && windowMove->findModal() == w )
+                w->disablePainting( EffectWindow::PAINT_DISABLED_BY_DESKTOP );
             }
         else
             w->disablePainting( EffectWindow::PAINT_DISABLED_BY_DESKTOP );
@@ -261,10 +292,12 @@ void DesktopGridEffect::paintWindow( EffectWindow* w, int mask, QRegion region,
     {
     if( timeline.value() != 0 || (isUsingPresentWindows() && isMotionManagerMovingWindows()) )
         {
-        if( isUsingPresentWindows() && w == windowMove )
+        if( isUsingPresentWindows() && w == windowMove && wasWindowMove )
             {
             return; // will be painted on top of all other windows
             }
+        if( m_desktopButtonsViews.values().contains( w ) )
+            return; // will be painted on top of all other windows
 
         double xScale = data.xScale;
         double yScale = data.yScale;
@@ -321,7 +354,7 @@ void DesktopGridEffect::paintWindow( EffectWindow* w, int mask, QRegion region,
             d.xTranslate += qRound( newPos.x() - w->x() );
             d.yTranslate += qRound( newPos.y() - w->y() );
 
-            if( isUsingPresentWindows() && w->isDock() )
+            if( isUsingPresentWindows() && ( w->isDock() || w->isSkipSwitcher() ) )
                 {
                 // fade out panels if present windows is used
                 d.opacity *= ( 1.0 - timeline.value() );
@@ -391,7 +424,7 @@ void DesktopGridEffect::windowAdded( EffectWindow* w )
 
 void DesktopGridEffect::windowClosed( EffectWindow* w )
     {
-    if ( !activated )
+    if ( !activated && timeline.value() == 0 )
         return;
     if( w == windowMove )
         {
@@ -411,14 +444,51 @@ void DesktopGridEffect::windowClosed( EffectWindow* w )
             }
         else
             {
-            WindowMotionManager& manager = m_managers[(w->desktop()-1)*effects->numScreens()+w->screen()];
-            manager.unmanage(w);
-            m_proxy->calculateWindowTransformations(manager.managedWindows(), w->screen(), manager);
+            if( w->desktop() <= effects->numberOfDesktops() )
+                {
+                WindowMotionManager& manager = m_managers[(w->desktop()-1)*effects->numScreens()+w->screen()];
+                manager.unmanage(w);
+                m_proxy->calculateWindowTransformations(manager.managedWindows(), w->screen(), manager);
+                }
+            }
+        }
+    for( QHash< DesktopButtonsView*, EffectWindow*>::iterator it = m_desktopButtonsViews.begin();
+         it != m_desktopButtonsViews.end(); ++it )
+        {
+        if( it.value() && it.value() == w )
+            {
+            w->refWindow();
+            break;
             }
         }
     effects->addRepaintFull();
     }
 
+void DesktopGridEffect::windowGeometryShapeChanged( EffectWindow* w, const QRect& old )
+    {
+    Q_UNUSED( old )
+    if( !activated )
+        return;
+    if( w == windowMove && wasWindowMove )
+        return;
+    if( isUsingPresentWindows() )
+        {
+        if( w->isOnAllDesktops() )
+            {
+            for( int i=0; i<effects->numberOfDesktops(); i++ )
+                {
+                WindowMotionManager& manager = m_managers[i*effects->numScreens()+w->screen()];
+                m_proxy->calculateWindowTransformations(manager.managedWindows(), w->screen(), manager);
+                }
+            }
+        else
+            {
+            WindowMotionManager& manager = m_managers[(w->desktop()-1)*effects->numScreens()+w->screen()];
+            m_proxy->calculateWindowTransformations(manager.managedWindows(), w->screen(), manager);
+            }
+        }
+    }
+
 void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
     {
     if((   e->type() != QEvent::MouseMove
@@ -427,6 +497,20 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
         || timeline.value() != 1 ) // Block user input during animations
         return;
     QMouseEvent* me = static_cast< QMouseEvent* >( e );
+    for( QHash< DesktopButtonsView*, EffectWindow*>::iterator it = m_desktopButtonsViews.begin();
+         it != m_desktopButtonsViews.end(); ++it )
+        {
+        DesktopButtonsView* view = it.key();
+        if( !wasWindowMove && !wasDesktopMove && view->geometry().contains( me->pos() ) )
+            {
+            const QPoint widgetPos = view->mapFromGlobal( me->pos() );
+            const QPointF scenePos = view->mapToScene( widgetPos );
+            QMouseEvent event( me->type(), widgetPos, me->pos(), me->button(), me->buttons(), me->modifiers() );
+            view->windowInputMouseEvent( &event );
+            return;
+            }
+        }
+
     if( e->type() == QEvent::MouseMove )
         {
         int d = posToDesktop( me->pos());
@@ -450,6 +534,11 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
                             m_windowMoveStartPoint = me->pos();
                             }
                         manager.unmanage( windowMove );
+                        if( EffectWindow* modal = windowMove->findModal() )
+                            {
+                            if( manager.isManaging( modal ) )
+                                manager.unmanage( modal );
+                            }
                         m_proxy->calculateWindowTransformations( manager.managedWindows(), windowMove->screen(), manager );
                         }
                     }
@@ -464,6 +553,11 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
                     m_windowMoveStartPoint = me->pos();
 
                     manager.unmanage( windowMove );
+                    if( EffectWindow* modal = windowMove->findModal() )
+                        {
+                        if( manager.isManaging( modal ) )
+                            manager.unmanage( modal );
+                        }
                     m_proxy->calculateWindowTransformations( manager.managedWindows(), windowMove->screen(), manager );
                     }
                 XDefineCursor( display(), input, QCursor( Qt::ClosedHandCursor ).handle() );
@@ -613,10 +707,18 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
                     {
                     if( windowMove->isOnAllDesktops() )
                         {
+                        const int targetDesktop = posToDesktop( cursorPos() );
                         for( int i=0; i<effects->numberOfDesktops(); ++i )
                             {
                             WindowMotionManager& manager = m_managers[ (i)*(effects->numScreens()) + windowMove->screen() ];
                             manager.manage( windowMove );
+                            if( EffectWindow* modal = windowMove->findModal() )
+                                manager.manage( modal );
+                            if( i+1 == targetDesktop )
+                                {
+                                // for the desktop the window is dropped on, we use the current geometry
+                                manager.setTransformedGeometry( windowMove, moveGeometryToDesktop( targetDesktop ) );
+                                }
                             m_proxy->calculateWindowTransformations( manager.managedWindows(), windowMove->screen(), manager );
                             }
                         }
@@ -624,6 +726,9 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
                         {
                         WindowMotionManager& manager = m_managers[ (windowMove->desktop()-1)*(effects->numScreens()) + windowMove->screen() ];
                         manager.manage( windowMove );
+                        if( EffectWindow* modal = windowMove->findModal() )
+                            manager.manage( modal );
+                        manager.setTransformedGeometry( windowMove, moveGeometryToDesktop( windowMove->desktop() ) );
                         m_proxy->calculateWindowTransformations( manager.managedWindows(), windowMove->screen(), manager );
                         }
                     effects->addRepaintFull();
@@ -691,6 +796,12 @@ void DesktopGridEffect::grabbedKeyboardEvent( QKeyEvent* e )
                 setCurrentDesktop( highlightedDesktop );
                 setActive( false );
                 return;
+            case Qt::Key_Plus:
+                slotAddDesktop();
+                break;
+            case Qt::Key_Minus:
+                slotRemoveDesktop();
+                break;
             default:
                 break;
             }
@@ -1020,8 +1131,6 @@ void DesktopGridEffect::setActive( bool active )
     {
     if( effects->activeFullScreenEffect() && effects->activeFullScreenEffect() != this )
         return; // Only one fullscreen effect at a time thanks
-    if( effects->numberOfDesktops() < 2 )
-        return; // No point if there is only one desktop
     if( active && isMotionManagerMovingWindows() )
         return; // Still moving windows from last usage - don't activate
     if( activated == active )
@@ -1044,6 +1153,11 @@ void DesktopGridEffect::setActive( bool active )
                 }
             }
         setHighlightedDesktop( effects->currentDesktop() ); // Ensure selected desktop is highlighted
+        for( QHash< DesktopButtonsView*, EffectWindow*>::iterator it = m_desktopButtonsViews.begin();
+             it != m_desktopButtonsViews.end(); ++it )
+            {
+            it.key()->hide();
+            }
         }
     effects->addRepaintFull();
     }
@@ -1069,19 +1183,64 @@ void DesktopGridEffect::setup()
     // Create desktop name textures if enabled
     if( desktopNameAlignment )
         {
-        desktopNames = new EffectFrame*[effects->numberOfDesktops()];
         QFont font;
         font.setBold( true );
         font.setPointSize( 12 );
         for( int i = 0; i < effects->numberOfDesktops(); i++ )
             {
-            desktopNames[i] = new EffectFrame( EffectFrame::Unstyled, false );
-            desktopNames[i]->setFont( font );
-            desktopNames[i]->setText( effects->desktopName( i+1 ));
-            desktopNames[i]->setAlignment( desktopNameAlignment );
+            EffectFrame* frame = new EffectFrame( EffectFrame::Unstyled, false );
+            frame->setFont( font );
+            frame->setText( effects->desktopName( i+1 ));
+            frame->setAlignment( desktopNameAlignment );
+            desktopNames.append( frame );
             }
         }
+    setupGrid();
+    setCurrentDesktop( effects->currentDesktop() );
 
+    // setup the motion managers
+    if( m_usePresentWindows )
+        m_proxy = static_cast<PresentWindowsEffectProxy*>( effects->getProxy( "presentwindows" ) );
+    if( isUsingPresentWindows() )
+        {
+        for( int i=1; i<=effects->numberOfDesktops(); i++ )
+            {
+            for( int j=0; j<effects->numScreens(); j++ )
+                {
+                WindowMotionManager manager;
+                foreach( EffectWindow* w, effects->stackingOrder() )
+                    {
+                    if( w->isOnDesktop( i ) && w->screen() == j && !w->isDesktop() && !w->isDock() &&
+                        w->visibleInClientGroup() && !w->isSkipSwitcher() )
+                        {
+                        manager.manage( w );
+                        }
+                    }
+                m_proxy->calculateWindowTransformations(manager.managedWindows(), j, manager);
+                m_managers.append(manager);
+                }
+            }
+        }
+    bool enableAdd = effects->numberOfDesktops() < 20;
+    bool enableRemove = effects->numberOfDesktops() > 1;
+    for( int i=0; i<effects->numScreens(); ++i )
+        {
+        DesktopButtonsView* view = new DesktopButtonsView();
+        view->setAddDesktopEnabled( enableAdd );
+        view->setRemoveDesktopEnabled( enableRemove );
+        connect( view, SIGNAL(addDesktop()), SLOT(slotAddDesktop()));
+        connect( view, SIGNAL(removeDesktop()), SLOT(slotRemoveDesktop()));
+        const QRect screenRect = effects->clientArea( FullScreenArea, i, 1 );
+        view->setGeometry( screenRect.right() + 1 - view->sceneRect().width(),
+                           screenRect.bottom() + 1 - view->sceneRect().height(),
+                           view->sceneRect().width(), view->sceneRect().height() );
+        view->show();
+        m_desktopButtonsViews.insert( view, NULL );
+        }
+    }
+
+void DesktopGridEffect::setupGrid()
+    {
     // We need these variables for every paint so lets cache them
     int x, y;
     int numDesktops = effects->numberOfDesktops();
@@ -1107,7 +1266,6 @@ void DesktopGridEffect::setup()
             gridSize.setHeight( customLayoutRows );
             break;
         }
-    setCurrentDesktop( effects->currentDesktop() );
     scale.clear();
     unscaledBorder.clear();
     scaledSize.clear();
@@ -1134,39 +1292,14 @@ void DesktopGridEffect::setup()
         scaledSize.append( size );
         scaledOffset.append( offset );
         }
-
-    // setup the motion managers
-    if( m_usePresentWindows )
-        m_proxy = static_cast<PresentWindowsEffectProxy*>( effects->getProxy( "presentwindows" ) );
-    if( isUsingPresentWindows() )
-        {
-        for( int i=1; i<=effects->numberOfDesktops(); i++ )
-            {
-            for( int j=0; j<effects->numScreens(); j++ )
-                {
-                WindowMotionManager manager;
-                foreach( EffectWindow* w, effects->stackingOrder() )
-                    {
-                    if( w->isOnDesktop( i ) && w->screen() == j && !w->isDesktop() && !w->isDock() &&
-                        w->visibleInClientGroup() )
-                        {
-                        manager.manage( w );
-                        }
-                    }
-                m_proxy->calculateWindowTransformations(manager.managedWindows(), j, manager);
-                m_managers.append(manager);
-                }
-            }
-        }
     }
 
 void DesktopGridEffect::finish()
     {
     if( desktopNameAlignment )
         {
-        for( int i = 0; i < effects->numberOfDesktops(); i++ )
-            delete desktopNames[i];
-        delete[] desktopNames;
+        qDeleteAll(desktopNames);
+        desktopNames.clear();
         }
 
     if( keyboardGrab )
@@ -1183,6 +1316,15 @@ void DesktopGridEffect::finish()
             }
         m_proxy = 0;
         }
+    while( !m_desktopButtonsViews.isEmpty() )
+        {
+        DesktopButtonsView* view = m_desktopButtonsViews.begin().key();
+        m_desktopButtonsViews[ view ]->unrefWindow();
+        m_desktopButtonsViews[ view ] = 0;
+        m_desktopButtonsViews.remove( view );
+        delete view;
+        view = 0;
+        }
     }
 
 void DesktopGridEffect::globalShortcutChanged( const QKeySequence& seq )
@@ -1209,6 +1351,250 @@ bool DesktopGridEffect::isUsingPresentWindows() const
     return (m_proxy != NULL);
     }
 
+// transforms the geometry of the moved window to a geometry on the desktop
+// internal method only used when a window is dropped onto a desktop
+QRectF DesktopGridEffect::moveGeometryToDesktop( int desktop ) const
+    {
+    QPointF point = unscalePos( m_windowMoveGeometry.topLeft() + cursorPos() - m_windowMoveStartPoint );
+    const double scaleFactor = scale[ windowMove->screen() ];
+    if( posToDesktop( m_windowMoveGeometry.topLeft() + cursorPos() - m_windowMoveStartPoint ) != desktop )
+        {
+        // topLeft is not on the desktop - check other corners
+        // if all corners are not on the desktop the window is bigger than the desktop - no matter what it will look strange
+        if( posToDesktop( m_windowMoveGeometry.topRight() + cursorPos() - m_windowMoveStartPoint ) == desktop )
+            {
+            point = unscalePos( m_windowMoveGeometry.topRight() + cursorPos() - m_windowMoveStartPoint ) -
+                QPointF( m_windowMoveGeometry.width(), 0) / scaleFactor;
+            }
+        else if( posToDesktop( m_windowMoveGeometry.bottomLeft() + cursorPos() - m_windowMoveStartPoint ) == desktop )
+            {
+            point = unscalePos( m_windowMoveGeometry.bottomLeft() + cursorPos() - m_windowMoveStartPoint ) -
+                QPointF(0, m_windowMoveGeometry.height() ) / scaleFactor;
+            }
+        else if( posToDesktop( m_windowMoveGeometry.bottomRight() + cursorPos() - m_windowMoveStartPoint ) == desktop )
+            {
+            point = unscalePos( m_windowMoveGeometry.bottomRight() + cursorPos() - m_windowMoveStartPoint ) -
+                QPointF(m_windowMoveGeometry.width(), m_windowMoveGeometry.height() ) / scaleFactor;
+            }
+        }
+    return QRectF( point, m_windowMoveGeometry.size() / scaleFactor );
+    }
+
+void DesktopGridEffect::slotAddDesktop()
+    {
+    effects->setNumberOfDesktops( effects->numberOfDesktops() + 1 );
+    }
+
+void DesktopGridEffect::slotRemoveDesktop()
+    {
+    effects->setNumberOfDesktops( effects->numberOfDesktops() - 1 );
+    }
+
+void DesktopGridEffect::numberDesktopsChanged( int old )
+    {
+    if( !activated )
+        return;
+    const int desktop = effects->numberOfDesktops();
+    bool enableAdd = desktop < 20;
+    bool enableRemove = desktop > 1;
+    for( QHash< DesktopButtonsView*, EffectWindow* >::iterator it = m_desktopButtonsViews.begin();
+         it != m_desktopButtonsViews.end(); ++it )
+        {
+        it.key()->setAddDesktopEnabled( enableAdd );
+        it.key()->setRemoveDesktopEnabled( enableRemove );
+        }
+    if( old < desktop )
+        desktopsAdded( old );
+    else
+        desktopsRemoved( old );
+    }
+
+void DesktopGridEffect::desktopsAdded( int old )
+    {
+    const int desktop = effects->numberOfDesktops();
+    for( int i=old; i<=effects->numberOfDesktops(); i++ )
+        {
+        // add a timeline for the new desktop
+        TimeLine newTimeline( animationTime( zoomDuration ));
+        newTimeline.setCurveShape( TimeLine::EaseInOutCurve );
+        hoverTimeline.append( newTimeline );
+        }
+
+    // Create desktop name textures if enabled
+    if( desktopNameAlignment )
+        {
+        QFont font;
+        font.setBold( true );
+        font.setPointSize( 12 );
+        for( int i = old; i < desktop; i++ )
+            {
+            EffectFrame* frame = new EffectFrame( EffectFrame::Unstyled, false );
+            frame->setFont( font );
+            frame->setText( effects->desktopName( i+1 ));
+            frame->setAlignment( desktopNameAlignment );
+            desktopNames.append( frame );
+            }
+        }
+
+    if( isUsingPresentWindows() )
+        {
+        for( int i=old; i<=effects->numberOfDesktops(); i++ )
+            {
+            for( int j=0; j<effects->numScreens(); j++ )
+                {
+                WindowMotionManager manager;
+                foreach( EffectWindow* w, effects->stackingOrder() )
+                    {
+                    if( w->isOnDesktop( i ) && w->screen() == j && !w->isDesktop() && !w->isDock() &&
+                        w->visibleInClientGroup() )
+                        {
+                        manager.manage( w );
+                        }
+                    }
+                m_proxy->calculateWindowTransformations(manager.managedWindows(), j, manager);
+                m_managers.append(manager);
+                }
+            }
+        }
+
+    setupGrid();
+
+    // and repaint
+    effects->addRepaintFull();
+    }
+
+void DesktopGridEffect::desktopsRemoved( int old )
+    {
+    const int desktop = effects->numberOfDesktops();
+    for( int i = desktop; i < old; i++ )
+        {
+        hoverTimeline.removeLast();
+        if( desktopNameAlignment )
+            {
+            delete desktopNames.last();
+            desktopNames.removeLast();
+            }
+        if( isUsingPresentWindows() )
+            {
+            for( int j=0; j<effects->numScreens(); ++j )
+                {
+                WindowMotionManager& manager = m_managers.last();
+                manager.unmanageAll();
+                m_managers.removeLast();
+                }
+            }
+        }
+    // add removed windows to the last desktop
+    if( isUsingPresentWindows() )
+        {
+        for( int j=0; j<effects->numScreens(); ++j )
+            {
+            WindowMotionManager& manager = m_managers[ (desktop-1)*(effects->numScreens())+j ];
+            foreach( EffectWindow* w, effects->stackingOrder() )
+                {
+                if( !manager.isManaging( w ) && w->isOnDesktop( desktop ) && w->screen() == j &&
+                    !w->isDesktop() && !w->isDock() && w->visibleInClientGroup() )
+                    {
+                    manager.manage( w );
+                    }
+                }
+            m_proxy->calculateWindowTransformations( manager.managedWindows(), j, manager );
+            }
+        }
+
+    setupGrid();
+
+    // and repaint
+    effects->addRepaintFull();
+    }
+
+/************************************************
+* DesktopButtonView
+************************************************/
+DesktopButtonsView::DesktopButtonsView( QWidget* parent )
+    : QGraphicsView( parent )
+    {
+    setWindowFlags( Qt::X11BypassWindowManagerHint );
+    setAttribute( Qt::WA_TranslucentBackground );
+    setFrameShape( QFrame::NoFrame );
+    QPalette pal = palette();
+    pal.setColor( backgroundRole(), Qt::transparent );
+    setPalette( pal );
+    setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+    setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+
+    // setup the scene
+    QGraphicsScene* scene = new QGraphicsScene( this );
+    m_addDesktopButton = new Plasma::PushButton();
+    m_addDesktopButton->setIcon( KIcon( "list-add" ) );
+    m_removeDesktopButton = new Plasma::PushButton();
+    m_removeDesktopButton->setIcon(KIcon("list-remove"));
+    scene->addItem( m_addDesktopButton );
+    scene->addItem( m_removeDesktopButton );
+    connect( m_addDesktopButton, SIGNAL(clicked()), SIGNAL(addDesktop()));
+    connect( m_removeDesktopButton, SIGNAL(clicked()), SIGNAL(removeDesktop()));
+
+    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout;
+    layout->addItem( m_addDesktopButton );
+    layout->addItem( m_removeDesktopButton );
+
+    QGraphicsWidget *form = new QGraphicsWidget;
+    form->setLayout( layout );
+    form->setGeometry(0, 0, 64*2, 64);
+    scene->addItem( form );
+
+    m_frame = new Plasma::FrameSvg( this );
+    m_frame->setImagePath( "dialogs/background" );
+    m_frame->setCacheAllRenderedFrames( true );
+    m_frame->setEnabledBorders( Plasma::FrameSvg::AllBorders );
+    qreal left, top, right, bottom;
+    m_frame->getMargins( left, top, right, bottom );
+    qreal width = form->size().width() + left + right;
+    qreal height = form->size().height() + top + bottom;
+    m_frame->resizeFrame( QSizeF( width, height ) );
+    Plasma::WindowEffects::enableBlurBehind( winId(), true, m_frame->mask() );
+    form->setPos( left, top );
+    scene->setSceneRect( QRectF( QPointF( 0, 0 ), QSizeF( width, height ) ) );
+    setScene( scene );
+    }
+
+void DesktopButtonsView::windowInputMouseEvent( QMouseEvent* e )
+    {
+    if( e->type() == QEvent::MouseMove )
+        {
+        mouseMoveEvent( e );
+        }
+    else if( e->type() == QEvent::MouseButtonPress )
+        {
+        mousePressEvent( e );
+        }
+    else if( e->type() == QEvent::MouseButtonDblClick )
+        {
+        mouseDoubleClickEvent( e );
+        }
+    else if( e->type() == QEvent::MouseButtonRelease )
+        {
+        mouseReleaseEvent( e );
+        }
+    }
+
+void DesktopButtonsView::setAddDesktopEnabled( bool enable )
+    {
+    m_addDesktopButton->setEnabled( enable );
+    }
+
+void DesktopButtonsView::setRemoveDesktopEnabled( bool enable )
+    {
+    m_removeDesktopButton->setEnabled( enable );
+    }
+
+void DesktopButtonsView::drawBackground( QPainter* painter, const QRectF& rect )
+    {
+    Q_UNUSED( rect )
+    painter->setRenderHint( QPainter::Antialiasing );
+    m_frame->paintFrame( painter );
+    }
+
 } // namespace
 
 #include "desktopgrid.moc"
