@@ -34,6 +34,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QFile>
 
 
+#define DEBUG_GLRENDERTARGET 0
 
 #define MAKE_GL_VERSION(major, minor, release)  ( ((major) << 16) | ((minor) << 8) | (release) )
 
@@ -61,7 +62,7 @@ void initGLX()
     glXVersion = MAKE_GL_VERSION( major, minor, 0 );
     // Get list of supported GLX extensions
     glxExtensions = QString((const char*)glXQueryExtensionsString(
-        display(), DefaultScreen( display()))).split(" ");
+        display(), DefaultScreen( display()))).split(' ');
 
     glxResolveFunctions();
     }
@@ -74,7 +75,7 @@ void initGL()
     glVersion = MAKE_GL_VERSION(glversioninfo[0].toInt(), glversioninfo[1].toInt(),
                                     glversioninfo.count() > 2 ? glversioninfo[2].toInt() : 0);
     // Get list of supported OpenGL extensions
-    glExtensions = QString((const char*)glGetString(GL_EXTENSIONS)).split(" ");
+    glExtensions = QString((const char*)glGetString(GL_EXTENSIONS)).split(' ');
 
     // handle OpenGL extensions functions
     glResolveFunctions();
@@ -99,12 +100,27 @@ bool hasGLExtension(const QString& extension)
     return glExtensions.contains(extension) || glxExtensions.contains(extension);
     }
 
+static QString formatGLError( GLenum err )
+    {
+    switch ( err )
+        {
+        case GL_NO_ERROR:          return "GL_NO_ERROR";
+        case GL_INVALID_ENUM:      return "GL_INVALID_ENUM";
+        case GL_INVALID_VALUE:     return "GL_INVALID_VALUE";
+        case GL_INVALID_OPERATION: return "GL_INVALID_OPERATION";
+        case GL_STACK_OVERFLOW:    return "GL_STACK_OVERFLOW";
+        case GL_STACK_UNDERFLOW:   return "GL_STACK_UNDERFLOW";
+        case GL_OUT_OF_MEMORY:     return "GL_OUT_OF_MEMORY";
+        default: return QString( "0x" ) + QString::number( err, 16 );
+        }
+    }
+
 bool checkGLError( const char* txt )
     {
     GLenum err = glGetError();
     if( err != GL_NO_ERROR )
         {
-        kWarning(1212) << "GL error (" << txt << "): 0x" << QString::number( err, 16 ) ;
+        kWarning(1212) << "GL error (" << txt << "): " << formatGLError( err );
         return true;
         }
     return false;
@@ -358,11 +374,12 @@ GLTexture::GLTexture( int width, int height )
         mTarget = GL_TEXTURE_2D;
         mScale.setWidth( 1.0 / width);
         mScale.setHeight( 1.0 / height);
+        mSize = QSize( width, height );
         can_use_mipmaps = true;
 
         glGenTextures( 1, &mTexture );
         bind();
-        glTexImage2D( mTarget, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
+        glTexImage2D( mTarget, 0, GL_RGBA8, width, height, 0, GL_BGRA, GL_UNSIGNED_BYTE, 0);
         unbind();
         }
     }
@@ -439,8 +456,8 @@ bool GLTexture::load( const QImage& image, GLenum target )
     if( isNull())
         glGenTextures( 1, &mTexture );
     bind();
-    glTexImage2D( mTarget, 0, GL_RGBA, img.width(), img.height(), 0,
-        GL_RGBA, GL_UNSIGNED_BYTE, img.bits());
+    glTexImage2D( mTarget, 0, GL_RGBA8, img.width(), img.height(), 0,
+        GL_BGRA, GL_UNSIGNED_BYTE, img.bits());
     unbind();
     return true;
     }
@@ -647,27 +664,69 @@ void GLTexture::enableFilter()
         }
     }
 
-QImage GLTexture::convertToGLFormat( const QImage& img ) const
-    {
-    // This method has been copied from Qt's QGLWidget::convertToGLFormat()
-    QImage res = img.convertToFormat(QImage::Format_ARGB32);
-    res = res.mirrored();
+static void convertToGLFormatHelper(QImage &dst, const QImage &img, GLenum texture_format)
+    { // Copied from Qt
+    Q_ASSERT(dst.size() == img.size());
+    Q_ASSERT(dst.depth() == 32);
+    Q_ASSERT(img.depth() == 32);
 
-    if (QSysInfo::ByteOrder == QSysInfo::BigEndian) {
-        // Qt has ARGB; OpenGL wants RGBA
-        for (int i=0; i < res.height(); i++) {
-            uint *p = (uint*)res.scanLine(i);
-            uint *end = p + res.width();
-            while (p < end) {
-                *p = (*p << 8) | ((*p >> 24) & 0xFF);
-                p++;
+    const int width = img.width();
+    const int height = img.height();
+    const uint *p = (const uint*) img.scanLine(img.height() - 1);
+    uint *q = (uint*) dst.scanLine(0);
+
+    if (texture_format == GL_BGRA) {
+        if (QSysInfo::ByteOrder == QSysInfo::BigEndian) {
+            // mirror + swizzle
+            for (int i=0; i < height; ++i) {
+                const uint *end = p + width;
+                while (p < end) {
+                    *q = ((*p << 24) & 0xff000000)
+                         | ((*p >> 24) & 0x000000ff)
+                         | ((*p << 8) & 0x00ff0000)
+                         | ((*p >> 8) & 0x0000ff00);
+                    p++;
+                    q++;
+                }
+                p -= 2 * width;
+            }
+        } else {
+            const uint bytesPerLine = img.bytesPerLine();
+            for (int i=0; i < height; ++i) {
+                memcpy(q, p, bytesPerLine);
+                q += width;
+                p -= width;
+            }
+        }
+    } else {
+        if (QSysInfo::ByteOrder == QSysInfo::BigEndian) {
+            for (int i=0; i < height; ++i) {
+                const uint *end = p + width;
+                while (p < end) {
+                    *q = (*p << 8) | ((*p >> 24) & 0xFF);
+                    p++;
+                    q++;
+                }
+                p -= 2 * width;
+            }
+        } else {
+            for (int i=0; i < height; ++i) {
+                const uint *end = p + width;
+                while (p < end) {
+                    *q = ((*p << 16) & 0xff0000) | ((*p >> 16) & 0xff) | (*p & 0xff00ff00);
+                    p++;
+                    q++;
+                }
+                p -= 2 * width;
             }
         }
     }
-    else {
-        // Qt has ARGB; OpenGL wants ABGR (i.e. RGBA backwards)
-        res = res.rgbSwapped();
-    }
+}
+
+QImage GLTexture::convertToGLFormat( const QImage& img ) const
+    { // Copied from Qt's QGLWidget::convertToGLFormat()
+    QImage res(img.size(), QImage::Format_ARGB32);
+    convertToGLFormatHelper(res, img.convertToFormat(QImage::Format_ARGB32), GL_BGRA);
     return res;
     }
 
@@ -690,20 +749,15 @@ void GLShader::initStatic()
 GLShader::GLShader(const QString& vertexfile, const QString& fragmentfile)
     {
     mValid = false;
-    mVariableLocations = 0;
     mProgram = 0;
+    mTextureWidth = -1.0f;
+    mTextureHeight = -1.0f;
 
     loadFromFiles(vertexfile, fragmentfile);
     }
 
 GLShader::~GLShader()
 {
-    if(mVariableLocations)
-    {
-        mVariableLocations->clear();
-        delete mVariableLocations;
-    }
-
     if(mProgram)
     {
         glDeleteProgram(mProgram);
@@ -834,8 +888,6 @@ bool GLShader::load(const QString& vertexsource, const QString& fragmentsource)
         kDebug(1212) << "Shader linking log:"<< log;
     delete[] log;
 
-    mVariableLocations = new QHash<QString, int>;
-
     mValid = true;
     return true;
     }
@@ -850,21 +902,13 @@ void GLShader::unbind()
     glUseProgram(0);
     }
 
-int GLShader::uniformLocation(const QString& name)
+int GLShader::uniformLocation(const char* name)
     {
-    if(!mVariableLocations)
-        {
-        return -1;
-        }
-    if(!mVariableLocations->contains(name))
-        {
-        int location = glGetUniformLocation(mProgram, name.toLatin1().data());
-        mVariableLocations->insert(name, location);
-        }
-    return mVariableLocations->value(name);
+    int location = glGetUniformLocation(mProgram, name);
+    return location;
     }
 
-bool GLShader::setUniform(const QString& name, float value)
+bool GLShader::setUniform(const char* name, float value)
     {
     int location = uniformLocation(name);
     if(location >= 0)
@@ -874,7 +918,7 @@ bool GLShader::setUniform(const QString& name, float value)
     return (location >= 0);
     }
 
-bool GLShader::setUniform(const QString& name, int value)
+bool GLShader::setUniform(const char* name, int value)
     {
     int location = uniformLocation(name);
     if(location >= 0)
@@ -884,21 +928,13 @@ bool GLShader::setUniform(const QString& name, int value)
     return (location >= 0);
     }
 
-int GLShader::attributeLocation(const QString& name)
+int GLShader::attributeLocation(const char* name)
     {
-    if(!mVariableLocations)
-        {
-        return -1;
-        }
-    if(!mVariableLocations->contains(name))
-        {
-        int location = glGetAttribLocation(mProgram, name.toLatin1().data());
-        mVariableLocations->insert(name, location);
-        }
-    return mVariableLocations->value(name);
+    int location = glGetAttribLocation(mProgram, name);
+    return location;
     }
 
-bool GLShader::setAttribute(const QString& name, float value)
+bool GLShader::setAttribute(const char* name, float value)
     {
     int location = attributeLocation(name);
     if(location >= 0)
@@ -908,6 +944,25 @@ bool GLShader::setAttribute(const QString& name, float value)
     return (location >= 0);
     }
 
+void GLShader::setTextureHeight(float height)
+    {
+    mTextureHeight = height;
+    }
+
+void GLShader::setTextureWidth(float width)
+    {
+    mTextureWidth = width;
+    }
+
+float GLShader::textureHeight()
+    {
+    return mTextureHeight;
+    }
+
+float GLShader::textureWidth()
+    {
+    return mTextureWidth;
+    }
 
 
 /***  GLRenderTarget  ***/
@@ -916,6 +971,26 @@ bool GLRenderTarget::mSupported = false;
 void GLRenderTarget::initStatic()
     {
     mSupported = hasGLExtension("GL_EXT_framebuffer_object") && glFramebufferTexture2D;
+    if( mSupported )
+        {
+        // some drivers claim to support the extension, but fail to return a complete FBO (see Bug 240956)
+        // generate a FBO and test if it is valid. If it isn't valid there's no need to claim that FBO's are supported
+        int w = displayWidth();
+        int h = displayHeight();
+        if ( !GLTexture::NPOTTextureSupported() )
+            {
+            w = nearestPowerOfTwo( w );
+            h = nearestPowerOfTwo( h );
+            }
+        GLTexture* tex = new GLTexture( w, h );
+        tex->setFilter( GL_LINEAR );
+        tex->setWrapMode( GL_CLAMP_TO_EDGE );
+        GLRenderTarget* target = new GLRenderTarget( tex );
+        if( !target->valid() )
+            mSupported = false;
+        delete target;
+        delete tex;
+        }
     }
 
 GLRenderTarget::GLRenderTarget(GLTexture* color)
@@ -970,22 +1045,95 @@ bool GLRenderTarget::disable()
     return true;
     }
 
+static QString formatFramebufferStatus( GLenum status )
+    {
+    switch( status )
+        {
+        case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:
+            // An attachment is the wrong type / is invalid / has 0 width or height
+            return "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT";
+        case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT:
+            // There are no images attached to the framebuffer
+            return "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT";
+        case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT:
+            // Not all attached images have the same width and height
+            return "GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT";
+        case GL_FRAMEBUFFER_UNSUPPORTED_EXT:
+            // A format or the combination of formats of the attachments is unsupported
+            return "GL_FRAMEBUFFER_UNSUPPORTED";
+        case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT:
+            // The color attachments don't have the same format
+            return "GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT";
+        case GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT:
+            // The attachments don't have the same number of samples
+            return "GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE";
+        case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT:
+            // The draw buffer is missing
+            return "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER";
+        case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT:
+            // The read buffer is missing
+            return "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER";
+        default:
+            return "Unknown (0x" + QString::number(status, 16) + ")";
+        }
+    }
+
 void GLRenderTarget::initFBO()
     {
+#if DEBUG_GLRENDERTARGET
+    GLenum err = glGetError();
+    if( err != GL_NO_ERROR )
+        kError(1212) << "Error status when entering GLRenderTarget::initFBO: " << formatGLError( err );
+#endif
+
     glGenFramebuffers(1, &mFramebuffer);
+
+#if DEBUG_GLRENDERTARGET
+    if( (err = glGetError()) != GL_NO_ERROR )
+        {
+        kError(1212) << "glGenFramebuffers failed: " << formatGLError( err );
+        return;
+        }
+#endif
+
     glBindFramebuffer(GL_FRAMEBUFFER_EXT, mFramebuffer);
 
+#if DEBUG_GLRENDERTARGET
+    if( (err = glGetError()) != GL_NO_ERROR )
+        {
+        kError(1212) << "glBindFramebuffer failed: " << formatGLError( err );
+        glDeleteFramebuffers(1, &mFramebuffer);
+        return;
+        }
+#endif
+
     glFramebufferTexture2D(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT,
                            mTexture->target(), mTexture->texture(), 0);
 
-    GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER_EXT);
-    if(status != GL_FRAMEBUFFER_COMPLETE_EXT)
+#if DEBUG_GLRENDERTARGET
+    if( (err = glGetError()) != GL_NO_ERROR )
         {
-        kError(1212) << "Invalid fb status: " << status << endl;
+        kError(1212) << "glFramebufferTexture2D failed: " << formatGLError( err );
+        glBindFramebuffer(GL_FRAMEBUFFER_EXT, 0);
+        glDeleteFramebuffers(1, &mFramebuffer);
+        return;
         }
+#endif
+
+    const GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER_EXT);
 
     glBindFramebuffer(GL_FRAMEBUFFER_EXT, 0);
 
+    if( status != GL_FRAMEBUFFER_COMPLETE_EXT )
+        { // We have an incomplete framebuffer, consider it invalid
+        if (status == 0)
+            kError(1212) << "glCheckFramebufferStatus failed: " << formatGLError( glGetError() );
+        else
+            kError(1212) << "Invalid framebuffer status: " << formatFramebufferStatus( status );
+        glDeleteFramebuffers(1, &mFramebuffer);
+        return;
+        }
+
     mValid = true;
     }
 
