@@ -124,6 +124,10 @@ unsigned long Options::updateSettings()
     shadeHover = config.readEntry("ShadeHover", false);
     shadeHoverInterval = config.readEntry("ShadeHoverInterval", 250 );
 
+    tilingOn = config.readEntry( "TilingOn", false );
+    tilingLayout = static_cast<TilingLayoutFactory::Layouts>(config.readEntry( "TilingDefaultLayout", 0 ));
+    tilingRaisePolicy = config.readEntry( "TilingRaisePolicy", 0 );
+
     // important: autoRaise implies ClickRaise
     clickRaise = autoRaise || config.readEntry("ClickRaise", true);
 
@@ -261,7 +265,7 @@ void Options::reloadCompositingSettings()
         glMode = GLFallback;
     glDirect = config.readEntry("GLDirect", prefs.enableDirectRendering() );
     glVSync = config.readEntry("GLVSync", prefs.enableVSync() );
-    smoothScale = qBound( -1, config.readEntry( "GLTextureFilter", -1 ), 2 );
+    glSmoothScale = qBound( -1, config.readEntry( "GLTextureFilter", -1 ), 2 );
     glStrictBinding = config.readEntry( "GLStrictBinding", prefs.strictBinding());
 
     xrenderSmoothScale = config.readEntry("XRenderSmoothScale", false );
@@ -278,9 +282,6 @@ void Options::reloadCompositingSettings()
 
     unredirectFullscreen = config.readEntry( "UnredirectFullscreen", true );
     animationSpeed = qBound( 0, config.readEntry( "AnimationSpeed", 3 ), 6 );
-
-    if( !disableCompositingChecks && !prefs.validateSetup( compositingMode ))
-        useCompositing = false;
     }
 
 
@@ -289,6 +290,8 @@ ElectricBorderAction Options::electricBorderAction( const QString& name )
     QString lowerName = name.toLower();
     if( lowerName == "dashboard" ) return ElectricActionDashboard;
     else if( lowerName == "showdesktop" ) return ElectricActionShowDesktop;
+    else if( lowerName == "lockscreen" ) return ElectricActionLockScreen;
+    else if( lowerName == "preventscreenlocking" ) return ElectricActionLockScreen;
     return ElectricActionNone;
     }
 
