@@ -30,6 +30,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "client.h"
 #include "workspace.h"
 #include "effects.h"
+#include "tile.h"
+#include "tilinglayout.h"
+
+#include "kactivityinfo.h"
 
 #include <fixx11h.h>
 #include <QPushButton>
@@ -112,7 +116,7 @@ QMenu* Workspace::clientPopup()
         action->setIcon( KIcon( "preferences-system-windows-actions" ) );
         action->setData( Options::WindowRulesOp );
 
-        action = advanced_popup->addAction( i18n("&Special Application Settings...") );
+        action = advanced_popup->addAction( i18n("S&pecial Application Settings...") );
         action->setIcon( KIcon( "preferences-system-windows-actions" ) );
         action->setData( Options::ApplicationRulesOp );
 
@@ -121,12 +125,14 @@ QMenu* Workspace::clientPopup()
             trans_popup = new QMenu( popup );
             trans_popup->setFont(KGlobalSettings::menuFont());
             connect( trans_popup, SIGNAL( triggered(QAction*) ), this, SLOT( setPopupClientOpacity(QAction*)));
+            trans_popup_group = new QActionGroup(trans_popup);
             const int levels[] = { 100, 90, 75, 50, 25, 10 };
             for( unsigned int i = 0;
                  i < sizeof( levels ) / sizeof( levels[ 0 ] );
                  ++i )
                 {
                 action = trans_popup->addAction( QString::number( levels[ i ] ) + "%" );
+                action->setActionGroup( trans_popup_group );
                 action->setCheckable( true );
                 action->setData( levels[ i ] );
                 }
@@ -172,13 +178,13 @@ QMenu* Workspace::clientPopup()
         // Actions for window tabbing
         if( decorationSupportsClientGrouping() )
             {
-            mRemoveTabGroup = popup->addAction( i18n("Remove &from group") );
+            mRemoveTabGroup = popup->addAction( i18n("Remove &From Group") );
             kaction = qobject_cast<KAction*>( keys->action("Remove TabGroup") );
             if( kaction!=0 )
                 mRemoveTabGroup->setShortcut( kaction->globalShortcut().primary() );
             mRemoveTabGroup->setData( Options::RemoveClientFromGroupOp );
 
-            mCloseGroup = popup->addAction( i18n("Close entire &group") );
+            mCloseGroup = popup->addAction( i18n("Close Entire &Group") );
             mCloseGroup->setIcon( KIcon( "window-close" ) );
             kaction = qobject_cast<KAction*>( keys->action("Close TabGroup") );
             if( kaction!=0 )
@@ -188,6 +194,22 @@ QMenu* Workspace::clientPopup()
             popup->addSeparator();
             }
 
+        // create it anyway
+        mTilingStateOpAction = popup->addAction( i18nc("When in tiling mode, toggle's the window's floating/tiled state", "&Float Window") );
+        // then hide it
+        mTilingStateOpAction->setVisible(false);
+        // actions for window tiling
+        if( tilingEnabled() )
+            {
+            kaction = qobject_cast<KAction*>( keys->action("Toggle Floating") );
+            mTilingStateOpAction->setCheckable( true );
+            mTilingStateOpAction->setData( Options::ToggleClientTiledStateOp );
+            if( kaction!=0 )
+                mTilingStateOpAction->setShortcut( kaction->globalShortcut().primary() );
+            }
+
+        popup->addSeparator();
+
         action = popup->addMenu( advanced_popup );
         action->setText( i18n("Ad&vanced") );
 
@@ -217,6 +239,7 @@ void Workspace::discardPopup()
     delete popup;
     popup = NULL; 
     desk_popup = NULL;
+    activity_popup = NULL;
     switch_to_tab_popup = NULL;
     add_tabs_popup = NULL;
     }
@@ -248,6 +271,17 @@ void Workspace::clientPopupAboutToShow()
         {
         initDesktopPopup();
         }
+    QStringList act = activityController_.availableActivities();
+    kDebug() << "activities:" << act.size();
+    if ( act.size() < 2 )
+        {
+        delete activity_popup;
+        activity_popup = 0;
+        }
+    else
+        {
+        initActivityPopup();
+        }
 
     mResizeOpAction->setEnabled( active_popup_client->isResizable() );
     mMoveOpAction->setEnabled( active_popup_client->isMovableAcrossScreens() );
@@ -264,6 +298,18 @@ void Workspace::clientPopupAboutToShow()
     mMinimizeOpAction->setEnabled( active_popup_client->isMinimizable() );
     mCloseOpAction->setEnabled( active_popup_client->isCloseable() );
 
+    if( tilingEnabled() )
+        {
+        int desktop = active_popup_client->desktop();
+        if( tilingLayouts.value(desktop) )
+            {
+            Tile *t = tilingLayouts[desktop]->findTile(active_popup_client);
+            if( t )
+                mTilingStateOpAction->setChecked( t->floating() );
+            }
+        }
+    mTilingStateOpAction->setVisible( tilingEnabled() );
+
     delete switch_to_tab_popup;
     switch_to_tab_popup = 0;
     delete add_tabs_popup;
@@ -305,7 +351,7 @@ void Workspace::initSwitchToTab()
 
     QAction* action = switch_to_tab_popup->menuAction();
     popup->insertAction( mRemoveTabGroup, action );
-    action->setText( i18n("Switch to window tab") );
+    action->setText( i18n("Switch to Window Tab") );
     }
 
 void Workspace::slotSwitchToTab( QAction* action )
@@ -346,9 +392,9 @@ void Workspace::switchToTabPopupAboutToShow()
     if( !switch_to_tab_popup )
         return;
     switch_to_tab_popup->clear();
-    QAction* action = switch_to_tab_popup->addAction( i18n( "To the left" ));
+    QAction* action = switch_to_tab_popup->addAction( i18n( "To the Left" ));
     action->setData( 0 );
-    action = switch_to_tab_popup->addAction( i18n( "To the right" ));
+    action = switch_to_tab_popup->addAction( i18n( "To the Right" ));
     action->setData( 1 );
     switch_to_tab_popup->addSeparator();
     int index = 2;
@@ -377,7 +423,7 @@ void Workspace::initAddToTabGroup()
 
     QAction* action = add_tabs_popup->menuAction();
     popup->insertAction( mRemoveTabGroup, action );
-    action->setText( i18n("Move window to group") );
+    action->setText( i18n("Move Window to Group") );
     }
 
 void Workspace::slotAddToTabGroup( QAction* action )
@@ -423,6 +469,29 @@ void Workspace::initDesktopPopup()
     action->setText( i18n("To &Desktop") );
     }
 
+/*!
+  Creates activity popup.
+  I'm going with checkable ones instead of "copy to" and "move to" menus; I *think* it's an easier way.
+  Oh, and an 'all' option too of course
+ */
+void Workspace::initActivityPopup()
+    {
+    if (activity_popup)
+        return;
+
+    activity_popup = new QMenu( popup );
+    activity_popup->setFont(KGlobalSettings::menuFont());
+    connect( activity_popup, SIGNAL( triggered(QAction*) ),
+             this, SLOT( slotToggleOnActivity(QAction*) ) );
+    connect( activity_popup, SIGNAL( aboutToShow() ),
+             this, SLOT( activityPopupAboutToShow() ) );
+
+    QAction *action = activity_popup->menuAction();
+    // set it as the first item
+    popup->insertAction( trans_popup ? trans_popup->menuAction() : mMoveOpAction, action);
+    action->setText( i18n("Ac&tivities") ); //FIXME is that a good string?
+    }
+
 /*!
   Adjusts the desktop popup to the current values and the location of
   the popup client.
@@ -457,6 +526,37 @@ void Workspace::desktopPopupAboutToShow()
         }
     }
 
+/*!
+  Adjusts the activity popup to the current values and the location of
+  the popup client.
+ */
+void Workspace::activityPopupAboutToShow()
+    {
+    if ( !activity_popup )
+        return;
+
+    activity_popup->clear();
+    QAction *action = activity_popup->addAction( i18n("&All Activities") );
+    action->setData( QString() );
+    action->setCheckable( true );
+
+    if ( active_popup_client && active_popup_client->isOnAllActivities() )
+        action->setChecked( true );
+    activity_popup->addSeparator();
+
+    foreach (const QString &activity, activityController_.availableActivities()) {
+        QString name = KActivityInfo::name(activity);
+        name.replace('&', "&&");
+        action = activity_popup->addAction( name );
+        action->setData( activity );
+        action->setCheckable( true );
+
+        if ( active_popup_client &&
+             !active_popup_client->isOnAllActivities() && active_popup_client->isOnActivity(activity) )
+            action->setChecked( true );
+        }
+    }
+
 void Workspace::closeActivePopup()
     {
     if( active_popup )
@@ -666,6 +766,16 @@ void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
     if ( !c )
         return;
 
+    // Allows us to float a window when it is maximized, if it is tiled.
+    if( tilingEnabled()
+        && (    op == Options::MaximizeOp
+             || op == Options::HMaximizeOp
+             || op == Options::VMaximizeOp
+             || op == Options::RestoreOp ) )
+        {
+        notifyTilingWindowMaximized( c, op );
+        }
+
     if (op == Options::MoveOp || op == Options::UnrestrictedMoveOp )
         QCursor::setPos( c->geometry().center() );
     if (op == Options::ResizeOp || op == Options::UnrestrictedResizeOp )
@@ -756,26 +866,40 @@ void Workspace::performWindowOperation( Client* c, Options::WindowOperation op )
             break;
         case Options::MoveClientInGroupLeftOp:
             {
-            int c_id = c->clientGroup()->indexOfClient( c );
-            int size = c->clientGroup()->clients().count();
-            if( c_id > 0 )
-                c->clientGroup()->setVisible( c_id - 1 );
-            else
-                c->clientGroup()->setVisible( size - 1 );
+            if( c->clientGroup() )
+                {
+                int c_id = c->clientGroup()->indexOfClient( c );
+                int size = c->clientGroup()->clients().count();
+                if( c_id > 0 )
+                    c->clientGroup()->setVisible( c_id - 1 );
+                else
+                    c->clientGroup()->setVisible( size - 1 );
+		}
             break;
             }
         case Options::MoveClientInGroupRightOp:
             {
-            int c_id = c->clientGroup()->indexOfClient( c );
-            int size = c->clientGroup()->clients().count();
-            if( c_id < size - 1 )
-                c->clientGroup()->setVisible( c_id + 1 );
-            else
-                c->clientGroup()->setVisible( 0 );
+            if( c->clientGroup() )
+                {
+                int c_id = c->clientGroup()->indexOfClient( c );
+                int size = c->clientGroup()->clients().count();
+                if( c_id < size - 1 )
+                    c->clientGroup()->setVisible( c_id + 1 );
+                else
+                    c->clientGroup()->setVisible( 0 );
+		}
             break;
             }
         case Options::CloseClientGroupOp:
             c->clientGroup()->closeAll();
+        case Options::ToggleClientTiledStateOp:
+            {
+            int desktop = c->desktop();
+            if( tilingLayouts.value( desktop ) )
+                {
+                tilingLayouts[desktop]->toggleFloatTile( c );
+                }
+            }
         }
     }
 
@@ -964,28 +1088,22 @@ bool Client::performMouseCommand( Options::MouseCommand command, const QPoint &g
             break;
         case Options::MouseLeftGroupWindow:
             {
-            if( clientGroup() ) 
-                {
-                int c_id = clientGroup()->indexOfClient( this );
-                int size = clientGroup()->clients().count();
-                if( c_id > 0 )
-                    clientGroup()->setVisible( c_id - 1 );
-                else
-                    clientGroup()->setVisible( size - 1 );
-	        }
+            int c_id = clientGroup()->indexOfClient( this );
+            int size = clientGroup()->clients().count();
+            if( c_id > 0 )
+                clientGroup()->setVisible( c_id - 1 );
+            else
+                clientGroup()->setVisible( size - 1 );
             }
             break;
         case Options::MouseRightGroupWindow:
-	    {
-            if( clientGroup() ) 
-                {
-                int c_id = clientGroup()->indexOfClient( this );
-                int size = clientGroup()->clients().count();
-                if( c_id < size - 1 )
-                    clientGroup()->setVisible( c_id + 1 );
-                else
-                    clientGroup()->setVisible( 0 );
-	        }
+            {
+            int c_id = clientGroup()->indexOfClient( this );
+            int size = clientGroup()->clients().count();
+            if( c_id < size - 1 )
+                clientGroup()->setVisible( c_id + 1 );
+            else
+                clientGroup()->setVisible( 0 );
             }
             break;
         case Options::MouseClose:
@@ -1461,6 +1579,26 @@ void Workspace::slotSendToDesktop( QAction *action )
 
     }
 
+/*!
+  Toggles whether the popup client is on the \a activity
+
+  Internal slot for the window operation menu
+ */
+void Workspace::slotToggleOnActivity( QAction *action )
+    {
+      QString activity = action->data().toString();
+    if ( !active_popup_client )
+        return;
+    if ( activity.isEmpty() )
+        { // the 'on_all_activities' menu entry
+        active_popup_client->setOnAllActivities( !active_popup_client->isOnAllActivities());
+        return;
+        }
+
+    toggleClientOnActivity( active_popup_client, activity, false );
+
+    }
+
 /*!
   Switches to the nearest window in given direction
  */
