@@ -155,14 +155,14 @@ struct KSysGuardProcessListPrivate {
         mNeedToExpandInit = false;
         mNumItemsSelected = -1;
         mResortCountDown = 2; //The items added initially will be already sorted, but without CPU info.  On the second refresh we will have CPU usage, so /then/ we can resort
-        renice = new KAction(i18np("Renice Process...", "Renice Processes...", 1), q);
+        renice = new KAction(i18np("Set Priority...", "Set Priority...", 1), q);
         renice->setShortcut(Qt::Key_F8);
         selectParent = new KAction(i18n("Jump to Parent Process"), q);
 
         selectTracer = new KAction(i18n("Jump to Process Debugging This One"), q);
         window = new KAction(i18n("Show Application Window"), q);
         resume = new KAction(i18n("Resume Stopped Process"), q);
-        kill = new KAction(i18np("Kill Process...", "Kill Processes...", 1), q);
+        kill = new KAction(i18np("End Process...", "End Processes...", 1), q);
         kill->setIcon(KIcon("process-stop"));
         kill->setShortcut(Qt::Key_Delete);
 
@@ -276,14 +276,16 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 
     d->mFilterModel.setFilterKeyColumn(-1);
 
-    /*  Hide the vm size column by default since it's not very useful */
+    /*  Hide various columns by default, to reduce information overload */
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingVmSize);
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingNiceness);
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingTty);
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingCommand);
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingPid);
+    d->mUi->treeView->header()->hideSection(ProcessModel::HeadingCPUTime);
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingIoRead);
     d->mUi->treeView->header()->hideSection(ProcessModel::HeadingIoWrite);
+    d->mUi->treeView->header()->hideSection(ProcessModel::HeadingXMemory);
     // NOTE!  After this is all setup, the settings for the header are restored
     // from the user's last run.  (in restoreHeaderState)
     // So making changes here only affects the default settings.  To
@@ -327,6 +329,7 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
     retranslateUi();
 
     d->mUi->btnKillProcess->setIcon(KIcon("process-stop"));
+    d->mUi->btnKillProcess->setToolTip(i18n("<qt>End the selected process. Warning - you may lose unsaved work.<br>Right click on a process to send other signals.<br>See What's This for technical information.<br>To target a specific window to kill, press ctrl+alt+esc at any time."));
 }
 
 KSysGuardProcessList::~KSysGuardProcessList()
@@ -404,8 +407,8 @@ void KSysGuardProcessList::selectionChanged()
     d->mNumItemsSelected = numSelected;
     d->mUi->btnKillProcess->setEnabled( numSelected != 0 );
 
-    d->renice->setText(i18np("Renice Process...", "Renice Processes...", numSelected));
-    d->kill->setText(i18np("Kill Process", "Kill Processes", numSelected));
+    d->renice->setText(i18np("Set Priority...", "Set Priority...", numSelected));
+    d->kill->setText(i18ncp("Context menu", "End Process", "End Processes", numSelected));
 }
 void KSysGuardProcessList::showProcessContextMenu(const QModelIndex &index) {
     if(!index.isValid()) return;
@@ -592,6 +595,10 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
         int num_headings = d->mFilterModel.columnCount();
         for(int i = 0; i < num_headings; ++i) {
             if(d->mUi->treeView->header()->isSectionHidden(i)) {
+#ifndef HAVE_XRES
+                if(i == ProcessModel::HeadingXMemory)
+                    continue;
+#endif
                 action = new QAction(menu);
                 action->setText(i18n("Show Column '%1'", d->mFilterModel.headerData(i, Qt::Horizontal, Qt::DisplayRole).toString()));
                 action->setData(i); //We set data to be negative (and minus 1) to hide a column, and positive to show a column
@@ -617,7 +624,7 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
                      d->mModel.ioInformation() == ProcessModel::SyscallsRate ||
                      d->mModel.ioInformation() == ProcessModel::ActualBytesRate;
 
-    if( index == ProcessModel::HeadingVmSize || index == ProcessModel::HeadingMemory ||  index == ProcessModel::HeadingSharedMemory || ( (index == ProcessModel::HeadingIoRead || index == ProcessModel::HeadingIoWrite) && d->mModel.ioInformation() != ProcessModel::Syscalls)) {
+    if( index == ProcessModel::HeadingVmSize || index == ProcessModel::HeadingMemory || index == ProcessModel::HeadingXMemory || index == ProcessModel::HeadingSharedMemory || ( (index == ProcessModel::HeadingIoRead || index == ProcessModel::HeadingIoWrite) && d->mModel.ioInformation() != ProcessModel::Syscalls)) {
         //If the user right clicks on a column that contains a memory size, show a toggle option for displaying
         //the memory in different units.  e.g.  "2000 k" or "2 m"
         menu->addSeparator()->setText(i18n("Display Units"));
@@ -799,6 +806,7 @@ void KSysGuardProcessList::showColumnContextMenu(const QPoint &point){
         d->mUi->treeView->hideColumn(-1-i);
     else {
         d->mUi->treeView->showColumn(i);
+        updateList();
         d->mUi->treeView->resizeColumnToContents(i);
         d->mUi->treeView->resizeColumnToContents(d->mFilterModel.columnCount());
     }
@@ -901,9 +909,11 @@ void KSysGuardProcessList::retranslateUi()
 void KSysGuardProcessList::updateList()
 {
     if(isVisible()) {
-        KSysGuard::Processes::UpdateFlags updateFlags = 0;
+        KSysGuard::Processes::UpdateFlags updateFlags = KSysGuard::Processes::StandardInformation;
         if(!d->mUi->treeView->isColumnHidden(ProcessModel::HeadingIoRead) || !d->mUi->treeView->isColumnHidden(ProcessModel::HeadingIoWrite))
-            updateFlags = KSysGuard::Processes::IOStatistics;
+            updateFlags |= KSysGuard::Processes::IOStatistics;
+        if(!d->mUi->treeView->isColumnHidden(ProcessModel::HeadingXMemory))
+            updateFlags |= KSysGuard::Processes::XMemory;
         d->mModel.update(d->mUpdateIntervalMSecs, updateFlags);
         if(d->mUpdateTimer)
             d->mUpdateTimer->start(d->mUpdateIntervalMSecs);
@@ -1231,15 +1241,15 @@ void KSysGuardProcessList::killSelectedProcesses()
     else
     {
         int count = selectedAsStrings.count();
-        QString  msg = i18np("Are you sure you want to kill this process?",
-                "Are you sure you want to kill these %1 processes?",
+        QString  msg = i18np("Are you sure you want to end this process?  Any unsaved work may be lost.",
+                "Are you sure you want to end these %1 processes?  Any unsaved work may be lost",
                 count);
 
         int res = KMessageBox::warningContinueCancelList(this, msg, selectedAsStrings,
-                i18np("Kill process", "Kill %1 processes", count),
-                KGuiItem(i18n("Kill"), "process-stop"),
+                i18ncp("Dialog title", "End Process", "End %1 Processes", count),
+                KGuiItem(i18n("End"), "process-stop"),
                 KStandardGuiItem::cancel(),
-                "killconfirmation");
+                "endconfirmation");
         if (res != KMessageBox::Continue)
         {
             return;
