@@ -1,5 +1,5 @@
 /*
- *   Copyright 2008 Davide Bettio <davide.bettio@kdemail.net>
+ *   Copyright 2008,2010 Davide Bettio <davide.bettio@kdemail.net>
  *   Copyright 2009 John Layt <john@layt.net>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include <QtGui/QGraphicsSceneWheelEvent>
 #include <QtGui/QGraphicsLinearLayout>
 #include <QtGui/QGraphicsProxyWidget>
+#include <QtGui/QGraphicsView>
 #include <QtGui/QLabel>
 #include <QtGui/QMenu>
 #include <QtGui/QSpinBox>
@@ -46,14 +47,37 @@
 #include <Plasma/LineEdit>
 #include <Plasma/SpinBox>
 #include <Plasma/ToolButton>
+#include <Plasma/ToolTipManager>
 #include <Plasma/DataEngine>
 
+#include <kephal/screens.h>
+
+#include "wheelytoolbutton.h"
+
 namespace Plasma
 {
 
+static const int s_yearWidgetIndex = 3;
+
 class CalendarPrivate
 {
     public:
+        CalendarPrivate()
+            : back(0),
+              spacer0(0),
+              month(0),
+              yearSpinBox(0),
+              year(0),
+              spacer1(0),
+              forward(0),
+              calendarTable(0),
+              dateText(0),
+              jumpToday(0),
+              monthMenu(0),
+              weekSpinBox(0)
+        {
+        }
+
         ToolButton *back;
         Plasma::Label *spacer0;
         Plasma::ToolButton *month;
@@ -96,32 +120,42 @@ Calendar::~Calendar()
 
 void Calendar::init(CalendarTable *calendarTable)
 {
-    QGraphicsLinearLayout *m_layout = new QGraphicsLinearLayout(Qt::Vertical, this);
-    QGraphicsLinearLayout *m_hLayout = new QGraphicsLinearLayout(m_layout);
-    QGraphicsLinearLayout *m_layoutTools = new QGraphicsLinearLayout(m_layout);
+    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical, this);
+    QGraphicsLinearLayout *hLayout = new QGraphicsLinearLayout(layout);
+    QGraphicsLinearLayout *layoutTools = new QGraphicsLinearLayout(layout);
 
     d->calendarTable = calendarTable;
     d->calendarTable->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+    connect(d->calendarTable, SIGNAL(dateChanged(const QDate &)), this, SLOT(dateUpdated(const QDate &)));
+    connect(d->calendarTable, SIGNAL(dateHovered(const QDate &)), this, SIGNAL(dateHovered(const QDate &)));
+    connect(d->calendarTable, SIGNAL(dateSelected(const QDate &)), this, SLOT(showTip(const QDate &)));
+    connect(this, SIGNAL(dateHovered(const QDate &)), this, SLOT(showTip(const QDate &)));
 
     d->back = new Plasma::ToolButton(this);
     d->back->setText("<");
     d->back->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
     connect(d->back, SIGNAL(clicked()), this, SLOT(prevMonth()));
-    m_hLayout->addItem(d->back);
+    hLayout->addItem(d->back);
 
-    m_hLayout->addStretch();
+    hLayout->addStretch();
 
-    d->month = new Plasma::ToolButton(this);
+    d->month = new WheelyToolButton(this);
     d->month->setText(calendar()->monthName(calendar()->month(date()), calendar()->year(date())));
     d->month->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+    d->monthMenu = new QMenu();
+    d->month->nativeWidget()->setMenu(d->monthMenu);
     connect(d->month, SIGNAL(clicked()), this, SLOT(monthsPopup()));
-    m_hLayout->addItem(d->month);
+    connect(d->month, SIGNAL(wheelUp()), this, SLOT(prevMonth()));
+    connect(d->month, SIGNAL(wheelDown()), this, SLOT(nextMonth()));
+    hLayout->addItem(d->month);
 
-    d->year = new Plasma::ToolButton(this);
+    d->year = new WheelyToolButton(this);
     d->year->setText(calendar()->yearString(date()));
     d->year->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
+    connect(d->year, SIGNAL(wheelUp()), this, SLOT(prevYear()));
+    connect(d->year, SIGNAL(wheelDown()), this, SLOT(nextYear()));
     connect(d->year, SIGNAL(clicked()), this, SLOT(showYearSpinBox()));
-    m_hLayout->addItem(d->year);
+    hLayout->addItem(d->year);
 
     d->yearSpinBox = new Plasma::SpinBox(this);
     d->yearSpinBox->setRange(calendar()->year(calendar()->earliestValidDate()), calendar()->year(calendar()->latestValidDate()));
@@ -129,41 +163,36 @@ void Calendar::init(CalendarTable *calendarTable)
     d->yearSpinBox->hide();
     connect(d->yearSpinBox->nativeWidget(), SIGNAL(editingFinished()), this, SLOT(hideYearSpinBox()));
 
-    m_hLayout->addStretch();
+    hLayout->addStretch();
 
     d->forward = new Plasma::ToolButton(this);
     d->forward->setText(">");
     d->forward->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
     connect(d->forward, SIGNAL(clicked()), this, SLOT(nextMonth()));
-    m_hLayout->addItem(d->forward);
-
-    m_layout->addItem(m_hLayout);
-
-    m_layout->addItem(d->calendarTable);
-    connect(d->calendarTable, SIGNAL(dateChanged(const QDate &)), this, SLOT(dateUpdated(const QDate &)));
+    hLayout->addItem(d->forward);
 
     d->jumpToday = new Plasma::ToolButton(this);
     d->jumpToday->nativeWidget()->setIcon(KIcon("go-jump-today"));
     d->jumpToday->nativeWidget()->setMinimumWidth(25);
     connect(d->jumpToday, SIGNAL(clicked()), this, SLOT(goToToday()));
-    m_layoutTools->addItem(d->jumpToday);
-    m_layoutTools->addStretch();
+    layoutTools->addItem(d->jumpToday);
+    layoutTools->addStretch();
 
     d->dateText = new Plasma::LineEdit(this);
     d->dateText->setText(calendar()->formatDate(date(),  KLocale::ShortDate));
     connect(d->dateText->nativeWidget(), SIGNAL(returnPressed()), this, SLOT(manualDateChange()));
-    m_layoutTools->addItem(d->dateText);
-    m_layoutTools->addStretch();
+    layoutTools->addItem(d->dateText);
+    layoutTools->addStretch();
 
     d->weekSpinBox = new Plasma::SpinBox(this);
     d->weekSpinBox->setMinimum(1);
     d->weekSpinBox->setMaximum(calendar()->weeksInYear(date()));
     connect(d->weekSpinBox, SIGNAL(valueChanged(int)), this, SLOT(goToWeek(int)));
-    m_layoutTools->addItem(d->weekSpinBox);
+    layoutTools->addItem(d->weekSpinBox);
 
-    m_layout->addItem(m_layoutTools);
-
-    d->monthMenu = 0;
+    layout->addItem(hLayout);
+    layout->addItem(d->calendarTable);
+    layout->addItem(layoutTools);
 
     dateUpdated(date());
 }
@@ -173,24 +202,22 @@ CalendarTable *Calendar::calendarTable() const
     return d->calendarTable;
 }
 
-bool Calendar::setCalendar(const QString &newCalendarType)
+void Calendar::setCalendar(const QString &newCalendarType)
 {
-    bool ret = calendarTable()->setCalendar(newCalendarType);
+    calendarTable()->setCalendar(newCalendarType);
     d->weekSpinBox->setMaximum(calendar()->weeksInYear(date()));
     d->yearSpinBox->setRange(calendar()->year(calendar()->earliestValidDate()),
                              calendar()->year(calendar()->latestValidDate()));
     refreshWidgets();
-    return ret;
 }
 
-bool Calendar::setCalendar(const KCalendarSystem *newCalendar)
+void Calendar::setCalendar(const KCalendarSystem *newCalendar)
 {
-    bool ret = calendarTable()->setCalendar(newCalendar);
+    calendarTable()->setCalendar(newCalendar);
     d->weekSpinBox->setMaximum(calendar()->weeksInYear(date()));
     d->yearSpinBox->setRange(calendar()->year(calendar()->earliestValidDate()),
                              calendar()->year(calendar()->latestValidDate()));
     refreshWidgets();
-    return ret;
 }
 
 const KCalendarSystem *Calendar::calendar() const
@@ -198,18 +225,17 @@ const KCalendarSystem *Calendar::calendar() const
     return calendarTable()->calendar();
 }
 
-bool Calendar::setDate(const QDate &toDate)
+void Calendar::setDate(const QDate &toDate)
 {
-    bool ret = d->calendarTable->setDate(toDate);
+    d->calendarTable->setDate(toDate);
 
     //If set date failed force refresh of nav widgets to reset any user entry
     //If set date successful refresh will be triggered through signal/slot
-    if (!ret) {
+    if (d->calendarTable->date() != toDate) {
         refreshWidgets();
     }
-    d->weekSpinBox->setMaximum(calendar()->weeksInYear(date()));
 
-    return ret;
+    d->weekSpinBox->setMaximum(calendar()->weeksInYear(date()));
 }
 
 const QDate& Calendar::date() const
@@ -227,9 +253,9 @@ const Plasma::DataEngine *Calendar::dataEngine() const
     return calendarTable()->dataEngine();
 }
 
-bool Calendar::setDisplayHolidays(bool showHolidays)
+void Calendar::setDisplayHolidays(bool showHolidays)
 {
-    return calendarTable()->setDisplayHolidays(showHolidays);
+    calendarTable()->setDisplayHolidays(showHolidays);
 }
 
 bool Calendar::displayHolidays()
@@ -237,9 +263,9 @@ bool Calendar::displayHolidays()
     return calendarTable()->displayHolidays();
 }
 
-bool Calendar::setHolidaysRegion(const QString &region)
+void Calendar::setHolidaysRegion(const QString &region)
 {
-    return calendarTable()->setHolidaysRegion(region);
+    calendarTable()->setHolidaysRegion(region);
 }
 
 QString Calendar::holidaysRegion() const
@@ -247,19 +273,34 @@ QString Calendar::holidaysRegion() const
     return calendarTable()->holidaysRegion();
 }
 
-void Calendar::clearDateProperties()
+bool Calendar::dateHasDetails(const QDate &date) const
 {
-    calendarTable()->clearDateProperties();
+    return calendarTable()->dateHasDetails(date);
 }
 
-void Calendar::setDateProperty(QDate date, const QString &description)
+QString Calendar::dateDetails(const QDate &date) const
 {
-    calendarTable()->setDateProperty(date, description);
+    return calendarTable()->dateDetails(date);
 }
 
-QString Calendar::dateProperty(QDate date) const
+void Calendar::setAutomaticUpdateEnabled(bool automatic)
 {
-    return calendarTable()->dateProperty(date);
+    calendarTable()->setAutomaticUpdateEnabled(automatic);
+}
+
+bool Calendar::isAutomaticUpdateEnabled() const
+{
+    return calendarTable()->isAutomaticUpdateEnabled();
+}
+
+void Calendar::setCurrentDate(const QDate &date)
+{
+    calendarTable()->setCurrentDate(date);  
+}
+
+const QDate& Calendar::currentDate() const
+{
+    return calendarTable()->currentDate();
 }
 
 void Calendar::applyConfiguration(KConfigGroup cg)
@@ -301,12 +342,33 @@ void Calendar::dateUpdated(const QDate &newDate)
 {
     // Ignore the date passed in, only ever show the date to match the CalendarTable
     Q_UNUSED(newDate);
-
     refreshWidgets();
-
     emit dateChanged(date());
 }
 
+void Calendar::showTip(const QDate &date)
+{
+    QGraphicsWidget *item = parentWidget();
+    if (!item) {
+        item = this;
+    }
+
+    if (dateHasDetails(date)) {
+        const QString details = dateDetails(date);
+        Plasma::ToolTipContent content(calendar()->formatDate(date),
+                                       details,
+                                       KIcon("view-pim-calendar"));
+        content.setAutohide(false);
+        Plasma::ToolTipManager::self()->setContent(item, content);
+        Plasma::ToolTipManager::self()->show(item);
+    } else {
+        if (Plasma::ToolTipManager::self()->isVisible(item)) {
+            Plasma::ToolTipManager::self()->hide(item);
+        }
+        Plasma::ToolTipManager::self()->setContent(item, Plasma::ToolTipContent());
+    }
+}
+
 // Update the nav widgets to show the current date in the CalendarTable
 void Calendar::refreshWidgets()
 {
@@ -331,11 +393,19 @@ void Calendar::nextMonth()
     setDate(calendar()->addMonths(date(), 1));
 }
 
+void Calendar::prevYear()
+{
+    setDate(calendar()->addYears(date(), -1));
+}
+
+void Calendar::nextYear()
+{
+    setDate(calendar()->addYears(date(), 1));
+}
+
 void Calendar::monthsPopup()
 {
-    delete d->monthMenu;
-    d->monthMenu = new QMenu();
-
+    d->monthMenu->clear();
     int year = calendar()->year(date());
     int monthsInYear = calendar()->monthsInYear(date());
 
@@ -346,7 +416,21 @@ void Calendar::monthsPopup()
         d->monthMenu->addAction(tmpAction);
     }
 
-    d->monthMenu->popup(QCursor::pos());
+    QGraphicsView *view = Plasma::viewFor(d->month);
+    if (view) {
+        d->monthMenu->adjustSize();
+        const int x = d->month->sceneBoundingRect().center().x() - d->monthMenu->width() / 2;
+        QPoint pos(x, d->month->sceneBoundingRect().bottom());
+        pos = view->mapToGlobal(view->mapFromScene(pos));
+        QRect r = Kephal::ScreenUtils::screenGeometry(Kephal::ScreenUtils::screenId(view->geometry().center()));
+        if (pos.y() + d->monthMenu->height() > r.bottom()) {
+            pos = QPoint(x, d->month->sceneBoundingRect().top() - d->monthMenu->height());
+            pos = view->mapToGlobal(view->mapFromScene(pos));
+        }
+        d->monthMenu->popup(pos);
+    } else {
+        d->monthMenu->popup(QCursor::pos());
+    }
 }
 
 void Calendar::monthTriggered()
@@ -370,22 +454,23 @@ void Calendar::goToWeek(int newWeek)
 
 void Calendar::showYearSpinBox()
 {
-    QGraphicsLinearLayout *m_hLayout = (QGraphicsLinearLayout*)d->year->parentLayoutItem();
+    QGraphicsLinearLayout *hLayout = (QGraphicsLinearLayout*)d->year->parentLayoutItem();
 
-    d->yearSpinBox->setValue(calendar()->year(date()));
-    m_hLayout->removeItem(d->year);
-    m_hLayout->insertItem(3,d->yearSpinBox);
-    //d->yearSpinBox->setGeometry(d->year->geometry());
     d->year->hide();
+    hLayout->removeItem(d->year);
+    d->yearSpinBox->setValue(calendar()->year(date()));
+    d->yearSpinBox->setMinimumWidth(d->yearSpinBox->preferredSize().width());
+    hLayout->insertItem(s_yearWidgetIndex, d->yearSpinBox);
+    hLayout->activate();
     d->yearSpinBox->show();
     d->yearSpinBox->nativeWidget()->setFocus(Qt::MouseFocusReason);
 }
 
 void Calendar::hideYearSpinBox()
 {
-    QGraphicsLinearLayout *m_hLayout = (QGraphicsLinearLayout*)d->yearSpinBox->parentLayoutItem();
-    m_hLayout->removeItem(d->yearSpinBox);
-    m_hLayout->insertItem(3,d->year);
+    QGraphicsLinearLayout *hLayout = (QGraphicsLinearLayout*)d->yearSpinBox->parentLayoutItem();
+    hLayout->removeItem(d->yearSpinBox);
+    hLayout->insertItem(s_yearWidgetIndex, d->year);
     d->yearSpinBox->hide();
 
     int newYear = d->yearSpinBox->value();
