@@ -48,6 +48,7 @@
 #include <KGlobalSettings>
 #include <KRun>
 #include <KLocale>
+#include <KPassivePopup>
 #include <KService>
 #include <KServiceTypeTrader>
 #include <KTimeZone>
@@ -64,7 +65,6 @@
 #include <Plasma/Label>
 
 #include "calendar.h"
-#include "dateextenderwidget.h"
 
 #include "ui_timezonesConfig.h"
 #include "ui_generalConfig.h"
@@ -136,30 +136,11 @@ public:
             Plasma::ExtenderItem *eItem = new Plasma::ExtenderItem(q->extender());
             eItem->setName("today");
             q->initExtenderItem(eItem);
-
         } else if (todayExtender && !isHoliday) {
             todayExtender->destroy();
         }
     }
 
-    void createDateExtender(const QDate& date)
-    {
-        Plasma::ExtenderItem *eItem = new Plasma::ExtenderItem(q->extender());
-        eItem->setName("dateExtender-" + date.toString(Qt::ISODate));
-        q->initExtenderItem(eItem);
-    }
-
-    void destroyDateExtenders()
-    {
-        QList<Plasma::ExtenderItem *> extenders = q->extender()->items();
-        for (int i = 0; i < extenders.size(); i++){
-            Plasma::ExtenderItem *eItem = extenders.at(i);
-            if (eItem->name().startsWith(QLatin1String("dateExtender-")) && !eItem->isDetached()){
-                eItem->destroy();
-            }
-        }
-    }
-
     void setPrettyTimezone()
     {
         QString timezonetranslated = i18n(timezone.toUtf8().data());
@@ -210,7 +191,7 @@ void ClockApplet::speakTime(const QTime &time)
         if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kttsd")) {
             QString error;
             if (KToolInvocation::startServiceByDesktopName("kttsd", QStringList(), &error)) {
-                KMessageBox::error(0, i18n( "Starting KTTSD Failed"), error );
+                KPassivePopup::message(i18n("Starting KTTSD Failed"), error, static_cast<QWidget *>(0));
                 return;
             }
         }
@@ -299,8 +280,8 @@ void ClockApplet::updateTipContent()
         d->addTzToTipText(subText, tz);
     }
 
-    QString property = d->calendarWidget->dateProperty(tipDate);
-    if (!property.isEmpty()) {
+    if (d->calendarWidget->dateHasDetails(tipDate)) {
+        QString property = d->calendarWidget->dateDetails(tipDate);
         QString countryString = KGlobal::locale()->countryCodeToName(d->calendarWidget->holidaysRegion());
         if (countryString.isEmpty()) {
             subText.append("<br>").append(property);
@@ -333,6 +314,18 @@ void ClockApplet::updateTipContent()
     Plasma::ToolTipManager::self()->setContent(this, tipData);
 }
 
+void ClockApplet::updateClockApplet()
+{
+    bool updateSelectedDate = (d->calendarWidget->currentDate() == d->calendarWidget->date());
+
+    Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
+    d->calendarWidget->setCurrentDate(data["Date"].toDate());
+    
+    if (updateSelectedDate){
+        d->calendarWidget->setDate(d->calendarWidget->currentDate());
+    }
+}
+
 Plasma::ToolTipContent ClockApplet::toolTipContent()
 {
     return Plasma::ToolTipContent();
@@ -367,7 +360,6 @@ void ClockApplet::createConfigurationInterface(KConfigDialog *parent)
     }
     d->ui.clockDefaultsTo->setCurrentIndex(defaultSelection);
 
-    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
     connect(d->ui.timeZones, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(updateClockDefaultsTo()));
 }
 
@@ -383,6 +375,10 @@ void ClockApplet::clockConfigChanged()
 
 void ClockApplet::configChanged()
 {
+    if (isUserConfiguring()) {
+        configAccepted();
+    }
+
     KConfigGroup cg = config();
     d->selectedTimezones = cg.readEntry("timeZones", QStringList());
     d->timezone = cg.readEntry("timezone", d->timezone);
@@ -392,13 +388,19 @@ void ClockApplet::configChanged()
     d->announceInterval = cg.readEntry("announceInterval", 0);
 
     clockConfigChanged();
-   // d->calendarWidget->configAccepted(cg); is called in configAccepted(), 
-    // as is setCurrentTimezone
-    // so we only need to do this in the case where the user hasn't been
-    // configuring things
-    d->calendarWidget->applyConfiguration(cg);
-    Plasma::DataEngine::Data data = dataEngine("time")->query(d->timezone);
-    d->calendarWidget->setDate(data["Date"].toDate());
+
+    if (isUserConfiguring()) {
+        constraintsEvent(Plasma::SizeConstraint);
+        update();
+    } else {
+        // d->calendarWidget->configAccepted(cg); is called in configAccepted(), 
+        // as is setCurrentTimezone
+        // so we only need to do this in the case where the user hasn't been
+        // configuring things
+        d->calendarWidget->applyConfiguration(cg);
+        Plasma::DataEngine::Data data = dataEngine("time")->query(d->timezone);
+        d->calendarWidget->setDate(data["Date"].toDate());
+    }
 }
 
 void ClockApplet::clockConfigAccepted()
@@ -414,13 +416,12 @@ void ClockApplet::configAccepted()
 
     if (d->ui.clockDefaultsTo->currentIndex() == 0) {
         //The first position in ui.clockDefaultsTo is "Local"
-        cg.writeEntry("defaultTimezone", localTimezoneUntranslated());
+        d->defaultTimezone = localTimezoneUntranslated();
     } else {
-        cg.writeEntry("defaultTimezone",
-                      d->ui.clockDefaultsTo->itemData(d->ui.clockDefaultsTo->currentIndex()).toString());
+        d->defaultTimezone = d->ui.clockDefaultsTo->itemData(d->ui.clockDefaultsTo->currentIndex()).toString();
     }
 
-    configChanged();
+    cg.writeEntry("defaultTimezone", d->defaultTimezone);
     QString cur = currentTimezone();
     setCurrentTimezone(d->defaultTimezone);
     changeEngineTimezone(cur, d->defaultTimezone);
@@ -430,9 +431,7 @@ void ClockApplet::configAccepted()
     cg.writeEntry("announceInterval", d->generalUi.interval->value());
 
     clockConfigAccepted();
-    constraintsEvent(Plasma::SizeConstraint);
-    update();
-    
+
     emit configNeedsSaving();
 }
 
@@ -479,8 +478,8 @@ QList<QAction *> ClockApplet::contextualActions()
         KService::List offers = KServiceTypeTrader::self()->query("KCModule", "Library == 'kcm_clock'");
         if (!offers.isEmpty() && hasAuthorization("LaunchApp")) {
             d->adjustSystemTimeAction = new QAction(this);
-            d->adjustSystemTimeAction->setText(i18n("Adjust Date and Time"));
-            d->adjustSystemTimeAction->setIcon(KIcon("preferences-system-time"));
+            d->adjustSystemTimeAction->setText(i18n("Adjust Date and Time..."));
+            d->adjustSystemTimeAction->setIcon(KIcon(icon()));
             connect(d->adjustSystemTimeAction, SIGNAL(triggered()), this, SLOT(launchTimeControlPanel()));
         }
     }
@@ -558,16 +557,6 @@ void ClockApplet::initExtenderItem(Plasma::ExtenderItem *item)
         item->setIcon("view-pim-calendar");
         d->label = new Plasma::Label();
         item->setWidget(d->label);
-    } else if (item->name().startsWith(QLatin1String("dateExtender-"))) {
-        item->setIcon("view-pim-calendar");
-        item->showCloseButton();
-        QDate date = QDate::fromString(item->name().remove(0, 13), Qt::ISODate);
-        DateExtenderWidget *widget = 0;
-
-        item->setTitle(calendar()->formatDate(date));
-        widget = new DateExtenderWidget(d->calendarWidget->dateProperty(date));
-
-        item->setWidget(widget);
     }
 }
 
@@ -576,9 +565,9 @@ void ClockApplet::init()
     Plasma::ToolTipManager::self()->registerWidget(this);
 
     d->calendarWidget = new Plasma::Calendar();
+    d->calendarWidget->setAutomaticUpdateEnabled(false);
     d->calendarWidget->setMinimumSize(QSize(230, 220));
     d->calendarWidget->setDataEngine(dataEngine("calendar"));
-    connect(d->calendarWidget, SIGNAL(dateChanged(const QDate &)), this, SLOT(dateChanged(const QDate &)));
     d->createCalendarExtender();
 
     extender();
@@ -595,8 +584,6 @@ void ClockApplet::popupEvent(bool show)
 
     Plasma::DataEngine::Data data = dataEngine("time")->query(currentTimezone());
     d->calendarWidget->setDate(data["Date"].toDate());
-
-    d->destroyDateExtenders();
 }
 
 void ClockApplet::constraintsEvent(Plasma::Constraints constraints)
@@ -660,15 +647,6 @@ QString ClockApplet::localTimezoneUntranslated()
     return "Local";
 }
 
-void ClockApplet::dateChanged(const QDate &date)
-{
-    d->destroyDateExtenders();
-
-    if (!d->calendarWidget->dateProperty(date).isEmpty()) {
-        d->createDateExtender(date);
-    }
-}
-
 void ClockApplet::updateClipboardMenu()
 {
     d->clipboardMenu->clear();
