@@ -57,6 +57,13 @@ public:
     virtual QIcon icon() const;
     virtual QString name() const;
 
+    virtual WindowList winIds() const;
+
+    /**
+     * @return unique identifier of this item
+     */
+    int id() const;
+
     /**
     *   Returns the parent group of this item
     */
@@ -70,6 +77,8 @@ public:
     bool isGroupMember(const GroupPtr group) const;
     virtual bool isGroupItem() const = 0;
 
+    virtual bool isStartupItem() const;
+
     virtual bool isOnCurrentDesktop() const = 0;
     virtual bool isOnAllDesktops() const = 0;
     virtual int desktop() const = 0;
