@@ -266,9 +266,10 @@ void TaskGroupItem::updateToolTip()
 
     Plasma::ToolTipContent data(m_group.data()->name(), QString());
     int desktop = m_group.data()->desktop();
-    if (desktop != 0) {
+    if (desktop != 0 &&
+        (!m_applet->groupManager().showOnlyCurrentDesktop() || !m_group.data()->isOnCurrentDesktop())) {
         data.setSubText(i18nc("Which virtual desktop a window is currently on", "On %1",
-                        KWindowSystem::desktopName(m_group.data()->desktop())));
+                        KWindowSystem::desktopName(desktop)));
     }
 
     data.setImage(m_group.data()->icon());
@@ -374,6 +375,7 @@ void TaskGroupItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *e)
         QGraphicsWidget::contextMenuEvent(e);
         return;
     }
+
     Q_ASSERT(m_applet);
     //we are the master group item
     if (m_applet == parentWidget()) {
@@ -381,8 +383,9 @@ void TaskGroupItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *e)
         return;
     }
 
-    QAction *a;
+    QList <QAction*> actionList;
 
+    QAction *a;
     if (!collapsed()) {
         a = new QAction(i18n("Collapse Group"), this);
         connect(a, SIGNAL(triggered()), this, SLOT(collapse()));
@@ -390,10 +393,13 @@ void TaskGroupItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *e)
         a = new QAction(i18n("Expand Group"), this);
         connect(a, SIGNAL(triggered()), this, SLOT(expand()));
     }
-
-    QList <QAction*> actionList;
     actionList.append(a);
 
+    a = m_applet->action("configure");
+    if (a && a->isEnabled()) {
+        actionList.append(a);
+    }
+
     TaskManager::BasicMenu menu(qobject_cast<QWidget*>(this), m_group.data(), &m_applet->groupManager(), actionList);
     menu.adjustSize();
 
@@ -600,6 +606,22 @@ void TaskGroupItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
     AbstractTaskItem::mouseReleaseEvent(event);
 }
 
+void TaskGroupItem::handleActiveWindowChanged(WId id)
+{
+    if (!m_popupDialog) {
+        return;
+    }
+
+    if (id == m_popupDialog->winId()) {
+        return;
+    }
+
+    m_popupDialog->hide();
+
+    QRect rect = iconGeometry();
+    publishIconGeometry(rect);
+}
+
 void TaskGroupItem::popupMenu()
 {
     //kDebug();
@@ -631,7 +653,7 @@ void TaskGroupItem::popupMenu()
         KWindowSystem::setType(m_popupDialog->winId(), NET::PopupMenu);
         m_popupDialog->setAttribute(Qt::WA_X11NetWmWindowTypeDock);
         connect(m_popupDialog, SIGNAL(dialogVisible(bool)), m_applet, SLOT(setPopupDialog(bool)));
-        connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)), m_popupDialog, SLOT(hide()));
+        connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)), this, SLOT(handleActiveWindowChanged(WId)));
         KWindowSystem::setState(m_popupDialog->winId(), NET::SkipTaskbar| NET::SkipPager);
         m_popupDialog->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
         //TODO in the future it may be possible to use the Qt::Popup flag instead of the eventFilter, but for now the focus works better with the eventFilter
@@ -650,10 +672,16 @@ void TaskGroupItem::popupMenu()
         } else {
             m_popupDialog->hide();
         }
+
+        QRect rect = iconGeometry();
+        publishIconGeometry(rect);
     } else {
         m_tasksLayout->setOrientation(Plasma::Vertical);
         m_tasksLayout->setMaximumRows(1);
-        m_offscreenWidget->adjustSize();
+        m_offscreenWidget->layout()->activate();
+        m_offscreenWidget->resize(m_offscreenWidget->effectiveSizeHint(Qt::PreferredSize));
+        m_popupDialog->syncToGraphicsWidget();
+
         if (m_applet->containment() && m_applet->containment()->corona()) {
             m_popupDialog->move(m_applet->containment()->corona()->popupPosition(this, m_popupDialog->size(), Qt::AlignCenter));
         }
@@ -663,11 +691,8 @@ void TaskGroupItem::popupMenu()
         } else {
             m_popupDialog->show();
         }
-        m_offscreenWidget->layout()->activate();
-        m_offscreenWidget->resize(m_offscreenWidget->effectiveSizeHint(Qt::PreferredSize));
+
         m_popupDialog->raise();
-        KWindowSystem::activateWindow(m_popupDialog->winId());
-        //kDebug() << m_popupDialog->size() << m_tasksLayout->size();
     }
 }
 
@@ -681,6 +706,9 @@ bool TaskGroupItem::eventFilter(QObject *watched, QEvent *event)
         } else {
             m_popupDialog->hide();
         }
+
+        QRect rect = iconGeometry();
+        publishIconGeometry(rect);
         QTimer::singleShot(100, this, SLOT(clearPopupLostFocus()));
     }
 
@@ -703,7 +731,6 @@ bool TaskGroupItem::focusSubTask(bool next, bool activate)
             for (int i = 0; i < subTasks; ++i) {
                 if (selectSubTask(i)->taskFlags() & TaskHasFocus) {
                     index = i;
-
                     break;
                 }
             }
@@ -728,19 +755,16 @@ bool TaskGroupItem::focusSubTask(bool next, bool activate)
 
         if (taskItem) {
             taskItem->setFocus();
-
             m_activeTaskIndex = index;
         }
 
         if (activate && taskItem) {
             stopWindowHoverEffect();
-
             taskItem->activate();
         }
 
         return true;
-    }
-    else {
+    } else {
         return false;
     }
 }
@@ -1168,7 +1192,7 @@ void TaskGroupItem::updateActive(AbstractTaskItem *task)
     m_activeTaskIndex = indexOf(task);
 }
 
-int TaskGroupItem::indexOf(AbstractTaskItem *task)
+int TaskGroupItem::indexOf(AbstractTaskItem *task, bool descendGroups)
 {
     if (!m_group || !task) {
         //kDebug() << "Error";
@@ -1181,29 +1205,35 @@ int TaskGroupItem::indexOf(AbstractTaskItem *task)
         AbstractTaskItem *taskItem = abstractTaskItem(item);
         if (taskItem) {
             if (task == taskItem) {
-                TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
-                if (groupItem) {
-                    int subIndex = groupItem->indexOf(groupItem->activeSubTask());
-                    if (subIndex == -1) {
-                        index += groupItem->count();
-                    } else {
-                        return index + subIndex;
+                if (descendGroups) {
+                    TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
+                    if (groupItem) {
+                        int subIndex = groupItem->indexOf(groupItem->activeSubTask());
+                        if (subIndex == -1) {
+                            index += groupItem->count();
+                        } else {
+                            return index + subIndex;
+                        }
                     }
                 }
 
                 return index;
             }
 
-            TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
-            if (groupItem) {
-                int subIndex = groupItem->indexOf(task);
-                if(subIndex == -1) {
-                    index += groupItem->count();
+            if (descendGroups) {
+                TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
+                if (groupItem) {
+                    int subIndex = groupItem->indexOf(task);
+                    if (subIndex == -1) {
+                        index += groupItem->count();
+                    } else {
+                        return index+subIndex;
+                    }
                 } else {
-                    return index+subIndex;
+                    ++index;
                 }
             } else {
-                index++;
+                ++index;
             }
         }
     }
@@ -1253,7 +1283,7 @@ int TaskGroupItem::totalSubTasks()
 
 AbstractTaskItem * TaskGroupItem::selectSubTask(int index)
 {
-    foreach(AbstractGroupableItem *item, group()->members()) {
+    foreach (AbstractGroupableItem *item, group()->members()) {
         AbstractTaskItem *taskItem = abstractTaskItem(item);
         if (taskItem) {
             TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(taskItem);
@@ -1275,7 +1305,7 @@ AbstractTaskItem * TaskGroupItem::selectSubTask(int index)
 
 void TaskGroupItem::wheelEvent(QGraphicsSceneWheelEvent *event)
 {
-    focusSubTask((event->delta() > 0), true);
+    focusSubTask((event->delta() < 0), true);
 }
 
 int TaskGroupItem::maxRows()
