@@ -18,7 +18,7 @@
  */
 
 #include "panelappletoverlay.h"
-
+#include "panelapplethandle.h"
 
 #include <QApplication>
 #include <QGraphicsLinearLayout>
@@ -32,10 +32,15 @@
 
 #include <Plasma/Applet>
 #include <Plasma/Containment>
+#include <Plasma/Corona>
 #include <Plasma/PaintUtils>
 #include <Plasma/Theme>
 #include <Plasma/View>
 
+
+PanelAppletHandle *PanelAppletOverlay::s_appletHandle = 0;
+int PanelAppletOverlay::s_appletHandleCount = 0;
+
 class AppletMoveSpacer : public QGraphicsWidget
 {
 public:
@@ -77,15 +82,28 @@ PanelAppletOverlay::PanelAppletOverlay(Plasma::Applet *applet, QWidget *parent)
       m_applet(applet),
       m_spacer(0),
       m_layout(dynamic_cast<QGraphicsLinearLayout*>(applet->containment()->layout())), // ++assumptions;
-      m_menuButtonRect(2,2,10,10),
       m_index(0),
       m_clickDrag(false)
 {
+    if (!s_appletHandle) {
+        s_appletHandle = new PanelAppletHandle();
+    }
+
+    ++s_appletHandleCount;
+
+    connect(s_appletHandle, SIGNAL(mousePressed(Plasma::Applet *, QMouseEvent *)), 
+            this, SLOT(handleMousePressed(Plasma::Applet *, QMouseEvent *)));
+    connect(s_appletHandle, SIGNAL(mouseMoved(Plasma::Applet *, QMouseEvent *)), 
+            this, SLOT(handleMouseMoved(Plasma::Applet *, QMouseEvent *)));
+    connect(s_appletHandle, SIGNAL(mouseReleased(Plasma::Applet *, QMouseEvent *)), 
+            this, SLOT(handleMouseReleased(Plasma::Applet *, QMouseEvent *)));
+
     syncIndex();
     syncOrientation();
     syncGeometry();
     setMouseTracking(true);
 
+
     connect(m_applet, SIGNAL(destroyed(QObject*)), this, SLOT(appletDestroyed()));
     connect(m_applet, SIGNAL(geometryChanged()), this, SLOT(delaySyncGeometry()));
 }
@@ -109,6 +127,13 @@ PanelAppletOverlay::~PanelAppletOverlay()
         m_spacer->deleteLater();
         m_spacer = 0;
     }
+
+    --s_appletHandleCount;
+    if (s_appletHandleCount < 1) {
+        delete s_appletHandle;
+        s_appletHandle = 0;
+        s_appletHandleCount = 0;
+    }
 }
 
 void PanelAppletOverlay::paintEvent(QPaintEvent *event)
@@ -130,9 +155,6 @@ void PanelAppletOverlay::paintEvent(QPaintEvent *event)
     QRect iconRect;
 
 
-    KIcon configIcon("arrow-up");
-    p.drawPixmap(m_menuButtonRect, configIcon.pixmap(m_menuButtonRect.size()));
-
     if (!m_applet) {
         return;
     }
@@ -151,6 +173,9 @@ void PanelAppletOverlay::paintEvent(QPaintEvent *event)
 void PanelAppletOverlay::mousePressEvent(QMouseEvent *event)
 {
     Q_UNUSED(event)
+    if (!m_applet) {
+        return;
+    }
 
     m_lastGlobalPos = event->globalPos();
     //kDebug() << m_clickDrag;
@@ -161,8 +186,7 @@ void PanelAppletOverlay::mousePressEvent(QMouseEvent *event)
         return;
     }
 
-    if (event->button() != Qt::LeftButton ||
-        m_menuButtonRect.contains(event->pos())) {
+    if (event->button() != Qt::LeftButton) {
         //kDebug() << "sending even to" << (QWidget*)parent();
         Plasma::View *view = dynamic_cast<Plasma::View*>(parent());
 
@@ -225,7 +249,7 @@ void PanelAppletOverlay::mousePressEvent(QMouseEvent *event)
 
 void PanelAppletOverlay::mouseMoveEvent(QMouseEvent *event)
 {
-    if (!m_layout && !m_applet) {
+    if (!m_layout || !m_applet) {
         //kDebug() << "no layout";
         return;
     }
@@ -261,6 +285,10 @@ void PanelAppletOverlay::mouseMoveEvent(QMouseEvent *event)
         Plasma::View *view = Plasma::View::topLevelViewAt(event->globalPos());
         //kDebug() << "checking view" << view << m_applet->view();
 
+        if (!view) {
+            view = dynamic_cast<Plasma::View*>(parent());
+        }
+
         if (!view) {
             return;
         }
@@ -350,7 +378,8 @@ void PanelAppletOverlay::mouseMoveEvent(QMouseEvent *event)
     if (m_index > 0 && m_layout->itemAt(m_index - 1)) {
         const bool prevIsApplet = dynamic_cast<Plasma::Applet*>(m_layout->itemAt(m_index - 1)) != 0;
         const bool nextIsApplet = dynamic_cast<Plasma::Applet*>(m_layout->itemAt(m_index + 1)) != 0;
-    
+
+
         // swap items if we pass completely over the next/previous item or cross
         // more than halfway across it, whichever comes first
         if (m_orientation == Qt::Horizontal) {
@@ -372,6 +401,7 @@ void PanelAppletOverlay::mouseMoveEvent(QMouseEvent *event)
             swapWithNext();
         }
     }
+
     m_lastGlobalPos = event->globalPos();
     //kDebug() << "=================================";
 }
@@ -379,7 +409,7 @@ void PanelAppletOverlay::mouseMoveEvent(QMouseEvent *event)
 void PanelAppletOverlay::mouseReleaseEvent(QMouseEvent *event)
 {
     Q_UNUSED(event)
-    if (!m_spacer) {
+    if (!m_spacer || !m_applet) {
         //kDebug() << "releasing as we don't have a spacer";
         releaseMouse();
         setMouseTracking(false);
@@ -419,16 +449,42 @@ void PanelAppletOverlay::mouseReleaseEvent(QMouseEvent *event)
     m_applet->setZValue(m_applet->zValue() - 1);
 }
 
+void PanelAppletOverlay::handleMousePressed(Plasma::Applet *applet, QMouseEvent *event)
+{
+    if (applet == m_applet) {
+        QMouseEvent ownEvent(event->type(), mapFromGlobal(event->globalPos()), event->globalPos(), event->button(), event->buttons(), event->modifiers());
+        mousePressEvent(&ownEvent);
+    }
+}
+
+void PanelAppletOverlay::handleMouseMoved(Plasma::Applet *applet, QMouseEvent *event)
+{
+    if (applet == m_applet) {
+        QMouseEvent ownEvent(event->type(), mapFromGlobal(event->globalPos()), event->globalPos(), event->button(), event->buttons(), event->modifiers());
+        mouseMoveEvent(&ownEvent);
+    }
+}
+
+void PanelAppletOverlay::handleMouseReleased(Plasma::Applet *applet, QMouseEvent *event)
+{
+    if (applet == m_applet) {
+        QMouseEvent ownEvent(event->type(), mapFromGlobal(event->globalPos()), event->globalPos(), event->button(), event->buttons(), event->modifiers());
+        mouseReleaseEvent(&ownEvent);
+    }
+}
+
 void PanelAppletOverlay::enterEvent(QEvent *event)
 {
     Q_UNUSED(event)
     update();
+    s_appletHandle->setApplet(m_applet);
 }
 
 void PanelAppletOverlay::leaveEvent(QEvent *event)
 {
     setCursor(Qt::ArrowCursor);
     Q_UNUSED(event)
+    s_appletHandle->startHideTimeout();
     update();
 }
 
@@ -493,7 +549,7 @@ void PanelAppletOverlay::delaySyncGeometry()
 
 void PanelAppletOverlay::syncGeometry()
 {
-    if (!m_layout) {
+    if (!m_layout || !m_applet) {
         return;
     }
 
@@ -512,6 +568,10 @@ void PanelAppletOverlay::syncGeometry()
     } else {
         m_nextGeom = QRectF();
     }
+
+    if (m_applet->containment() && m_applet->containment()->corona()) {
+        s_appletHandle->move(m_applet->containment()->corona()->popupPosition(m_applet, s_appletHandle->size(), Qt::AlignCenter));
+    }
 }
 
 void PanelAppletOverlay::syncIndex()
@@ -532,6 +592,10 @@ void PanelAppletOverlay::syncIndex()
 
 void PanelAppletOverlay::syncOrientation()
 {
+    if (!m_applet) {
+        return;
+    }
+
     m_orientation = m_applet->formFactor() == Plasma::Horizontal ? Qt::Horizontal : Qt::Vertical;
 }
 
