@@ -85,7 +85,8 @@ PanelController::PanelController(QWidget* parent)
      m_extLayout(0),
      m_layout(0),
      m_dragging(NoElement),
-     m_startDragPos(0,0),
+     m_startDragControllerPos(0,0),
+     m_startDragMousePos(0,0),
      m_optionsDialog(0),
      m_leftAlignTool(0),
      m_centerAlignTool(0),
@@ -246,7 +247,7 @@ void PanelController::setContainment(Plasma::Containment *c)
     }
 
     ControllerWindow::setContainment(c);
-    PlasmaApp::self()->hideWidgetExplorer(containment()->screen());
+    PlasmaApp::self()->hideController(containment()->screen());
 
     QWidget *child;
     while (!m_actionWidgets.isEmpty()) {
@@ -277,8 +278,8 @@ void PanelController::setContainment(Plasma::Containment *c)
     action = containment()->action("lock widgets");
     if (action && action->isEnabled()) {
         ToolButton *lockWidgetsTool = addTool(action, this);
-        m_layout->insertWidget(insertIndex, lockWidgetsTool);
-        ++insertIndex;
+        m_optDialogLayout->addWidget(lockWidgetsTool, m_optDialogLayout->count() - 2);
+        connect(lockWidgetsTool, SIGNAL(clicked()), m_optionsDialog, SLOT(hide()));
         connect(lockWidgetsTool, SIGNAL(clicked()), this, SLOT(hide()));
     }
 
@@ -293,6 +294,14 @@ void PanelController::setContainment(Plasma::Containment *c)
     syncRuler();
 }
 
+void PanelController::moveEvent(QMoveEvent *event)
+{
+    //FIXME: this is a glorious hack: it causes the window to be positioned correctly after kwin positions it incorrectly.
+    //TODO: the proper way to do is to give a window type specific to the panel controller, that gets positioned correctly
+    emit offsetChanged(m_ruler->offset());
+    ControllerWindow::moveEvent(event);
+}
+
 void PanelController::setLocation(const Plasma::Location &loc)
 {
     if (location() == loc) {
@@ -381,6 +390,8 @@ void PanelController::setLocation(const Plasma::Location &loc)
     m_ruler->setMinimumSize(m_ruler->sizeHint());
     m_ruler->setMaximumSize(m_ruler->sizeHint());
     syncRuler();
+
+    resize(sizeHint());
 }
 
 void PanelController::setOffset(int newOffset)
@@ -475,7 +486,7 @@ bool PanelController::eventFilter(QObject *watched, QEvent *event)
 {
     ControllerWindow::eventFilter(watched, event);
 
-    if (watched == m_optionsDialog && event->type() == QEvent::WindowDeactivate && (!isWidgetExplorerVisible())) {
+    if (watched == m_optionsDialog && event->type() == QEvent::WindowDeactivate && (!isControllerViewVisible())) {
         if (!m_settingsTool->underMouse()) {
             m_optionsDialog->hide();
         }
@@ -487,8 +498,10 @@ bool PanelController::eventFilter(QObject *watched, QEvent *event)
     } else if (watched == m_moveTool) {
         if (event->type() == QEvent::MouseButtonPress) {
             m_dragging = MoveButtonElement;
+            m_moveTool->grabMouse();
         } else if (event->type() == QEvent::MouseButtonRelease) {
             m_dragging = NoElement;
+            m_moveTool->releaseMouse();
         } else if (event->type() == QEvent::MouseMove) {
             QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
             mouseMoveFilter(mouseEvent);
@@ -496,11 +509,13 @@ bool PanelController::eventFilter(QObject *watched, QEvent *event)
     } else if (watched == m_sizeTool) {
         if (event->type() == QEvent::MouseButtonPress) {
             QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
-            m_startDragPos = mouseEvent->pos();
+            m_startDragMousePos = mouseEvent->globalPos();
+            m_startDragControllerPos = pos(); // the global position of the controller window
             m_dragging = ResizeButtonElement;
         } else if (event->type() == QEvent::MouseButtonRelease) {
             //resets properties saved during the drag
-            m_startDragPos = QPoint(0, 0);
+            m_startDragMousePos = QPoint(0, 0);
+            m_startDragControllerPos = QPoint(0, 0);
             m_dragging = NoElement;
             setCursor(Qt::ArrowCursor);
         } else if (event->type() == QEvent::MouseMove) {
@@ -521,10 +536,6 @@ void PanelController::mouseMoveFilter(QMouseEvent *event)
     QRect screenGeom = Kephal::ScreenUtils::screenGeometry(containment()->screen());
 
     if (m_dragging == MoveButtonElement) {
-        //only move when the mouse cursor is out of the controller to avoid an endless reposition cycle
-        if (geometry().contains(event->globalPos())) {
-            return;
-        }
 
         if (!screenGeom.contains(event->globalPos())) {
             //move panel to new screen if dragged there
@@ -589,40 +600,36 @@ void PanelController::mouseMoveFilter(QMouseEvent *event)
     //Resize handle moved
     switch (location()) {
     case Plasma::LeftEdge: {
-        int newX = mapToGlobal(event->pos()).x() - m_startDragPos.x();
-        if (newX - MINIMUM_HEIGHT > screenGeom.left() &&
-            newX - screenGeom.left() <= screenGeom.width()/3) {
-            move(newX, pos().y());
-            resizeFrameHeight(geometry().left() - screenGeom.left());
-        }
+        int newX = m_startDragControllerPos.x() + event->globalX() - m_startDragMousePos.x();
+        newX = qMax(newX, screenGeom.left() + MINIMUM_HEIGHT);
+        newX = qMin(newX, screenGeom.left() + screenGeom.width()/3);
+        move(newX, pos().y());
+        resizeFrameHeight(geometry().left() - screenGeom.left());
         break;
     }
     case Plasma::RightEdge: {
-        int newX = mapToGlobal(event->pos()).x() - m_startDragPos.x();
-        if (newX + width() + MINIMUM_HEIGHT < screenGeom.right() &&
-            newX + width() - screenGeom.left() >= 2*(screenGeom.width()/3)) {
-            move(newX, pos().y());
-            resizeFrameHeight(screenGeom.right() - geometry().right());
-        }
+        int newX = m_startDragControllerPos.x() + event->globalX() - m_startDragMousePos.x();
+        newX = qMin(newX, screenGeom.right() - MINIMUM_HEIGHT - width());
+        newX = qMax(newX, screenGeom.left() + 2*(screenGeom.width()/3) - width());
+        move(newX, pos().y());
+        resizeFrameHeight(screenGeom.right() - geometry().right());
         break;
     }
     case Plasma::TopEdge: {
-        int newY = mapToGlobal(event->pos()).y() - m_startDragPos.y();
-        if ( newY - MINIMUM_HEIGHT > screenGeom.top() &&
-             newY - screenGeom.top()<= screenGeom.height()/3) {
-            move(pos().x(), newY);
-            resizeFrameHeight(geometry().top() - screenGeom.top());
-        }
+        int newY = m_startDragControllerPos.y() + event->globalY() - m_startDragMousePos.y();
+        newY = qMax(newY, screenGeom.top() + MINIMUM_HEIGHT);
+        newY = qMin(newY, screenGeom.top() + screenGeom.height()/3);
+        move(pos().x(), newY);
+        resizeFrameHeight(geometry().top() - screenGeom.top());
         break;
     }
     case Plasma::BottomEdge:
     default: {
-        int newY = mapToGlobal(event->pos()).y() - m_startDragPos.y();
-        if ( newY + height() + MINIMUM_HEIGHT < screenGeom.bottom() &&
-             newY + height() - screenGeom.top() >= 2*(screenGeom.height()/3)) {
-            move(pos().x(), newY);
-            resizeFrameHeight(screenGeom.bottom() - geometry().bottom());
-        }
+        int newY = m_startDragControllerPos.y() + event->globalY() - m_startDragMousePos.y();
+        newY = qMin(newY, screenGeom.bottom() - MINIMUM_HEIGHT - height());
+        newY = qMax(newY, screenGeom.top() + 2*(screenGeom.height()/3) - height());
+        move(pos().x(), newY);
+        resizeFrameHeight(screenGeom.bottom() - geometry().bottom());
         break;
     }
     }
@@ -631,7 +638,7 @@ void PanelController::mouseMoveFilter(QMouseEvent *event)
 void PanelController::focusOutEvent(QFocusEvent * event)
 {
     Q_UNUSED(event)
-    if (!m_optionsDialog->isActiveWindow() && !isWidgetExplorerVisible()) {
+    if (!m_optionsDialog->isActiveWindow() && !isControllerViewVisible() && !isActiveWindow()) {
         m_optionsDialog->hide();
         close();
     }
