@@ -23,6 +23,7 @@
 #include <Plasma/Theme>
 #include <Plasma/ToolTipManager>
 #include <KConfigDialog>
+#include <KColorUtils>
 #include <QFileInfo>
 #include <QGraphicsLinearLayout>
 
@@ -45,24 +46,24 @@ void Hdd::init()
     setEngine(dataEngine("soliddevice"));
 
     setTitle(i18n("Disk Space"), true);
-    
+
     configChanged();
-    connectToEngine();
 }
 
 void Hdd::configChanged()
 {
     KConfigGroup cg = config();
-    
+
     QStringList items = cg.readEntry("uuids", QStringList());
     if (items.isEmpty()) {
         items = mounted();
     }
-    
+
     setItems(items);
     setInterval(cg.readEntry("interval", 2) * 60 * 1000);
+    connectToEngine();
 }
-        
+
 QStringList Hdd::mounted()
 {
     Plasma::DataEngine::Data data;
@@ -140,11 +141,8 @@ void Hdd::configAccepted()
 
     uint interval = ui.intervalSpinBox->value();
     cg.writeEntry("interval", interval);
-    interval *= 60 * 1000;
-    setInterval(interval);
 
     emit configNeedsSaving();
-    connectToEngine();
 }
 
 QString Hdd::hddTitle(const QString& uuid, const Plasma::DataEngine::Data &data)
@@ -197,7 +195,6 @@ bool Hdd::addMeter(const QString& source)
     Plasma::Meter *w;
     Plasma::DataEngine *engine = dataEngine("soliddevice");
     Plasma::DataEngine::Data data;
-    Plasma::Theme* theme = Plasma::Theme::defaultTheme();
 
     if (!engine) {
         return false;
@@ -225,25 +222,14 @@ bool Hdd::addMeter(const QString& source)
     } else {
         w->setSvg("system-monitor/hdd_panel");
     }
-    QColor text = theme->color(Plasma::Theme::TextColor);
-    QColor darkerText(adjustColor(text, 40));
     w->setLabel(0, hddTitle(source, data));
-    w->setLabelColor(0, text);
-    w->setLabelColor(1, darkerText);
-    w->setLabelColor(2, darkerText);
-    QFont font = theme->font(Plasma::Theme::DefaultFont);
-    font.setPointSize(9);
-    w->setLabelFont(0, font);
-    font.setPointSizeF(7.5);
-    w->setLabelFont(1, font);
-    w->setLabelFont(2, font);
     w->setLabelAlignment(0, Qt::AlignVCenter | Qt::AlignLeft);
     w->setLabelAlignment(1, Qt::AlignVCenter | Qt::AlignRight);
     w->setLabelAlignment(2, Qt::AlignVCenter | Qt::AlignCenter);
-    kDebug() << w->labelRect(2);
     w->setMaximum(data["Size"].toULongLong() / (1024 * 1024));
+    applyTheme(w);
     layout->addItem(w);
-    appendMeter(source, w);
+    m_meters[source] = w;
     mainLayout()->addItem(layout);
     dataUpdated(source, data);
     setPreferredItemHeight(layout->preferredSize().height());
@@ -259,28 +245,35 @@ bool Hdd::addMeter(const QString& source)
     return true;
 }
 
-void Hdd::themeChanged()
+void Hdd::applyTheme(Plasma::Meter *w)
 {
     Plasma::Theme* theme = Plasma::Theme::defaultTheme();
-    foreach (Plasma::Meter *w, meters()) {
-        QColor text = theme->color(Plasma::Theme::TextColor);
-        QColor darkerText(adjustColor(text, 40));
-        w->setLabelColor(0, text);
-        w->setLabelColor(1, darkerText);
-        w->setLabelColor(2, darkerText);
-        QFont font = theme->font(Plasma::Theme::DefaultFont);
-        font.setPointSize(9);
-        w->setLabelFont(0, font);
-        font.setPointSizeF(7.5);
-        w->setLabelFont(1, font);
-        w->setLabelFont(2, font);
+    QColor text = theme->color(Plasma::Theme::TextColor);
+    QColor bg = theme->color(Plasma::Theme::BackgroundColor);
+    QColor darkerText = KColorUtils::tint(text, bg, 0.4);
+    w->setLabelColor(0, text);
+    w->setLabelColor(1, darkerText);
+    w->setLabelColor(2, darkerText);
+    QFont font = theme->font(Plasma::Theme::DefaultFont);
+    font.setPointSize(9);
+    w->setLabelFont(0, font);
+    font.setPointSizeF(7.5);
+    w->setLabelFont(1, font);
+    w->setLabelFont(2, font);
+}
+
+void Hdd::themeChanged()
+{
+    foreach (Plasma::Meter *w, m_meters) {
+        applyTheme(w);
     }
 }
 
-void Hdd::deleteMeters(QGraphicsLinearLayout* layout)
+void Hdd::deleteMeters()
 {
-    Applet::deleteMeters(layout);
+    Applet::deleteMeters();
     m_diskMap.clear();
+    m_meters.clear();
 }
 
 bool Hdd::isValidDevice(const QString& uuid, Plasma::DataEngine::Data* data)
@@ -319,7 +312,7 @@ void Hdd::dataUpdated(const QString& source,
             }
         }
     } else {
-        Plasma::Meter *w = meters().value(source);
+        Plasma::Meter *w = m_meters.value(source);
         if (!w) {
             return;
         }
