@@ -22,6 +22,8 @@
 #include "newspaper.h"
 #include "appletoverlay.h"
 #include "applettitlebar.h"
+#include "appletscontainer.h"
+#include "appletsview.h"
 #include "../common/nettoolbox.h"
 
 #include <limits>
@@ -49,7 +51,6 @@
 #include <Plasma/FrameSvg>
 #include <Plasma/Theme>
 #include <Plasma/View>
-#include <Plasma/ScrollWidget>
 #include <Plasma/PopupApplet>
 #include <Plasma/Frame>
 #include <Plasma/ToolButton>
@@ -60,31 +61,25 @@ using namespace Plasma;
 Newspaper::Newspaper(QObject *parent, const QVariantList &args)
     : Containment(parent, args),
       m_orientation(Qt::Vertical),
+      m_expandAll(false),
       m_appletOverlay(0),
-      m_dragging(0),
-      m_leftArrow(0),
-      m_rightArrow(0)
+      m_dragging(0)
 {
     setContainmentType(Containment::CustomContainment);
 
+    m_scrollWidget = new AppletsView(this);
+    m_container = new AppletsContainer(m_scrollWidget);
+
     connect(this, SIGNAL(appletRemoved(Plasma::Applet*)),
             this, SLOT(refreshLayout()));
 
 
     connect(this, SIGNAL(toolBoxVisibilityChanged(bool)), this, SLOT(updateConfigurationMode(bool)));
-
-    m_updateSizeTimer = new QTimer(this);
-    m_updateSizeTimer->setSingleShot(true);
-    connect(m_updateSizeTimer, SIGNAL(timeout()), this, SLOT(updateSize()));
-
-    m_relayoutTimer = new QTimer(this);
-    m_relayoutTimer->setSingleShot(true);
-    connect(m_relayoutTimer, SIGNAL(timeout()), this, SLOT(updateSize()));
-    connect(m_relayoutTimer, SIGNAL(timeout()), this, SLOT(cleanupColumns()));
 }
 
 Newspaper::~Newspaper()
 {
+    delete m_appletOverlay;
     config().writeEntry("orientation", (int)m_orientation);
 }
 
@@ -92,18 +87,29 @@ void Newspaper::init()
 {
     m_externalLayout = new QGraphicsLinearLayout(this);
     m_externalLayout->setContentsMargins(0, 0, 0, 0);
-    m_scrollWidget = new Plasma::ScrollWidget(this);
     m_externalLayout->addItem(m_scrollWidget);
-    m_mainWidget = new QGraphicsWidget(m_scrollWidget);
-    m_scrollWidget->setWidget(m_mainWidget);
+    m_scrollWidget->setAppletsContainer(m_container);
+    connect(m_container, SIGNAL(appletActivated(Plasma::Applet *)), this, SLOT(appletActivated(Plasma::Applet *)));
+
+    m_updateSizeTimer = new QTimer(this);
+    m_updateSizeTimer->setSingleShot(true);
+    connect(m_updateSizeTimer, SIGNAL(timeout()), m_container, SLOT(updateSize()));
+
+    m_relayoutTimer = new QTimer(this);
+    m_relayoutTimer->setSingleShot(true);
+    connect(m_relayoutTimer, SIGNAL(timeout()), m_container, SLOT(updateSize()));
+    connect(m_relayoutTimer, SIGNAL(timeout()), m_container, SLOT(cleanupColumns()));
+    connect(m_container, SIGNAL(appletSizeHintChanged()), this, SLOT(appletSizeHintChanged()));
 
     m_orientation = (Qt::Orientation)config().readEntry("orientation", (int)Qt::Vertical);
+    m_container->setOrientation(m_orientation);
 
-    m_mainLayout = new QGraphicsLinearLayout(m_mainWidget);
-    addColumn();
+    m_expandAll = config().readEntry("ExpandAllApplets", false);
+    m_container->setExpandAll(m_expandAll);
+
+    m_container->addColumn();
     setOrientation(m_orientation);
 
-
     Plasma::Svg *borderSvg = new Plasma::Svg(this);
     borderSvg->setImagePath("newspaper/border");
 
@@ -121,10 +127,11 @@ void Newspaper::init()
         m_toolBox->addTool(a);
     }
 
-    a = new QAction(KIcon("view-pim-news"), i18n("Add page"), this);
-    addAction("add page", a);
+    a = new QAction(KIcon("view-fullscreen"), i18n("Expand widgets"), this);
+    addAction("expand widgets", a);
     m_toolBox->addTool(a);
-    connect(a, SIGNAL(triggered()), this, SLOT(addNewsPaper()));
+    connect(a, SIGNAL(triggered()), this, SLOT(toggleExpandAllApplets()));
+
 
     a = action("configure");
     if (a) {
@@ -132,23 +139,56 @@ void Newspaper::init()
         m_toolBox->addTool(a);
     }
 
-    KAction *lockAction = new KAction(this);
-    addAction("lock page", lockAction);
-    lockAction->setText(i18n("Lock page"));
-    lockAction->setIcon(KIcon("object-locked"));
-    QObject::connect(lockAction, SIGNAL(triggered(bool)), this, SLOT(toggleImmutability()));
+
+    QAction *lockAction = 0;
+    if (corona()) {
+        lockAction = corona()->action("lock widgets");
+    }
+
+    if (!lockAction || !lockAction->isEnabled()) {
+        lockAction = new QAction(this);
+        addAction("lock page", lockAction);
+        lockAction->setText(i18n("Lock page"));
+        lockAction->setIcon(KIcon("object-locked"));
+        QObject::connect(lockAction, SIGNAL(triggered(bool)), this, SLOT(toggleImmutability()));
+    }
     m_toolBox->addTool(lockAction);
 
-    a = action("remove");
-    if (a) {
-        a->setText(i18n("Remove page"));
+
+    //FIXME: two different use cases for the desktop and the newspaper, another reason to move the toolbox management out of here
+    QAction *activityAction = 0;
+    if (corona()) {
+        activityAction = corona()->action("manage activities");
+    }
+    if (activityAction) {
+        m_toolBox->addTool(activityAction);
+    } else {
+        a = new QAction(KIcon("view-pim-news"), i18n("Add page"), this);
+        addAction("add page", a);
         m_toolBox->addTool(a);
+        connect(a, SIGNAL(triggered()), this, SLOT(addNewsPaper()));
+
+        a = action("remove");
+        if (a) {
+            a->setText(i18n("Remove page"));
+            m_toolBox->addTool(a);
+        }
     }
 
     a = new QAction(i18n("Next activity"), this);
     addAction("next containment", a);
     a = new QAction(i18n("Previous activity"), this);
     addAction("previous containment", a);
+
+    if (corona()) {
+        connect(corona(), SIGNAL(availableScreenRegionChanged()), this, SLOT(availableScreenRegionChanged()));
+        availableScreenRegionChanged();
+    }
+}
+
+void Newspaper::updateSize()
+{
+    m_container->updateSize();
 }
 
 void Newspaper::toggleImmutability()
@@ -160,90 +200,24 @@ void Newspaper::toggleImmutability()
     }
 }
 
-void Newspaper::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
+void Newspaper::toggleExpandAllApplets()
 {
-    QGraphicsLinearLayout *lay = 0;
+    m_expandAll = !m_expandAll;
 
-    for (int i = 0; i < m_mainLayout->count(); ++i) {
-        QGraphicsLinearLayout *candidateLay = dynamic_cast<QGraphicsLinearLayout *>(m_mainLayout->itemAt(i));
+    QAction *a = action("expand widgets");
 
-        //normally should never happen
-        if (!candidateLay) {
-            continue;
-        }
-
-        if (m_orientation == Qt::Horizontal) {
-            if (pos.y() < candidateLay->geometry().bottom()) {
-                lay = candidateLay;
-                break;
-            }
-        //vertical
+    if (a) {
+        if (m_expandAll) {
+            a->setIcon(KIcon("view-restore"));
+            a->setText(i18n("Collapse widgets"));
         } else {
-            if (pos.x() < candidateLay->geometry().right()) {
-                lay = candidateLay;
-                break;
-            }
+            a->setIcon(KIcon("view-fullscreen"));
+            a->setText(i18n("Expand widgets"));
         }
     }
 
-    //couldn't decide: is the last column empty?
-    if (!lay) {
-        QGraphicsLinearLayout *candidateLay = dynamic_cast<QGraphicsLinearLayout *>(m_mainLayout->itemAt(m_mainLayout->count()-1));
-
-        if (candidateLay && candidateLay->count() == 1) {
-            lay = candidateLay;
-        }
-    }
-
-    //give up, make a new column
-    if (!lay) {
-        lay = addColumn();
-    }
-
-    int insertIndex = -1;
-
-    QPointF localPos = mapToItem(m_mainWidget, pos);
-
-    //if localPos is (-1,-1) insert at the end of the Newspaper
-    if (localPos != QPoint(-1, -1)) {
-        for (int i = 0; i < lay->count(); ++i) {
-            QGraphicsLayoutItem *li = lay->itemAt(i);
-
-            QRectF siblingGeometry = li->geometry();
-            if (m_orientation == Qt::Horizontal) {
-                qreal middle = (siblingGeometry.left() + siblingGeometry.right()) / 2.0;
-                if (localPos.x() < middle) {
-                    insertIndex = i;
-                    break;
-                } else if (localPos.x() <= siblingGeometry.right()) {
-                    insertIndex = i + 1;
-                    break;
-                }
-            } else { //Vertical
-                qreal middle = (siblingGeometry.top() + siblingGeometry.bottom()) / 2.0;
-
-                if (localPos.y() < middle) {
-                    insertIndex = i;
-                    break;
-                } else if (localPos.y() <= siblingGeometry.bottom()) {
-                    insertIndex = i + 1;
-                    break;
-                }
-            }
-        }
-    }
-
-
-    if (insertIndex == -1) {
-        lay->insertItem(lay->count()-1, applet);
-    } else {
-        lay->insertItem(qMin(insertIndex, lay->count()-1), applet);
-    }
-
-    connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(appletSizeHintChanged()));
-    updateSize();
-    createAppletTitle(applet);
-    syncColumnSizes();
+    m_container->setExpandAll(m_expandAll);
+    config().writeEntry("ExpandAllApplets", m_expandAll);
 }
 
 void Newspaper::refreshLayout()
@@ -253,62 +227,20 @@ void Newspaper::refreshLayout()
     }
 }
 
-void Newspaper::cleanupColumns()
-{
-    //clean up all empty columns
-    for (int i = 0; i < m_mainLayout->count(); ++i) {
-        QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(m_mainLayout->itemAt(i));
-
-        if (!lay) {
-            continue;
-        }
-
-        if (lay->count() == 1) {
-            removeColumn(i);
-        }
-    }
-}
-
 void Newspaper::setOrientation(Qt::Orientation orientation)
 {
     m_orientation = orientation;
-    m_mainLayout->setOrientation(orientation==Qt::Vertical?Qt::Horizontal:Qt::Vertical);
+    m_container->setOrientation(m_orientation);
     m_externalLayout->setOrientation(m_orientation);
 
     if (m_orientation == Qt::Vertical) {
-        m_mainWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-        if (m_leftArrow) {
-            m_leftArrow->deleteLater();
-            m_rightArrow->deleteLater();
-        }
+        m_container->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
     } else {
-        m_mainWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
-        if (!m_leftArrow) {
-            m_leftArrow = new Plasma::ToolButton(this);
-            m_leftArrow->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
-            m_leftArrow->setPreferredWidth(KIconLoader::SizeMedium);
-            m_leftArrow->setImage("widgets/arrows", "left-arrow");
-            connect(m_leftArrow, SIGNAL(clicked()), this, SLOT(goLeft()));
-            connect(m_leftArrow, SIGNAL(pressed()), this, SLOT(scrollTimeout()));
-
-            m_rightArrow = new Plasma::ToolButton(this);
-            m_rightArrow->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
-            m_rightArrow->setPreferredWidth(KIconLoader::SizeMedium);
-            m_rightArrow->setImage("widgets/arrows", "right-arrow");
-            connect(m_rightArrow, SIGNAL(clicked()), this, SLOT(goRight()));
-            connect(m_rightArrow, SIGNAL(pressed()), this, SLOT(scrollTimeout()));
-
-            m_externalLayout->insertItem(0, m_leftArrow);
-            m_externalLayout->addItem(m_rightArrow);
-
-            m_scrollTimer = new QTimer(this);
-            m_scrollTimer->setSingleShot(false);
-            connect(m_scrollTimer, SIGNAL(timeout()), this, SLOT(scrollTimeout()));
-        }
+        m_container->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
     }
 
-    for (int i = 0; i < m_mainLayout->count(); ++i) {
-        QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(m_mainLayout->itemAt(i));
+    for (int i = 0; i < m_container->count(); ++i) {
+        QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(m_container->itemAt(i));
 
         if (!lay) {
             continue;
@@ -318,39 +250,22 @@ void Newspaper::setOrientation(Qt::Orientation orientation)
     }
 }
 
-void Newspaper::goRight()
+Plasma::Applet *Newspaper::addApplet(const QString &appletName, const int row, const int column)
 {
-    QGraphicsSceneWheelEvent ev(QEvent::GraphicsSceneWheel);
-    ev.setDelta(-120);
-    scene()->sendEvent(m_scrollWidget, &ev);
-
-    if (m_mainWidget->geometry().right()-2 <= m_scrollWidget->viewportGeometry().right() && sender() == m_rightArrow) {
-        action("next containment")->trigger();
-    }
+    m_container->setAutomaticAppletLayout(false);
+    Plasma::Applet *applet = Containment::addApplet(appletName);
+    m_container->addApplet(applet, row, column);
+    m_container->setAutomaticAppletLayout(true);
+    return applet;
 }
 
-void Newspaper::goLeft()
+Plasma::Applet *Newspaper::addApplet(Plasma::Applet *applet, const int row, const int column)
 {
-    QGraphicsSceneWheelEvent ev(QEvent::GraphicsSceneWheel);
-    ev.setDelta(120);
-    scene()->sendEvent(m_scrollWidget, &ev);
-
-    if (m_mainWidget->geometry().left() >= -2 && sender() == m_leftArrow) {
-        action("previous containment")->trigger();
-    }
-}
-
-void Newspaper::scrollTimeout()
-{
-    if (!m_scrollTimer->isActive()) {
-        m_scrollTimer->start(250);
-    } else if (m_leftArrow->isDown()) {
-        goLeft();
-    } else if (m_rightArrow->isDown()) {
-        goRight();
-    } else {
-        m_scrollTimer->stop();
-    }
+    m_container->setAutomaticAppletLayout(false);
+    Containment::addApplet(applet);
+    m_container->addApplet(applet, row, column);
+    m_container->setAutomaticAppletLayout(true);
+    return applet;
 }
 
 Qt::Orientation Newspaper::orientation() const
@@ -365,41 +280,13 @@ void Newspaper::appletSizeHintChanged()
     }
 }
 
-void Newspaper::updateSize()
-{
-    QSizeF hint = m_mainWidget->effectiveSizeHint(Qt::PreferredSize);
-
-    //FIXME: it appears to work only with hardcoded values
-    if (m_orientation == Qt::Horizontal) {
-        m_mainWidget->resize(qMax((int)hint.width(), 300), m_mainWidget->size().height());
-    } else {
-        m_mainWidget->resize(m_mainWidget->size().width(), qMax((int)hint.height(), 300));
-    }
-}
-
-//FIXME: this is really bad, but without hardcoded maximum sizes the cpu usage goes nuts, causing continuous resizes for a minute or so
-void Newspaper::syncColumnSizes()
-{
-    for (int i = 0; i < m_mainLayout->count(); ++i) {
-        QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(m_mainLayout->itemAt(i));
-
-        if (m_orientation == Qt::Vertical) {
-            lay->setMaximumWidth(m_scrollWidget->viewportGeometry().size().width()/m_mainLayout->count());
-            lay->setMaximumHeight(QWIDGETSIZE_MAX);
-        } else {
-            lay->setMaximumHeight(m_scrollWidget->viewportGeometry().size().height()/m_mainLayout->count());
-            lay->setMaximumWidth(QWIDGETSIZE_MAX);
-        }
-    }
-}
-
 void Newspaper::constraintsEvent(Plasma::Constraints constraints)
 {
     kDebug() << "constraints updated with" << constraints << "!!!!!!";
 
     if (constraints & Plasma::StartupCompletedConstraint) {
         connect(this, SIGNAL(appletAdded(Plasma::Applet*,QPointF)),
-                this, SLOT(layoutApplet(Plasma::Applet*,QPointF)));
+                m_container, SLOT(layoutApplet(Plasma::Applet*,QPointF)));
         Plasma::Corona *c = corona();
         if (c) {
             connect(c, SIGNAL(containmentAdded(Plasma::Containment *)),
@@ -419,9 +306,10 @@ void Newspaper::constraintsEvent(Plasma::Constraints constraints)
     if (constraints & Plasma::SizeConstraint && m_appletOverlay) {
         m_appletOverlay->resize(size());
     }
-    
+
     if (constraints & Plasma::SizeConstraint) {
-        syncColumnSizes();
+        availableScreenRegionChanged();
+        m_container->syncColumnSizes();
     }
 
     if (constraints & Plasma::ImmutableConstraint) {
@@ -506,17 +394,11 @@ void Newspaper::updateConfigurationMode(bool config)
     m_externalLayout->setContentsMargins(extraLeft, extraTop, extraRight, extraBottom);
 
     if (!config) {
-        cleanupColumns();
+        m_container->cleanupColumns();
     }
 
 }
 
-void Newspaper::createAppletTitle(Plasma::Applet *applet)
-{
-    AppletTitleBar *appletTitleBar = new AppletTitleBar(applet);
-    appletTitleBar->show();
-}
-
 void Newspaper::changeEvent(QEvent *event)
 {
     if (event->type() == QEvent::ContentsRectChange) {
@@ -596,10 +478,10 @@ void Newspaper::restore(KConfigGroup &group)
     }
 
     //if the required columns does not exist, create them
-    if (m_mainLayout->count() < orderedApplets.count()) {
-        int columnsToAdd = orderedApplets.count()-m_mainLayout->count();
+    if (m_container->count() < orderedApplets.count()) {
+        int columnsToAdd = orderedApplets.count()-m_container->count();
         for (int i = 0; i < columnsToAdd; ++i) {
-            addColumn();
+            m_container->addColumn();
         }
     }
 
@@ -608,7 +490,7 @@ void Newspaper::restore(KConfigGroup &group)
     QMap<int, QMap<int, Applet *> >::const_iterator it = orderedApplets.constBegin();
 
     while (it != orderedApplets.constEnd()) {
-        QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(m_mainLayout->itemAt(column));
+        QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(m_container->itemAt(column));
         ++column;
 
         //this should never happen
@@ -619,17 +501,17 @@ void Newspaper::restore(KConfigGroup &group)
 
         foreach (Applet *applet, it.value()) {
             lay->insertItem(lay->count()-1, applet);
-            createAppletTitle(applet);
+            m_container->createAppletTitle(applet);
         }
         ++it;
     }
 
     //add all the remaining applets
     foreach (Applet *applet, unorderedApplets) {
-        layoutApplet(applet, applet->pos());
+        m_container->layoutApplet(applet, applet->pos());
     }
 
-    updateSize();
+    m_container->updateSize();
 }
 
 void Newspaper::saveContents(KConfigGroup &group) const
@@ -637,8 +519,8 @@ void Newspaper::saveContents(KConfigGroup &group) const
     Containment::saveContents(group);
 
     KConfigGroup appletsConfig(&group, "Applets");
-    for (int column = 0; column < m_mainLayout->count(); ++column) {
-        QGraphicsLinearLayout *lay = static_cast<QGraphicsLinearLayout *>(m_mainLayout->itemAt(column));
+    for (int column = 0; column < m_container->count(); ++column) {
+        QGraphicsLinearLayout *lay = static_cast<QGraphicsLinearLayout *>(m_container->itemAt(column));
         for (int row = 0; row < lay->count(); ++row) {
             const Applet *applet = dynamic_cast<Applet *>(lay->itemAt(row));
             if (applet) {
@@ -652,50 +534,6 @@ void Newspaper::saveContents(KConfigGroup &group) const
     }
 }
 
-QGraphicsLinearLayout *Newspaper::addColumn()
-{
-    QGraphicsLinearLayout *lay = new QGraphicsLinearLayout(m_orientation);
-    m_mainLayout->addItem(lay);
-
-    QGraphicsWidget *spacer = new QGraphicsWidget(m_mainWidget);
-    spacer->setPreferredSize(0, 0);
-    spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-    lay->addItem(spacer);
-
-    syncColumnSizes();
-
-    return lay;
-}
-
-void Newspaper::removeColumn(int column)
-{
-    QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout *>(m_mainLayout->itemAt(column));
-
-    if (!lay) {
-        return;
-    }
-
-    m_mainLayout->removeAt(column);
-
-    for (int i = 0; i < lay->count(); ++i) {
-        QGraphicsLayoutItem *item = lay->itemAt(i);
-        QGraphicsWidget *widget = dynamic_cast<QGraphicsWidget *>(item);
-        Plasma::Applet *applet = qobject_cast<Plasma::Applet *>(widget);
-
-        //find a new home for the applet
-        if (applet) {
-            layoutApplet(applet, applet->pos());
-        //delete spacers
-        } else if (widget) {
-            widget->deleteLater();
-        }
-    }
-
-    syncColumnSizes();
-
-    delete lay;
-}
-
 void Newspaper::updateRemoveActionVisibility()
 {
     int newspapers = 0;
@@ -713,6 +551,11 @@ void Newspaper::updateRemoveActionVisibility()
     }
 }
 
+void Newspaper::appletActivated(Plasma::Applet *applet)
+{
+    m_scrollWidget->ensureItemVisible(applet);
+}
+
 void Newspaper::containmentAdded(Plasma::Containment *containment)
 {
     //we now are sure there are at least two pages
@@ -727,6 +570,33 @@ void Newspaper::containmentAdded(Plasma::Containment *containment)
     }
 }
 
+void Newspaper::availableScreenRegionChanged()
+{
+    if (!corona()) {
+        return;
+    }
+
+    QRect maxRect;
+    int maxArea = 0;
+    //we don't want the bounding rect (that could include panels too), but the maximumone representing the desktop
+    foreach (QRect rect, corona()->availableScreenRegion(screen()).rects()) {
+        int area = rect.width() * rect.height();
+        if (area > maxArea) {
+            maxRect = rect;
+            maxArea = area;
+        }
+    }
+
+    QGraphicsView *ownView = view();
+    
+    //FIXME: the second check is a workaround to a qt bug: when a qwidget has just been created, maptoglobal and mapfromglobal aren't symmetryc. remove as soon as the bug is fixed
+    if (ownView && (ownView->mapFromGlobal(QPoint(0,0)) == -ownView->mapToGlobal(QPoint(0,0)))) {
+        maxRect.moveTopLeft(ownView->mapFromGlobal(maxRect.topLeft()));
+    }
+
+    setContentsMargins(maxRect.left(), maxRect.top(), qMax((qreal)0.0, size().width() - maxRect.right()), qMax((qreal)0.0, size().height() - maxRect.bottom()));
+}
+
 void Newspaper::containmentRemoved(QObject *containment)
 {
     if (!corona()) {
