@@ -26,12 +26,17 @@
 #include "../common/appletmovespacer.h"
 #include "../common/nettoolbox.h"
 #include "iconactioncollection.h"
+#include "models/commonmodel.h"
+#include "models/kservicemodel.h"
 
 #include <QAction>
 #include <QTimer>
 #include <QGraphicsSceneMouseEvent>
 #include <QGraphicsAnchorLayout>
+#include <QGraphicsView>
 #include <QApplication>
+#include <QFileInfo>
+#include <QListView>
 
 #include <KAction>
 #include <KDebug>
@@ -42,6 +47,8 @@
 #include <KService>
 #include <KServiceTypeTrader>
 #include <KConfigDialog>
+#include <KDesktopFile>
+#include <KRun>
 
 #include <Plasma/Theme>
 #include <Plasma/Frame>
@@ -52,20 +59,17 @@
 #include <Plasma/QueryMatch>
 #include <Plasma/ScrollWidget>
 #include <Plasma/ToolButton>
-#include <Plasma/ToolTipContent>
-#include <Plasma/ToolTipManager>
 
 
 SearchLaunch::SearchLaunch(QObject *parent, const QVariantList &args)
     : Containment(parent, args),
+      m_runnerModel(0),
       m_backButton(0),
       m_queryCounter(0),
       m_maxColumnWidth(0),
       m_searchField(0),
       m_resultsView(0),
       m_orientation(Qt::Vertical),
-      m_leftArrow(0),
-      m_rightArrow(0),
       m_firstItem(0),
       m_appletsLayout(0),
       m_appletOverlay(0),
@@ -88,8 +92,6 @@ SearchLaunch::~SearchLaunch()
     KConfigGroup cg = config();
     m_stripWidget->save(cg);
     config().writeEntry("orientation", (int)m_orientation);
-
-    delete m_runnermg;
 }
 
 void SearchLaunch::init()
@@ -102,68 +104,203 @@ void SearchLaunch::init()
 
     connect(this, SIGNAL(toolBoxVisibilityChanged(bool)), this, SLOT(updateConfigurationMode(bool)));
 
-    m_runnermg = new Plasma::RunnerManager(this);
-    m_runnermg->reloadConfiguration();
-    connect(m_runnermg, SIGNAL(matchesChanged(const QList<Plasma::QueryMatch>&)),
-            this, SLOT(setQueryMatches(const QList<Plasma::QueryMatch>&)));
-
     m_toolBox = new NetToolBox(this);
     setToolBox(m_toolBox);
     connect(m_toolBox, SIGNAL(toggled()), this, SIGNAL(toolBoxToggled()));
     connect(m_toolBox, SIGNAL(visibilityChanged(bool)), this, SIGNAL(toolBoxVisibilityChanged(bool)));
     m_toolBox->show();
 
-    m_orientation = (Qt::Orientation)config().readEntry("orientation", (int)Qt::Vertical);
-
     QAction *a = action("add widgets");
     if (a) {
         m_toolBox->addTool(a);
     }
 
+
     a = action("configure");
     if (a) {
         m_toolBox->addTool(a);
         a->setText(i18n("Configure Search and Launch"));
     }
 
-    KAction *lockAction = new KAction(this);
-    addAction("lock page", lockAction);
-    lockAction->setText(i18n("Lock Page"));
-    lockAction->setIcon(KIcon("object-locked"));
-    QObject::connect(lockAction, SIGNAL(triggered(bool)), this, SLOT(toggleImmutability()));
-    m_toolBox->addTool(lockAction);
 
-    KService::List services = KServiceTypeTrader::self()->query("Plasma/Sal/Menu");
-    if (!services.isEmpty()) {
-        foreach (const KService::Ptr &service, services) {
-            const QString query = service->property("X-Plasma-Sal-Query", QVariant::String).toString();
-            const QString runner = service->property("X-Plasma-Sal-Runner", QVariant::String).toString();
-            const int relevance = service->property("X-Plasma-Sal-Relevance", QVariant::Int).toInt();
-
-            Plasma::QueryMatch match(0);
-            match.setType(Plasma::QueryMatch::ExactMatch);
-            match.setRelevance(relevance);
-            match.setId(query);
-            match.setIcon(KIcon(service->icon()));
-            match.setText(service->name());
-            match.setSubtext(service->comment());
-            QStringList data;
-            data << query << runner;
-            match.setData(data);
-
-            m_defaultMatches.append(match);
-        }
+    QAction *lockAction = 0;
+    if (corona()) {
+        lockAction = corona()->action("lock widgets");
     }
 
-    //SAL doesn't want to be removed:
-    //FIXME: wise to do it here?
-    a = action("remove");
-    delete a;
+    if (!lockAction || !lockAction->isEnabled()) {
+        lockAction = new QAction(this);
+        addAction("lock page", lockAction);
+        lockAction->setText(i18n("Lock Page"));
+        lockAction->setIcon(KIcon("object-locked"));
+        QObject::connect(lockAction, SIGNAL(triggered(bool)), this, SLOT(toggleImmutability()));
+    }
+
+    m_toolBox->addTool(lockAction);
+
+    //FIXME: two different use cases for the desktop and the newspaper, another reason to move the toolbox management out of here
+    QAction *activityAction = 0;
+    if (corona()) {
+        activityAction = corona()->action("manage activities");
+    }
+    if (activityAction) {
+        m_toolBox->addTool(activityAction);
+    }
 
     a = new QAction(i18n("Next activity"), this);
     addAction("next containment", a);
     a = new QAction(i18n("Previous activity"), this);
     addAction("previous containment", a);
+
+    if (corona()) {
+        connect(corona(), SIGNAL(availableScreenRegionChanged()), this, SLOT(availableScreenRegionChanged()));
+        availableScreenRegionChanged();
+    }
+
+    Plasma::FormFactor form = formFactor();
+    Qt::Orientation layoutOtherDirection = form == Plasma::Vertical ? \
+            Qt::Horizontal : Qt::Vertical;
+
+    // create main layout
+    m_mainLayout = new QGraphicsLinearLayout();
+    m_mainLayout->setOrientation(layoutOtherDirection);
+    m_mainLayout->setContentsMargins(0, 0, 0, 0);
+    m_mainLayout->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
+                                            QSizePolicy::Expanding));
+    setLayout(m_mainLayout);
+
+    // create launch grid and make it centered
+    m_resultsLayout = new QGraphicsLinearLayout;
+
+
+    m_resultsView = new ItemView(this);
+
+    m_resultsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    m_resultsView->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+    m_resultsLayout->addItem(m_resultsView);
+
+    connect(m_resultsView, SIGNAL(dragStartRequested(QModelIndex)), this, SLOT(resultsViewRequestedDrag(QModelIndex)));
+    connect(m_resultsView, SIGNAL(itemActivated(QModelIndex)), this, SLOT(launch(QModelIndex)));
+    connect(m_resultsView, SIGNAL(addActionTriggered(const QModelIndex &)), this, SLOT(addFavourite(const QModelIndex &)));
+
+
+    //TODO how to do the strip widget?
+    m_stripWidget = new StripWidget(this);
+    m_stripWidget->setImmutability(immutability());
+    connect(m_stripWidget, SIGNAL(saveNeeded()), this, SLOT(saveFavourites()));
+
+    //load all config, only at this point we are sure it won't crash
+    configChanged();
+
+    m_appletsLayout = new QGraphicsLinearLayout(Qt::Horizontal);
+    m_appletsLayout->setPreferredHeight(KIconLoader::SizeMedium);
+    m_appletsLayout->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+    QGraphicsWidget *leftSpacer = new QGraphicsWidget(this);
+    QGraphicsWidget *rightSpacer = new QGraphicsWidget(this);
+    m_appletsLayout->addItem(leftSpacer);
+    m_appletsLayout->addItem(rightSpacer);
+
+    m_backButton = new Plasma::IconWidget(this);
+    m_backButton->setIcon(KIcon("go-previous"));
+    m_backButton->setText(i18n("Back"));
+    m_backButton->setOrientation(Qt::Horizontal);
+    m_backButton->setPreferredSize(m_backButton->sizeFromIconSize(KIconLoader::SizeSmallMedium));
+    connect(m_backButton, SIGNAL(clicked()), this, SLOT(reset()));
+    connect(m_resultsView, SIGNAL(resetRequested()), this, SLOT(reset()));
+
+    QGraphicsAnchorLayout *searchLayout = new QGraphicsAnchorLayout();
+    searchLayout->setSpacing(10);
+
+    m_searchField = new Plasma::LineEdit(this);
+    m_searchField->setPreferredWidth(200);
+    m_searchField->nativeWidget()->setClearButtonShown(true);
+    m_searchField->nativeWidget()->setClickMessage(i18n("Search..."));
+    connect(m_searchField, SIGNAL(returnPressed()), this, SLOT(searchReturnPressed()));
+    connect(m_searchField->nativeWidget(), SIGNAL(textEdited(const QString &)), this, SLOT(delayedQuery()));
+    m_searchTimer = new QTimer(this);
+    m_searchTimer->setSingleShot(true);
+    connect(m_searchTimer, SIGNAL(timeout()), this, SLOT(query()));
+    searchLayout->addAnchor(m_searchField, Qt::AnchorHorizontalCenter, searchLayout, Qt::AnchorHorizontalCenter);
+    searchLayout->addAnchors(m_searchField, searchLayout, Qt::Vertical);
+    searchLayout->addAnchors(m_backButton, searchLayout, Qt::Vertical);
+    searchLayout->addAnchor(m_backButton, Qt::AnchorRight, m_searchField, Qt::AnchorLeft);
+
+
+    // add our layouts to main vertical layout
+    m_mainLayout->addItem(m_stripWidget);
+    m_mainLayout->addItem(searchLayout);
+    m_mainLayout->addItem(m_resultsLayout);
+
+
+    // correctly set margins
+    m_mainLayout->activate();
+    m_mainLayout->updateGeometry();
+
+    setTabOrder(m_stripWidget, m_searchField);
+    setTabOrder(m_searchField, m_resultsView);
+    setFormFactorFromLocation(location());
+
+    if (action("remove")) {
+        m_toolBox->addTool(action("remove"));
+    }
+}
+
+void SearchLaunch::launchPackageManager()
+{
+    m_toolBox->setShowing(false);
+    KRun::run(*m_packageManagerService.data(), KUrl::List(), 0);
+}
+
+void SearchLaunch::configChanged()
+{
+    setOrientation((Qt::Orientation)config().readEntry("Orientation", (int)Qt::Vertical));
+
+    m_stripWidget->setIconSize(config().readEntry("FavouritesIconSize", (int)KIconLoader::SizeLarge));
+
+    m_resultsView->setIconSize(config().readEntry("ResultsIconSize", (int)KIconLoader::SizeHuge));
+
+    QString packageManagerName = config().readEntry("PackageManager", "kpackagekit");
+
+    if (!packageManagerName.isEmpty()) {
+        m_packageManagerService = KService::serviceByDesktopName(packageManagerName);
+
+        if (!action("add applications") && m_packageManagerService && !m_packageManagerService->exec().isEmpty()) {
+            KAction *addApplicationsAction = new KAction(this);
+            addAction("add applications", addApplicationsAction);
+            addApplicationsAction->setText(i18n("Add applications"));
+            addApplicationsAction->setIcon(KIcon("applications-other"));
+            m_toolBox->addTool(addApplicationsAction);
+
+            connect(addApplicationsAction, SIGNAL(triggered()), this, SLOT(launchPackageManager()));
+        }
+    }
+}
+
+void SearchLaunch::availableScreenRegionChanged()
+{
+    if (!corona()) {
+        return;
+    }
+
+    QRect maxRect;
+    int maxArea = 0;
+    //we don't want the bounding rect (that could include panels too), but the maximumone representing the desktop
+    foreach (QRect rect, corona()->availableScreenRegion(screen()).rects()) {
+        int area = rect.width() * rect.height();
+        if (area > maxArea) {
+            maxRect = rect;
+            maxArea = area;
+        }
+    }
+
+    QGraphicsView *ownView = view();
+    
+    //FIXME: the second check is a workaround to a qt bug: when a qwidget has just been created, maptoglobal and mapfromglobal aren't symmetryc. remove as soon as the bug is fixed
+    if (ownView && (ownView->mapFromGlobal(QPoint(0,0)) == -ownView->mapToGlobal(QPoint(0,0)))) {
+        maxRect.moveTopLeft(ownView->mapFromGlobal(maxRect.topLeft()));
+    }
+
+    setContentsMargins(maxRect.left(), maxRect.top(), qMax((qreal)0.0, size().width() - maxRect.right()), qMax((qreal)0.0, size().height() - maxRect.bottom()));
 }
 
 void SearchLaunch::toggleImmutability()
@@ -177,122 +314,70 @@ void SearchLaunch::toggleImmutability()
 
 void SearchLaunch::doSearch(const QString &query, const QString &runner)
 {
-    m_queryCounter = 0;
-    m_firstItem = 0;
-    m_resultsView->clear();
-
-    const bool stillEmpty = query.isEmpty() && m_runnermg->query().isEmpty();
-    if (!stillEmpty) {
-        m_resultsView->clear();
+    if (!query.isEmpty()) {
+        m_resultsView->setModel(m_runnerModel);
+    } else {
+        m_resultsView->setModel(m_serviceModel);
+        m_serviceModel->setPath("/");
     }
 
-    m_maxColumnWidth = 0;
-    m_matches.clear();
-
-    m_runnermg->launchQuery(query, runner);
-
-    if (m_resultsView && query.isEmpty()) {
-        if (m_resultsView->count()) {
-            m_resultsView->clear();
-        }
-
-        setQueryMatches(m_defaultMatches);
-        m_backButton->hide();
-    } else if (m_backButton) {
-        m_backButton->show();
+    m_runnerModel->setQuery(query, runner);
+    m_lastQuery = query;
+    //enable or disable drag and drop
+    if (immutability() == Plasma::Mutable && (m_resultsView->model() != m_serviceModel || m_serviceModel->path() != "/")) {
+        m_resultsView->setDragAndDropMode(ItemContainer::CopyDragAndDrop);
+    } else {
+        m_resultsView->setDragAndDropMode(ItemContainer::NoDragAndDrop);
     }
 }
 
 void SearchLaunch::reset()
 {
-    if (!m_runnermg->query().isEmpty()) {
+    if (m_resultsView->model() != m_serviceModel || m_serviceModel->path() != "/") {
         m_searchField->setText(QString());
         doSearch(QString());
+        m_serviceModel->setPath("/");
+        m_resultsView->setModel(m_serviceModel);
     }
 }
 
-void SearchLaunch::setQueryMatches(const QList<Plasma::QueryMatch> &matches)
+void SearchLaunch::launch(QModelIndex index)
 {
-    if (matches.isEmpty()) {
-        return;
-    }
+    KUrl url(index.data(CommonModel::Url).value<QString>());
 
-    // just add new QueryMatch
-
-    foreach (Plasma::QueryMatch match, matches) {
-
-        // create new IconWidget with information from the match
-        Plasma::IconWidget *icon = m_resultsView->createItem();
-        icon->setNumDisplayLines(4);
-        icon->hide();
-        icon->setOrientation(Qt::Vertical);
-        icon->setText(!match.text().isEmpty()?match.text():match.subtext());
-        icon->setIcon(match.icon());
-        Plasma::ToolTipContent toolTipData = Plasma::ToolTipContent();
-        toolTipData.setAutohide(true);
-        toolTipData.setMainText(match.text());
-        toolTipData.setSubText(match.subtext());
-        toolTipData.setImage(match.icon());
-
-        Plasma::ToolTipManager::self()->registerWidget(this);
-        Plasma::ToolTipManager::self()->setContent(icon, toolTipData);
-
-        m_resultsView->insertItem(icon, 1/match.relevance());
-
-        if (match.relevance() < 0.4) {
-            icon->setOrientation(Qt::Horizontal);
-            icon->setMinimumSize(icon->sizeFromIconSize(KIconLoader::SizeSmall));
-            icon->setMaximumSize(icon->sizeFromIconSize(KIconLoader::SizeSmall));
-        }
-
-        connect(icon, SIGNAL(activated()), this, SLOT(launch()));
-
-        if (!m_runnermg->searchContext()->query().isEmpty()) {
-            // create action to add to favourites strip
-            QAction *action = new QAction(icon);
-            action->setIcon(KIcon("favorites"));
-            icon->addIconAction(action);
-            m_iconActionCollection->addAction(action);
-            connect(action, SIGNAL(triggered()), this, SLOT(addFavourite()));
-        }
-
-        // add to data structure
-        m_matches.insert(icon, match);
-        if (!m_firstItem || match.relevance() > m_matches.value(m_firstItem, Plasma::QueryMatch(0)).relevance()) {
-            m_firstItem = icon;
-        }
-    }
-    m_queryCounter = matches.count();
-}
-
-void SearchLaunch::launch(Plasma::IconWidget *icon)
-{
-    Plasma::QueryMatch match = m_matches.value(icon, Plasma::QueryMatch(0));
-    if (m_runnermg->searchContext()->query().isEmpty()) {
-        QStringList data = match.data().value<QStringList>();
-
-        if (data.count() == 2) {
-            doSearch(data.first(), data.last());
-        } else {
-            doSearch(data.first());
-        }
-    } else {
-        m_runnermg->run(match);
+    if (m_resultsView->model() == m_runnerModel) {
+        KRunnerItemHandler::openUrl(url);
         reset();
+    } else {
+        QString id = url.path();
+        if (id.startsWith(QLatin1String("/"))) {
+            id = id.remove(0, 1);
+        }
+        if (url.protocol() == "kservicegroup") {
+            m_serviceModel->setPath(id);
+        } else if (url.protocol() == "krunner") {
+            m_resultsView->setModel(m_runnerModel);
+            m_runnerModel->setQuery(id, url.host());
+        } else {
+            KServiceItemHandler::openUrl(url);
+            reset();
+        }
+    }
+
+    //enable or disable drag and drop
+    if (immutability() == Plasma::Mutable && (m_resultsView->model() != m_serviceModel || m_serviceModel->path() != "/")) {
+        m_resultsView->setDragAndDropMode(ItemContainer::CopyDragAndDrop);
+    } else {
+        m_resultsView->setDragAndDropMode(ItemContainer::NoDragAndDrop);
     }
 }
 
-void SearchLaunch::launch()
+void SearchLaunch::addFavourite(const QModelIndex &index)
 {
-    Plasma::IconWidget *icon = static_cast<Plasma::IconWidget*>(sender());
-    launch(icon);
-}
-
-void SearchLaunch::addFavourite()
-{
-    Plasma::IconWidget *icon = static_cast<Plasma::IconWidget*>(sender()->parent());
-    Plasma::QueryMatch match = m_matches.value(icon, Plasma::QueryMatch(0));
-    m_stripWidget->add(match, m_runnermg->searchContext()->query());
+    QMimeData *mimeData = m_resultsView->model()->mimeData(QModelIndexList()<<index);
+    if (mimeData && !mimeData->urls().isEmpty()) {
+        m_stripWidget->add(mimeData->urls().first());
+    }
 }
 
 void SearchLaunch::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
@@ -349,7 +434,6 @@ void SearchLaunch::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
     }
 
     applet->setBackgroundHints(NoBackground);
-    connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updateSize()));
 }
 
 void SearchLaunch::appletRemoved(Plasma::Applet* applet)
@@ -361,115 +445,21 @@ void SearchLaunch::appletRemoved(Plasma::Applet* applet)
     }
 }
 
-void SearchLaunch::updateSize()
-{
-    Plasma::Applet *applet = qobject_cast<Plasma::Applet *>(sender());
-
-    if (applet) {
-        if (formFactor() == Plasma::Horizontal) {
-            const int delta = applet->preferredWidth() - applet->size().width();
-            //setting the preferred width when delta = 0 and preferredWidth() < minimumWidth()
-            // leads to the same thing as setPreferredWidth(minimumWidth())
-            if (delta != 0) {
-                setPreferredWidth(preferredWidth() + delta);
-            }
-        } else if (formFactor() == Plasma::Vertical) {
-            const int delta = applet->preferredHeight() - applet->size().height();
-            if (delta != 0) {
-                setPreferredHeight(preferredHeight() + delta);
-            }
-        }
-
-        resize(preferredSize());
-    }
-}
-
-
 void SearchLaunch::constraintsEvent(Plasma::Constraints constraints)
 {
-    kDebug() << "constraints updated with" << constraints << "!!!!!!";
+    //kDebug() << "constraints updated with" << constraints << "!!!!!!";
 
     if (constraints & Plasma::FormFactorConstraint ||
         constraints & Plasma::StartupCompletedConstraint) {
 
-        Plasma::FormFactor form = formFactor();
-
-        Qt::Orientation layoutOtherDirection = form == Plasma::Vertical ? \
-            Qt::Horizontal : Qt::Vertical;
-
-
-        // create our layout!
-        if (!layout()) {
-            // create main layout
-            m_mainLayout = new QGraphicsLinearLayout();
-            m_mainLayout->setOrientation(layoutOtherDirection);
-            m_mainLayout->setContentsMargins(0, 0, 0, 0);
-            m_mainLayout->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
-                                                    QSizePolicy::Expanding));
-            setLayout(m_mainLayout);
-
-            // create launch grid and make it centered
-            m_resultsLayout = new QGraphicsLinearLayout;
-
-
-            m_resultsView = new ItemView(this);
-
-            m_resultsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-            m_resultsView->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
-            m_resultsLayout->addItem(m_resultsView);
-            setOrientation(m_orientation);
-
-            connect(m_resultsView, SIGNAL(itemActivated(Plasma::IconWidget *)), this, SLOT(launch(Plasma::IconWidget *)));
-
-            m_stripWidget = new StripWidget(m_runnermg, this);
-            m_appletsLayout = new QGraphicsLinearLayout(Qt::Horizontal);
-            m_appletsLayout->setPreferredHeight(KIconLoader::SizeMedium);
-            m_appletsLayout->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
-            QGraphicsWidget *leftSpacer = new QGraphicsWidget(this);
-            QGraphicsWidget *rightSpacer = new QGraphicsWidget(this);
-            m_appletsLayout->addItem(leftSpacer);
-            m_appletsLayout->addItem(rightSpacer);
-
-            m_backButton = new Plasma::IconWidget(this);
-            m_backButton->setIcon(KIcon("go-previous"));
-            m_backButton->setText(i18n("Back"));
-            m_backButton->setOrientation(Qt::Horizontal);
-            m_backButton->setPreferredSize(m_backButton->sizeFromIconSize(KIconLoader::SizeSmallMedium));
-            connect(m_backButton, SIGNAL(activated()), this, SLOT(reset()));
-            connect(m_resultsView, SIGNAL(resetRequested()), this, SLOT(reset()));
-
-            QGraphicsAnchorLayout *searchLayout = new QGraphicsAnchorLayout();
-            searchLayout->setSpacing(10);
-
-            m_searchField = new Plasma::LineEdit(this);
-            m_searchField->setPreferredWidth(200);
-            m_searchField->nativeWidget()->setClearButtonShown(true);
-            m_searchField->nativeWidget()->setClickMessage(i18n("Enter your query here"));
-            connect(m_searchField, SIGNAL(returnPressed()), this, SLOT(searchReturnPressed()));
-            connect(m_searchField->nativeWidget(), SIGNAL(textEdited(const QString &)), this, SLOT(delayedQuery()));
-            m_searchTimer = new QTimer(this);
-            m_searchTimer->setSingleShot(true);
-            connect(m_searchTimer, SIGNAL(timeout()), this, SLOT(query()));
-            searchLayout->addAnchor(m_searchField, Qt::AnchorHorizontalCenter, searchLayout, Qt::AnchorHorizontalCenter);
-            searchLayout->addAnchors(m_searchField, searchLayout, Qt::Vertical);
-            searchLayout->addAnchors(m_backButton, searchLayout, Qt::Vertical);
-            searchLayout->addAnchor(m_backButton, Qt::AnchorRight, m_searchField, Qt::AnchorLeft);
-
-
-            // add our layouts to main vertical layout
-            m_mainLayout->addItem(m_stripWidget);
-            m_mainLayout->addItem(searchLayout);
-            m_mainLayout->addItem(m_resultsLayout);
-
-
-            // correctly set margins
-            m_mainLayout->activate();
-            m_mainLayout->updateGeometry();
-
-            setTabOrder(m_stripWidget, m_searchField);
-            setTabOrder(m_searchField, m_resultsView);
-            setFormFactorFromLocation(location());
+         // create the models
+        if (!m_runnerModel) {
+            m_runnerModel = new KRunnerModel(this);
+            m_serviceModel = new KServiceModel(config(), this);
+            m_resultsView->setModel(m_serviceModel);
         }
+        
+        resize(corona()->screenGeometry(screen()).size());
     }
 
     if (constraints & Plasma::LocationConstraint) {
@@ -477,6 +467,7 @@ void SearchLaunch::constraintsEvent(Plasma::Constraints constraints)
     }
 
     if (constraints & Plasma::SizeConstraint) {
+        availableScreenRegionChanged();
         if (m_appletsLayout) {
             m_appletsLayout->setMaximumHeight(size().height()/4);
         }
@@ -497,6 +488,7 @@ void SearchLaunch::constraintsEvent(Plasma::Constraints constraints)
     }
 
     if (constraints & Plasma::ImmutableConstraint) {
+        //update lock button
         QAction *a = action("lock page");
         if (a) {
             switch (immutability()) {
@@ -521,6 +513,7 @@ void SearchLaunch::constraintsEvent(Plasma::Constraints constraints)
             }
         }
 
+        //kill or create the config overlay if needed
         if (immutability() == Plasma::Mutable && !m_appletOverlay && m_toolBox->isShowing()) {
             m_appletOverlay = new LinearAppletOverlay(this, m_appletsLayout);
             m_appletOverlay->resize(size());
@@ -528,39 +521,25 @@ void SearchLaunch::constraintsEvent(Plasma::Constraints constraints)
             m_appletOverlay->deleteLater();
             m_appletOverlay = 0;
         }
+
+        //enable or disable drag and drop
+        if (immutability() == Plasma::Mutable && (m_resultsView->model() != m_serviceModel || m_serviceModel->path() != "/")) {
+            m_resultsView->setDragAndDropMode(ItemContainer::CopyDragAndDrop);
+        } else {
+            m_resultsView->setDragAndDropMode(ItemContainer::NoDragAndDrop);
+        }
+        m_stripWidget->setImmutability(immutability());
     }
 }
 
 void SearchLaunch::setOrientation(Qt::Orientation orientation)
 {
+    if (m_orientation == orientation) {
+        return;
+    }
+
     m_orientation = orientation;
     m_resultsView->setOrientation(orientation);
-    if (m_orientation == Qt::Vertical) {
-        if (m_leftArrow) {
-            m_leftArrow->deleteLater();
-            m_rightArrow->deleteLater();
-            m_leftArrow=0;
-            m_rightArrow=0;
-        }
-    } else {
-        if (!m_leftArrow) {
-            m_leftArrow = new Plasma::ToolButton(this);
-            m_leftArrow->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
-            m_leftArrow->setPreferredWidth(KIconLoader::SizeMedium);
-            m_leftArrow->setImage("widgets/arrows", "left-arrow");
-            connect(m_leftArrow, SIGNAL(clicked()), this, SLOT(goLeft()));
-            connect(m_leftArrow, SIGNAL(pressed()), this, SLOT(scrollTimeout()));
-
-            m_rightArrow = new Plasma::ToolButton(this);
-            m_rightArrow->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
-            m_rightArrow->setPreferredWidth(KIconLoader::SizeMedium);
-            m_rightArrow->setImage("widgets/arrows", "right-arrow");
-            connect(m_rightArrow, SIGNAL(clicked()), this, SLOT(goRight()));
-            connect(m_rightArrow, SIGNAL(pressed()), this, SLOT(scrollTimeout()));
-            m_resultsLayout->insertItem(0, m_leftArrow);
-            m_resultsLayout->addItem(m_rightArrow);
-        }
-    }
 }
 
 void SearchLaunch::setFormFactorFromLocation(Plasma::Location loc)
@@ -636,6 +615,25 @@ void SearchLaunch::overlayRequestedDrop(QGraphicsSceneDragDropEvent *event)
     dropEvent(event);
 }
 
+
+void SearchLaunch::resultsViewRequestedDrag(QModelIndex index)
+{
+    if (!m_resultsView->model()) {
+        return;
+    }
+
+
+    QModelIndexList list;
+    list.append(index);
+    QMimeData *mimeData = m_resultsView->model()->mimeData(list);
+
+    QDrag *drag = new QDrag(view());
+    drag->setMimeData(mimeData);
+    drag->setPixmap(index.data(Qt::DecorationRole).value<QIcon>().pixmap(KIconLoader::SizeHuge, KIconLoader::SizeHuge));
+
+    drag->exec(Qt::CopyAction);
+}
+
 void SearchLaunch::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *, const QRect &)
 {
     if (m_stripUninitialized) {
@@ -663,8 +661,8 @@ void SearchLaunch::searchReturnPressed()
 {
     QString query = m_searchField->text();
     //by pressing enter  do a query or
-    if (m_firstItem && query == m_lastQuery && !query.isEmpty()) {
-        m_runnermg->run(m_matches.value(m_firstItem, Plasma::QueryMatch(0)));
+    if (query == m_lastQuery && !query.isEmpty()) {
+        launch(m_resultsView->model()->index(0, 0, QModelIndex()));
         reset();
     } else {
         doSearch(query);
@@ -672,28 +670,6 @@ void SearchLaunch::searchReturnPressed()
     }
 }
 
-void SearchLaunch::goRight()
-{
-    QGraphicsSceneWheelEvent ev(QEvent::GraphicsSceneWheel);
-    ev.setDelta(-120);
-    scene()->sendEvent(m_resultsView, &ev);
-
-    if (m_resultsView->widget()->geometry().right()-2 <= m_resultsView->viewportGeometry().right() && sender() == m_rightArrow) {
-        action("next containment")->trigger();
-    }
-}
-
-void SearchLaunch::goLeft()
-{
-    QGraphicsSceneWheelEvent ev(QEvent::GraphicsSceneWheel);
-    ev.setDelta(120);
-    scene()->sendEvent(m_resultsView, &ev);
-
-    if (m_resultsView->widget()->geometry().left() >= -2 && sender() == m_leftArrow) {
-        action("previous containment")->trigger();
-    }
-}
-
 void SearchLaunch::dataUpdated(const QString &sourceName, const Plasma::DataEngine::Data &data)
 {
     Q_UNUSED(sourceName);
@@ -744,15 +720,30 @@ void SearchLaunch::changeEvent(QEvent *event)
             updateConfigurationMode(true);
         }
     }
+    Plasma::Containment::changeEvent(event);
 }
 
 void SearchLaunch::createConfigurationInterface(KConfigDialog *parent)
 {
-    RunnersConfig *runnersConfig = new RunnersConfig(m_runnermg, parent);
+    RunnersConfig *runnersConfig = new RunnersConfig(m_runnerModel->runnerManager(), parent);
     parent->addPage(runnersConfig, i18nc("Title of the page that lets the user choose the loaded krunner plugins", "Search plugins"), "edit-find");
 
     connect(parent, SIGNAL(applyClicked()), runnersConfig, SLOT(accept()));
     connect(parent, SIGNAL(okClicked()), runnersConfig, SLOT(accept()));
+
+    QListView *enabledEntries = new QListView(parent);
+    enabledEntries->setModel(m_serviceModel->allRootEntriesModel());
+    enabledEntries->setModelColumn(0);
+    parent->addPage(enabledEntries, i18nc("Title of the page that lets the user choose what entries will be allowed in the main menu", "Main menu"), "view-list-icons");
+
+    connect(parent, SIGNAL(applyClicked()), m_serviceModel, SLOT(saveConfig()));
+    connect(parent, SIGNAL(okClicked()), m_serviceModel, SLOT(saveConfig()));
+}
+
+void SearchLaunch::saveFavourites()
+{
+    KConfigGroup cg = config();
+    m_stripWidget->save(cg);
 }
 
 K_EXPORT_PLASMA_APPLET(sal, SearchLaunch)
