@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008 by Dario Freddi <drf@kdemod.ath.cx>                *
+ *   Copyright (C) 2008 by Dario Freddi <drf@kde.org>                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -25,6 +25,7 @@
 #include <solid/device.h>
 #include <solid/deviceinterface.h>
 #include <solid/processor.h>
+#include <solid/battery.h>
 
 #include <QtDBus/QDBusMessage>
 #include <QtDBus/QDBusReply>
@@ -48,10 +49,21 @@ GeneralPage::~GeneralPage()
 
 void GeneralPage::fillUi()
 {
+    int batteryCount = 0;
+
+    foreach(const Solid::Device &device, Solid::Device::listFromType(Solid::DeviceInterface::Battery, QString())) {
+        Solid::Device dev = device;
+        Solid::Battery *b = qobject_cast<Solid::Battery*> (dev.asDeviceInterface(Solid::DeviceInterface::Battery));
+        if(b->type() != Solid::Battery::PrimaryBattery) {
+            continue;
+        }
+        ++batteryCount;
+    }
+
     reloadAvailableProfiles();
 
-    toolBox->setItemIcon(0, KIcon("preferences-other"));
-    toolBox->setItemIcon(1, KIcon("battery"));
+    tabWidget->setTabIcon(0, KIcon("preferences-other"));
+    tabWidget->setTabIcon(1, KIcon("battery"));
 
     issueIcon->setPixmap(KIcon("dialog-warning").pixmap(32, 32));
     issueIcon->setVisible(false);
@@ -80,9 +92,6 @@ void GeneralPage::fillUi()
 
     connect(lockScreenOnResume, SIGNAL(stateChanged(int)), SLOT(emitChanged()));
     connect(enableDPMSBox, SIGNAL(stateChanged(int)), SLOT(emitChanged()));
-    connect(notificationsBox, SIGNAL(stateChanged(int)), SLOT(emitChanged()));
-    connect(warningNotificationsBox, SIGNAL(stateChanged(int)), SLOT(emitChanged()));
-    connect(suspendWait, SIGNAL(stateChanged(int)), SLOT(emitChanged()));
     connect(suspendWaitTime, SIGNAL(valueChanged(int)), SLOT(emitChanged()));
 
     connect(notificationsButton, SIGNAL(clicked()), SLOT(configureNotifications()));
@@ -91,24 +100,31 @@ void GeneralPage::fillUi()
     connect(warningSpin, SIGNAL(valueChanged(int)), SLOT(emitChanged()));
     connect(criticalSpin, SIGNAL(valueChanged(int)), SLOT(emitChanged()));
 
-    connect(suspendWait, SIGNAL(stateChanged(int)), SLOT(enableBoxes()));
-
     connect(BatteryCriticalCombo, SIGNAL(currentIndexChanged(int)), SLOT(emitChanged()));
 
     connect(acProfile, SIGNAL(currentIndexChanged(int)), SLOT(emitChanged()));
     connect(lowProfile, SIGNAL(currentIndexChanged(int)), SLOT(emitChanged()));
     connect(warningProfile, SIGNAL(currentIndexChanged(int)), SLOT(emitChanged()));
     connect(batteryProfile, SIGNAL(currentIndexChanged(int)), SLOT(emitChanged()));
+
+    // Disable stuff, eventually
+    if (batteryCount == 0) {
+        batteryProfile->setEnabled(false);
+        lowProfile->setEnabled(false);
+        warningProfile->setEnabled(false);
+        tabWidget->setTabEnabled(1, false);
+    }
 }
 
 void GeneralPage::load()
 {
     lockScreenOnResume->setChecked(PowerDevilSettings::configLockScreen());
-    notificationsBox->setChecked(PowerDevilSettings::enableNotifications());
     enableDPMSBox->setChecked(PowerDevilSettings::manageDPMS());
-    warningNotificationsBox->setChecked(PowerDevilSettings::enableWarningNotifications());
-    suspendWait->setChecked(PowerDevilSettings::waitBeforeSuspending());
-    suspendWaitTime->setValue(PowerDevilSettings::waitBeforeSuspendingTime());
+    if (PowerDevilSettings::waitBeforeSuspending()) {
+        suspendWaitTime->setValue(PowerDevilSettings::waitBeforeSuspendingTime());
+    } else {
+        suspendWaitTime->setValue(0);
+    }
 
     lowSpin->setValue(PowerDevilSettings::batteryLowLevel());
     warningSpin->setValue(PowerDevilSettings::batteryWarningLevel());
@@ -120,8 +136,6 @@ void GeneralPage::load()
     lowProfile->setCurrentIndex(acProfile->findText(PowerDevilSettings::lowProfile()));
     warningProfile->setCurrentIndex(acProfile->findText(PowerDevilSettings::warningProfile()));
     batteryProfile->setCurrentIndex(acProfile->findText(PowerDevilSettings::batteryProfile()));
-
-    enableBoxes();
 }
 
 void GeneralPage::configureNotifications()
@@ -132,10 +146,8 @@ void GeneralPage::configureNotifications()
 void GeneralPage::save()
 {
     PowerDevilSettings::setConfigLockScreen(lockScreenOnResume->isChecked());
-    PowerDevilSettings::setEnableNotifications(notificationsBox->isChecked());
     PowerDevilSettings::setManageDPMS(enableDPMSBox->isChecked());
-    PowerDevilSettings::setEnableWarningNotifications(warningNotificationsBox->isChecked());
-    PowerDevilSettings::setWaitBeforeSuspending(suspendWait->isChecked());
+    PowerDevilSettings::setWaitBeforeSuspending(suspendWaitTime->value() != 0);
     PowerDevilSettings::setWaitBeforeSuspendingTime(suspendWaitTime->value());
 
     PowerDevilSettings::setBatteryLowLevel(lowSpin->value());
@@ -157,11 +169,6 @@ void GeneralPage::emitChanged()
     emit changed(true);
 }
 
-void GeneralPage::enableBoxes()
-{
-    suspendWaitTime->setEnabled(suspendWait->isChecked());
-}
-
 void GeneralPage::enableIssue(bool enable)
 {
     issueIcon->setVisible(enable);
